cmd_fs/yaffs2/yaffs.o := arm-none-linux-gnueabi-ld -EL    -r -o fs/yaffs2/yaffs.o fs/yaffs2/yaffs_ecc.o fs/yaffs2/yaffs_fs.o fs/yaffs2/yaffs_guts.o fs/yaffs2/yaffs_checkptrw.o fs/yaffs2/yaffs_packedtags1.o fs/yaffs2/yaffs_packedtags2.o fs/yaffs2/yaffs_nand.o fs/yaffs2/yaffs_qsort.o fs/yaffs2/yaffs_tagscompat.o fs/yaffs2/yaffs_tagsvalidity.o fs/yaffs2/yaffs_mtdif.o fs/yaffs2/yaffs_mtdif1.o fs/yaffs2/yaffs_mtdif2.o 
