cmd_arch/arm/mach-wmt/built-in.o :=  arm-none-linux-gnueabi-ld -EL    -r -o arch/arm/mach-wmt/built-in.o arch/arm/mach-wmt/generic.o arch/arm/mach-wmt/irq.o arch/arm/mach-wmt/board.o arch/arm/mach-wmt/wmt_clk.o arch/arm/mach-wmt/dma.o arch/arm/mach-wmt/perf.o arch/arm/mach-wmt/pm.o arch/arm/mach-wmt/pm_cpai.o 
