#include <linux/module.h>
#include <linux/vermagic.h>
#include <linux/compiler.h>

MODULE_INFO(vermagic, VERMAGIC_STRING);

struct module __this_module
__attribute__((section(".gnu.linkonce.this_module"))) = {
 .name = KBUILD_MODNAME,
 .init = init_module,
#ifdef CONFIG_MODULE_UNLOAD
 .exit = cleanup_module,
#endif
 .arch = MODULE_ARCH_INIT,
};

static const struct modversion_info ____versions[]
__used
__attribute__((section("__versions"))) = {
	{ 0x2fd55fb3, "module_layout" },
	{ 0xf863e82a, "flush_work" },
	{ 0x12da5bb2, "__kmalloc" },
	{ 0x405c1144, "get_seconds" },
	{ 0xf88dacea, "rfkill_set_sw_state" },
	{ 0xcff53400, "kref_put" },
	{ 0x7c0b71f4, "register_pernet_device" },
	{ 0x7ecc09f2, "debugfs_create_dir" },
	{ 0x97255bdf, "strlen" },
	{ 0x49439411, "genl_unregister_family" },
	{ 0xc8b57c27, "autoremove_wake_function" },
	{ 0xc6aa1a20, "platform_device_register_simple" },
	{ 0x8728140, "malloc_sizes" },
	{ 0xc7a4fbed, "rtnl_lock" },
	{ 0xdcf22d58, "dev_change_net_namespace" },
	{ 0xc633495b, "schedule_work" },
	{ 0x63ecad53, "register_netdevice_notifier" },
	{ 0x23869dc7, "cancel_work_sync" },
	{ 0xe2fae716, "kmemdup" },
	{ 0x323222ba, "mutex_unlock" },
	{ 0x887aada6, "rfkill_register" },
	{ 0xd304be05, "debugfs_rename" },
	{ 0x2f66331e, "get_net_ns_by_pid" },
	{ 0x3c2c5af5, "sprintf" },
	{ 0x7d11c268, "jiffies" },
	{ 0xfe769456, "unregister_netdevice_notifier" },
	{ 0x22d2de57, "skb_trim" },
	{ 0xe2d5255a, "strcmp" },
	{ 0xfb58121b, "__init_waitqueue_head" },
	{ 0x41344088, "param_get_charp" },
	{ 0xfa2a45e, "__memzero" },
	{ 0xa0df7cfe, "device_del" },
	{ 0x37befc70, "jiffies_to_msecs" },
	{ 0x43e536b1, "rfkill_alloc" },
	{ 0x6c36a5c1, "__mutex_init" },
	{ 0xea147363, "printk" },
	{ 0x42224298, "sscanf" },
	{ 0x71c90087, "memcmp" },
	{ 0x72d09509, "unregister_pernet_device" },
	{ 0xaee06fbb, "class_unregister" },
	{ 0x83bee091, "rfkill_set_hw_state" },
	{ 0xc0580937, "rb_erase" },
	{ 0xf0f6570c, "nla_put" },
	{ 0x1c453f6c, "wireless_send_event" },
	{ 0x521bc3a1, "debugfs_remove" },
	{ 0x16305289, "warn_slowpath_null" },
	{ 0x79c4d530, "skb_push" },
	{ 0xb97d4c9c, "mutex_lock" },
	{ 0x67ffa78c, "dev_close" },
	{ 0xcb539642, "dev_get_by_index" },
	{ 0x394c39f3, "netlink_unicast" },
	{ 0x6622278a, "platform_device_unregister" },
	{ 0xbbc4917, "rfkill_pause_polling" },
	{ 0x903ec010, "genl_register_family_with_ops" },
	{ 0xa66b3c00, "sysfs_remove_link" },
	{ 0x7b08e9fc, "device_add" },
	{ 0xf143536a, "skb_pull" },
	{ 0xcbb607b6, "init_net" },
	{ 0x5855e8d5, "__class_register" },
	{ 0xce436115, "__dev_get_by_index" },
	{ 0x2196324, "__aeabi_idiv" },
	{ 0x9b6eb137, "ksize" },
	{ 0x5523d8c1, "kobject_uevent_env" },
	{ 0xe8d5d28a, "rfkill_resume_polling" },
	{ 0xd64878b7, "sysfs_create_link" },
	{ 0x3ff62317, "local_bh_disable" },
	{ 0xdc74cc24, "kmem_cache_alloc" },
	{ 0xf709bd69, "__alloc_skb" },
	{ 0xdf8f6057, "netlink_broadcast" },
	{ 0x67d0fa6, "put_device" },
	{ 0xa58b6804, "nla_parse" },
	{ 0x3bd1b1f6, "msecs_to_jiffies" },
	{ 0x1000e51, "schedule" },
	{ 0x562b754d, "kfree_skb" },
	{ 0x6b2dc060, "dump_stack" },
	{ 0x799aca4, "local_bh_enable" },
	{ 0xc27487dd, "__bug" },
	{ 0x67731558, "pskb_expand_head" },
	{ 0xffb249a0, "device_rename" },
	{ 0xa6dcc773, "rb_insert_color" },
	{ 0x6ad065f4, "param_set_charp" },
	{ 0xb6c70a7d, "__wake_up" },
	{ 0x627e279f, "genlmsg_multicast_allns" },
	{ 0x8d66a3a, "warn_slowpath_fmt" },
	{ 0x3ae831b6, "kref_init" },
	{ 0x37a0cba, "kfree" },
	{ 0xda22f64, "rfkill_destroy" },
	{ 0x9d669763, "memcpy" },
	{ 0x801678, "flush_scheduled_work" },
	{ 0x8085c7b1, "prepare_to_wait" },
	{ 0x8a1203a9, "kref_get" },
	{ 0x528e11b1, "device_initialize" },
	{ 0x20385c58, "genl_register_mc_group" },
	{ 0x2ef4a8c8, "rfkill_blocked" },
	{ 0x51493d94, "finish_wait" },
	{ 0x65899925, "dev_set_name" },
	{ 0x85670f1d, "rtnl_is_locked" },
	{ 0xa69aa4ec, "skb_put" },
	{ 0x68f8d1ac, "rfkill_unregister" },
	{ 0x6e720ff2, "rtnl_unlock" },
	{ 0xc2d711e1, "krealloc" },
};

static const char __module_depends[]
__used
__attribute__((section(".modinfo"))) =
"depends=";

