cmd_arch/arm/mm/built-in.o :=  arm-none-linux-gnueabi-ld -EL    -r -o arch/arm/mm/built-in.o arch/arm/mm/dma-mapping.o arch/arm/mm/extable.o arch/arm/mm/fault.o arch/arm/mm/init.o arch/arm/mm/iomap.o arch/arm/mm/fault-armv.o arch/arm/mm/flush.o arch/arm/mm/ioremap.o arch/arm/mm/mmap.o arch/arm/mm/pgd.o arch/arm/mm/mmu.o arch/arm/mm/proc-syms.o arch/arm/mm/alignment.o arch/arm/mm/abort-ev6.o arch/arm/mm/pabort-v6.o arch/arm/mm/cache-v6.o arch/arm/mm/copypage-v6.o arch/arm/mm/context.o arch/arm/mm/tlb-v6.o arch/arm/mm/proc-v6.o arch/arm/mm/cache-l2x0.o 
