cmd_net/mac80211/mac80211.ko := arm-none-linux-gnueabi-ld -EL -r  -T /home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/scripts/module-common.lds --build-id -o net/mac80211/mac80211.ko net/mac80211/mac80211.o net/mac80211/mac80211.mod.o
