cmd_fs/notify/built-in.o :=  arm-none-linux-gnueabi-ld -EL    -r -o fs/notify/built-in.o fs/notify/fsnotify.o fs/notify/notification.o fs/notify/group.o fs/notify/inode_mark.o fs/notify/dnotify/built-in.o fs/notify/inotify/built-in.o 
