cmd_fs/jffs2/jffs2.o := arm-none-linux-gnueabi-ld -EL    -r -o fs/jffs2/jffs2.o fs/jffs2/compr.o fs/jffs2/dir.o fs/jffs2/file.o fs/jffs2/ioctl.o fs/jffs2/nodelist.o fs/jffs2/malloc.o fs/jffs2/read.o fs/jffs2/nodemgmt.o fs/jffs2/readinode.o fs/jffs2/write.o fs/jffs2/scan.o fs/jffs2/gc.o fs/jffs2/symlink.o fs/jffs2/build.o fs/jffs2/erase.o fs/jffs2/background.o fs/jffs2/fs.o fs/jffs2/writev.o fs/jffs2/super.o fs/jffs2/debug.o fs/jffs2/wbuf.o fs/jffs2/compr_rtime.o fs/jffs2/compr_zlib.o 
