cmd_arch/arm/boot/compressed/vmlinux := arm-none-linux-gnueabi-ld -EL    --defsym zreladdr=0x00008000  --defsym initrd_phys=0x01000000 --defsym params_phys=0x00000100 -p --no-undefined -X /opt/codesourcery/arm/arm_201009_gcc4.5.1/bin/../lib/gcc/arm-none-linux-gnueabi/4.5.1/libgcc.a -T arch/arm/boot/compressed/vmlinux.lds arch/arm/boot/compressed/head.o arch/arm/boot/compressed/piggy.o arch/arm/boot/compressed/misc.o arch/arm/boot/compressed/head-wmt.o -o arch/arm/boot/compressed/vmlinux 
