cmd_net/mac80211/michael.o := arm-none-linux-gnueabi-gcc -Wp,-MD,net/mac80211/.michael.o.d  -nostdinc -isystem /opt/codesourcery/arm/arm_201009_gcc4.5.1/bin/../lib/gcc/arm-none-linux-gnueabi/4.5.1/include -Iinclude  -I/home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/arch/arm/include -include include/linux/autoconf.h -D__KERNEL__ -mlittle-endian -Iarch/arm/mach-wmt/include -Wall -Wundef -Wstrict-prototypes -Wno-trigraphs -fno-strict-aliasing -fno-common -Werror-implicit-function-declaration -Wno-format-security -fno-delete-null-pointer-checks -O2 -marm -fno-omit-frame-pointer -mapcs -mno-sched-prolog -mabi=aapcs-linux -mno-thumb-interwork -D__LINUX_ARM_ARCH__=6 -march=armv6 -mtune=arm1136j-s -msoft-float -Uarm -Wframe-larger-than=1024 -fno-stack-protector -fno-omit-frame-pointer -fno-optimize-sibling-calls -Wdeclaration-after-statement -Wno-pointer-sign -fno-strict-overflow -fno-dwarf2-cfi-asm -fconserve-stack -D__CHECK_ENDIAN__  -DMODULE -D"KBUILD_STR(s)=\#s" -D"KBUILD_BASENAME=KBUILD_STR(michael)"  -D"KBUILD_MODNAME=KBUILD_STR(mac80211)"  -c -o net/mac80211/.tmp_michael.o net/mac80211/michael.c

deps_net/mac80211/michael.o := \
  net/mac80211/michael.c \
  include/linux/types.h \
    $(wildcard include/config/uid16.h) \
    $(wildcard include/config/lbdaf.h) \
    $(wildcard include/config/phys/addr/t/64bit.h) \
    $(wildcard include/config/64bit.h) \
  /home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/arch/arm/include/asm/types.h \
  include/asm-generic/int-ll64.h \
  /home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/arch/arm/include/asm/bitsperlong.h \
  include/asm-generic/bitsperlong.h \
  include/linux/posix_types.h \
  include/linux/stddef.h \
  include/linux/compiler.h \
    $(wildcard include/config/trace/branch/profiling.h) \
    $(wildcard include/config/profile/all/branches.h) \
    $(wildcard include/config/enable/must/check.h) \
    $(wildcard include/config/enable/warn/deprecated.h) \
  include/linux/compiler-gcc.h \
    $(wildcard include/config/arch/supports/optimized/inlining.h) \
    $(wildcard include/config/optimize/inlining.h) \
  include/linux/compiler-gcc4.h \
  /home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/arch/arm/include/asm/posix_types.h \
  include/linux/bitops.h \
    $(wildcard include/config/generic/find/first/bit.h) \
    $(wildcard include/config/generic/find/last/bit.h) \
    $(wildcard include/config/generic/find/next/bit.h) \
  /home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/arch/arm/include/asm/bitops.h \
    $(wildcard include/config/smp.h) \
  /home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/arch/arm/include/asm/system.h \
    $(wildcard include/config/cpu/xsc3.h) \
    $(wildcard include/config/cpu/fa526.h) \
    $(wildcard include/config/cpu/sa1100.h) \
    $(wildcard include/config/cpu/sa110.h) \
    $(wildcard include/config/cpu/32v6k.h) \
  include/linux/linkage.h \
  /home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/arch/arm/include/asm/linkage.h \
  include/linux/irqflags.h \
    $(wildcard include/config/trace/irqflags.h) \
    $(wildcard include/config/irqsoff/tracer.h) \
    $(wildcard include/config/preempt/tracer.h) \
    $(wildcard include/config/trace/irqflags/support.h) \
    $(wildcard include/config/x86.h) \
  include/linux/typecheck.h \
  /home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/arch/arm/include/asm/irqflags.h \
  /home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/arch/arm/include/asm/ptrace.h \
    $(wildcard include/config/cpu/endian/be8.h) \
    $(wildcard include/config/arm/thumb.h) \
  /home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/arch/arm/include/asm/hwcap.h \
  include/asm-generic/cmpxchg-local.h \
  include/asm-generic/bitops/non-atomic.h \
  include/asm-generic/bitops/fls64.h \
  include/asm-generic/bitops/sched.h \
  include/asm-generic/bitops/hweight.h \
  include/asm-generic/bitops/lock.h \
  include/linux/ieee80211.h \
    $(wildcard include/config/len.h) \
  /home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/arch/arm/include/asm/byteorder.h \
  include/linux/byteorder/little_endian.h \
  include/linux/swab.h \
  /home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/arch/arm/include/asm/swab.h \
  include/linux/byteorder/generic.h \
  /home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/arch/arm/include/asm/unaligned.h \
  include/linux/unaligned/le_byteshift.h \
  include/linux/unaligned/be_byteshift.h \
  include/linux/unaligned/generic.h \
  net/mac80211/michael.h \

net/mac80211/michael.o: $(deps_net/mac80211/michael.o)

$(deps_net/mac80211/michael.o):
