extern struct platform_device mxc_uart_device0;
extern struct platform_device mxc_uart_device1;
extern struct platform_device mxc_uart_device2;

extern struct platform_device mxc_nand_device;

extern struct platform_device mxc_sdhc_device0;
extern struct platform_device mxc_sdhc_device1;

extern struct platform_device mxc_cspi_device0;
extern struct platform_device mxc_cspi_device1;

extern struct platform_device mxc_wdog_device0;
