cmd_arch/arm/boot/compressed/misc.o := arm-none-linux-gnueabi-gcc -Wp,-MD,arch/arm/boot/compressed/.misc.o.d  -nostdinc -isystem /opt/codesourcery/arm/arm_201009_gcc4.5.1/bin/../lib/gcc/arm-none-linux-gnueabi/4.5.1/include -Iinclude  -I/home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/arch/arm/include -include include/linux/autoconf.h -D__KERNEL__ -mlittle-endian -Iarch/arm/mach-wmt/include -Wall -Wundef -Wstrict-prototypes -Wno-trigraphs -fno-strict-aliasing -fno-common -Werror-implicit-function-declaration -Wno-format-security -fno-delete-null-pointer-checks -O2 -marm -fno-omit-frame-pointer -mapcs -mno-sched-prolog -mabi=aapcs-linux -mno-thumb-interwork -D__LINUX_ARM_ARCH__=6 -march=armv6 -mtune=arm1136j-s -msoft-float -Uarm -Wframe-larger-than=1024 -fno-stack-protector -fno-omit-frame-pointer -fno-optimize-sibling-calls -Wdeclaration-after-statement -Wno-pointer-sign -fno-strict-overflow -fno-dwarf2-cfi-asm -fconserve-stack -fpic -fno-builtin -Dstatic=   -D"KBUILD_STR(s)=\#s" -D"KBUILD_BASENAME=KBUILD_STR(misc)"  -D"KBUILD_MODNAME=KBUILD_STR(misc)"  -c -o arch/arm/boot/compressed/.tmp_misc.o arch/arm/boot/compressed/misc.c

deps_arch/arm/boot/compressed/misc.o := \
  arch/arm/boot/compressed/misc.c \
    $(wildcard include/config/debug/icedcc.h) \
    $(wildcard include/config/cpu/v6.h) \
    $(wildcard include/config/cpu/v7.h) \
    $(wildcard include/config/cpu/xscale.h) \
  include/linux/compiler.h \
    $(wildcard include/config/trace/branch/profiling.h) \
    $(wildcard include/config/profile/all/branches.h) \
    $(wildcard include/config/enable/must/check.h) \
    $(wildcard include/config/enable/warn/deprecated.h) \
  include/linux/compiler-gcc.h \
    $(wildcard include/config/arch/supports/optimized/inlining.h) \
    $(wildcard include/config/optimize/inlining.h) \
  include/linux/compiler-gcc4.h \
  include/linux/types.h \
    $(wildcard include/config/uid16.h) \
    $(wildcard include/config/lbdaf.h) \
    $(wildcard include/config/phys/addr/t/64bit.h) \
    $(wildcard include/config/64bit.h) \
  /home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/arch/arm/include/asm/types.h \
  include/asm-generic/int-ll64.h \
  /home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/arch/arm/include/asm/bitsperlong.h \
  include/asm-generic/bitsperlong.h \
  include/linux/posix_types.h \
  include/linux/stddef.h \
  /home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/arch/arm/include/asm/posix_types.h \
  /home/willy/Developing/Processing/WM8710/Android2.3.1/ANDROID_2.6.32/arch/arm/include/asm/string.h \
  arch/arm/mach-wmt/include/mach/uncompress.h \
  arch/arm/mach-wmt/include/mach/hardware.h \
  arch/arm/mach-wmt/include/mach/wmt.h \
  arch/arm/mach-wmt/include/mach/wmt_mmap.h \
  arch/arm/mach-wmt/include/mach/common_def.h \
  arch/arm/mach-wmt/include/mach/wmt_mc.h \
    $(wildcard include/config/reg.h) \
    $(wildcard include/config/val.h) \
  arch/arm/mach-wmt/include/mach/wmt_dma.h \
  arch/arm/mach-wmt/include/mach/wmt_scc.h \
  arch/arm/mach-wmt/include/mach/wmt_sdmmc.h \
  arch/arm/mach-wmt/include/mach/wmt_uart.h \
  arch/arm/mach-wmt/include/mach/wmt_rtc.h \
  arch/arm/mach-wmt/include/mach/wmt_gpio.h \
  arch/arm/mach-wmt/include/mach/wmt_pmc.h \
  arch/arm/mach-wmt/include/mach/wmt_intc.h \
  arch/arm/mach-wmt/include/mach/wmt_sf.h \
  arch/arm/mach-wmt/include/mach/wmt_ac97.h \
  arch/arm/mach-wmt/include/mach/wmt_i2s.h \
  arch/arm/mach-wmt/include/mach/wmt_pcm.h \
  arch/arm/mach-wmt/include/mach/wmt_evb.h \
  arch/arm/mach-wmt/include/mach/wmt_i2c.h \
    $(wildcard include/config/snd/soc/vt1603.h) \
  arch/arm/mach-wmt/include/mach/wmt_kpad.h \
  arch/arm/mach-wmt/include/mach/wmt_i8042.h \
  arch/arm/mach-wmt/include/mach/../../wmt_clk.h \
  arch/arm/boot/compressed/../../../../lib/inflate.c \

arch/arm/boot/compressed/misc.o: $(deps_arch/arm/boot/compressed/misc.o)

$(deps_arch/arm/boot/compressed/misc.o):
