/*
 * This was automagically generated from arch/arm/tools/mach-types!
 * Do NOT edit
 */

#ifndef __ASM_ARM_MACH_TYPE_H
#define __ASM_ARM_MACH_TYPE_H

#include <linux/config.h>

#ifndef __ASSEMBLY__
/* The type of machine we're running on */
extern unsigned int __machine_arch_type;
#endif

/* see arch/arm/kernel/arch.c for a description of these */
#define MACH_TYPE_EBSA110              0
#define MACH_TYPE_RISCPC               1
#define MACH_TYPE_NEXUSPCI             3
#define MACH_TYPE_EBSA285              4
#define MACH_TYPE_NETWINDER            5
#define MACH_TYPE_CATS                 6
#define MACH_TYPE_TBOX                 7
#define MACH_TYPE_CO285                8
#define MACH_TYPE_CLPS7110             9
#define MACH_TYPE_ARCHIMEDES           10
#define MACH_TYPE_A5K                  11
#define MACH_TYPE_ETOILE               12
#define MACH_TYPE_LACIE_NAS            13
#define MACH_TYPE_CLPS7500             14
#define MACH_TYPE_SHARK                15
#define MACH_TYPE_BRUTUS               16
#define MACH_TYPE_PERSONAL_SERVER      17
#define MACH_TYPE_ITSY                 18
#define MACH_TYPE_L7200                19
#define MACH_TYPE_PLEB                 20
#define MACH_TYPE_INTEGRATOR           21
#define MACH_TYPE_H3600                22
#define MACH_TYPE_IXP1200              23
#define MACH_TYPE_P720T                24
#define MACH_TYPE_ASSABET              25
#define MACH_TYPE_VICTOR               26
#define MACH_TYPE_LART                 27
#define MACH_TYPE_RANGER               28
#define MACH_TYPE_GRAPHICSCLIENT       29
#define MACH_TYPE_XP860                30
#define MACH_TYPE_CERF                 31
#define MACH_TYPE_NANOENGINE           32
#define MACH_TYPE_FPIC                 33
#define MACH_TYPE_EXTENEX1             34
#define MACH_TYPE_SHERMAN              35
#define MACH_TYPE_ACCELENT_SA          36
#define MACH_TYPE_ACCELENT_L7200       37
#define MACH_TYPE_NETPORT              38
#define MACH_TYPE_PANGOLIN             39
#define MACH_TYPE_YOPY                 40
#define MACH_TYPE_COOLIDGE             41
#define MACH_TYPE_HUW_WEBPANEL         42
#define MACH_TYPE_SPOTME               43
#define MACH_TYPE_FREEBIRD             44
#define MACH_TYPE_TI925                45
#define MACH_TYPE_RISCSTATION          46
#define MACH_TYPE_CAVY                 47
#define MACH_TYPE_JORNADA720           48
#define MACH_TYPE_OMNIMETER            49
#define MACH_TYPE_EDB7211              50
#define MACH_TYPE_CITYGO               51
#define MACH_TYPE_PFS168               52
#define MACH_TYPE_SPOT                 53
#define MACH_TYPE_FLEXANET             54
#define MACH_TYPE_WEBPAL               55
#define MACH_TYPE_LINPDA               56
#define MACH_TYPE_ANAKIN               57
#define MACH_TYPE_MVI                  58
#define MACH_TYPE_JUPITER              59
#define MACH_TYPE_PSIONW               60
#define MACH_TYPE_ALN                  61
#define MACH_TYPE_CAMELOT              62
#define MACH_TYPE_GDS2200              63
#define MACH_TYPE_PSION_SERIES7        64
#define MACH_TYPE_XFILE                65
#define MACH_TYPE_ACCELENT_EP9312      66
#define MACH_TYPE_IC200                67
#define MACH_TYPE_CREDITLART           68
#define MACH_TYPE_HTM                  69
#define MACH_TYPE_IQ80310              70
#define MACH_TYPE_FREEBOT              71
#define MACH_TYPE_ENTEL                72
#define MACH_TYPE_ENP3510              73
#define MACH_TYPE_TRIZEPS              74
#define MACH_TYPE_NESA                 75
#define MACH_TYPE_VENUS                76
#define MACH_TYPE_TARDIS               77
#define MACH_TYPE_MERCURY              78
#define MACH_TYPE_EMPEG                79
#define MACH_TYPE_I80200FCC            80
#define MACH_TYPE_ITT_CPB              81
#define MACH_TYPE_SVC                  82
#define MACH_TYPE_ALPHA2               84
#define MACH_TYPE_ALPHA1               85
#define MACH_TYPE_NETARM               86
#define MACH_TYPE_SIMPAD               87
#define MACH_TYPE_PDA1                 88
#define MACH_TYPE_LUBBOCK              89
#define MACH_TYPE_ANIKO                90
#define MACH_TYPE_CLEP7212             91
#define MACH_TYPE_CS89712              92
#define MACH_TYPE_WEARARM              93
#define MACH_TYPE_POSSIO_PX            94
#define MACH_TYPE_SIDEARM              95
#define MACH_TYPE_STORK                96
#define MACH_TYPE_SHANNON              97
#define MACH_TYPE_ACE                  98
#define MACH_TYPE_BALLYARM             99
#define MACH_TYPE_SIMPUTER             100
#define MACH_TYPE_NEXTERM              101
#define MACH_TYPE_SA1100_ELF           102
#define MACH_TYPE_GATOR                103
#define MACH_TYPE_GRANITE              104
#define MACH_TYPE_CONSUS               105
#define MACH_TYPE_AAED2000             106
#define MACH_TYPE_CDB89712             107
#define MACH_TYPE_GRAPHICSMASTER       108
#define MACH_TYPE_ADSBITSY             109
#define MACH_TYPE_PXA_IDP              110
#define MACH_TYPE_PLCE                 111
#define MACH_TYPE_PT_SYSTEM3           112
#define MACH_TYPE_MEDALB               113
#define MACH_TYPE_EAGLE                114
#define MACH_TYPE_DSC21                115
#define MACH_TYPE_DSC24                116
#define MACH_TYPE_TI5472               117
#define MACH_TYPE_AUTCPU12             118
#define MACH_TYPE_UENGINE              119
#define MACH_TYPE_BLUESTEM             120
#define MACH_TYPE_XINGU8               121
#define MACH_TYPE_BUSHSTB              122
#define MACH_TYPE_EPSILON1             123
#define MACH_TYPE_BALLOON              124
#define MACH_TYPE_PUPPY                125
#define MACH_TYPE_ELROY                126
#define MACH_TYPE_GMS720               127
#define MACH_TYPE_S24X                 128
#define MACH_TYPE_JTEL_CLEP7312        129
#define MACH_TYPE_CX821XX              130
#define MACH_TYPE_EDB7312              131
#define MACH_TYPE_BSA1110              132
#define MACH_TYPE_POWERPIN             133
#define MACH_TYPE_OPENARM              134
#define MACH_TYPE_WHITECHAPEL          135
#define MACH_TYPE_H3100                136
#define MACH_TYPE_H3800                137
#define MACH_TYPE_BLUE_V1              138
#define MACH_TYPE_PXA_CERF             139
#define MACH_TYPE_ARM7TEVB             140
#define MACH_TYPE_D7400                141
#define MACH_TYPE_PIRANHA              142
#define MACH_TYPE_SBCAMELOT            143
#define MACH_TYPE_KINGS                144
#define MACH_TYPE_SMDK2400             145
#define MACH_TYPE_COLLIE               146
#define MACH_TYPE_IDR                  147
#define MACH_TYPE_BADGE4               148
#define MACH_TYPE_WEBNET               149
#define MACH_TYPE_D7300                150
#define MACH_TYPE_CEP                  151
#define MACH_TYPE_FORTUNET             152
#define MACH_TYPE_VC547X               153
#define MACH_TYPE_FILEWALKER           154
#define MACH_TYPE_NETGATEWAY           155
#define MACH_TYPE_SYMBOL2800           156
#define MACH_TYPE_SUNS                 157
#define MACH_TYPE_FRODO                158
#define MACH_TYPE_MACH_TYTE_MS301      159
#define MACH_TYPE_MX1ADS               160
#define MACH_TYPE_H7201                161
#define MACH_TYPE_H7202                162
#define MACH_TYPE_AMICO                163
#define MACH_TYPE_IAM                  164
#define MACH_TYPE_TT530                165
#define MACH_TYPE_SAM2400              166
#define MACH_TYPE_JORNADA56X           167
#define MACH_TYPE_ACTIVE               168
#define MACH_TYPE_IQ80321              169
#define MACH_TYPE_WID                  170
#define MACH_TYPE_SABINAL              171
#define MACH_TYPE_IXP425_MATACUMBE     172
#define MACH_TYPE_MINIPRINT            173
#define MACH_TYPE_ADM510X              174
#define MACH_TYPE_SVS200               175
#define MACH_TYPE_ATG_TCU              176
#define MACH_TYPE_JORNADA820           177
#define MACH_TYPE_S3C44B0              178
#define MACH_TYPE_MARGIS2              179
#define MACH_TYPE_KS8695               180
#define MACH_TYPE_BRH                  181
#define MACH_TYPE_S3C2410              182
#define MACH_TYPE_POSSIO_PX30          183
#define MACH_TYPE_S3C2800              184
#define MACH_TYPE_FLEETWOOD            185
#define MACH_TYPE_OMAHA                186
#define MACH_TYPE_TA7                  187
#define MACH_TYPE_NOVA                 188
#define MACH_TYPE_HMK                  189
#define MACH_TYPE_KARO                 190
#define MACH_TYPE_FESTER               191
#define MACH_TYPE_GPI                  192
#define MACH_TYPE_SMDK2410             193
#define MACH_TYPE_I519                 194
#define MACH_TYPE_NEXIO                195
#define MACH_TYPE_BITBOX               196
#define MACH_TYPE_G200                 197
#define MACH_TYPE_GILL                 198
#define MACH_TYPE_PXA_MERCURY          199
#define MACH_TYPE_CEIVA                200
#define MACH_TYPE_FRET                 201
#define MACH_TYPE_EMAILPHONE           202
#define MACH_TYPE_H3900                203
#define MACH_TYPE_PXA1                 204
#define MACH_TYPE_KOAN369              205
#define MACH_TYPE_COGENT               206
#define MACH_TYPE_ESL_SIMPUTER         207
#define MACH_TYPE_ESL_SIMPUTER_CLR     208
#define MACH_TYPE_ESL_SIMPUTER_BW      209
#define MACH_TYPE_HHP_CRADLE           210
#define MACH_TYPE_HE500                211
#define MACH_TYPE_INHANDELF2           212
#define MACH_TYPE_INHANDFTIP           213
#define MACH_TYPE_DNP1110              214
#define MACH_TYPE_PNP1110              215
#define MACH_TYPE_CSB226               216
#define MACH_TYPE_ARNOLD               217
#define MACH_TYPE_VOICEBLUE            218
#define MACH_TYPE_JZ8028               219
#define MACH_TYPE_H5400                220
#define MACH_TYPE_FORTE                221
#define MACH_TYPE_ACAM                 222
#define MACH_TYPE_ABOX                 223
#define MACH_TYPE_ATMEL                224
#define MACH_TYPE_SITSANG              225
#define MACH_TYPE_CPU1110LCDNET        226
#define MACH_TYPE_MPL_VCMA9            227
#define MACH_TYPE_OPUS_A1              228
#define MACH_TYPE_DAYTONA              229
#define MACH_TYPE_KILLBEAR             230
#define MACH_TYPE_YOHO                 231
#define MACH_TYPE_JASPER               232
#define MACH_TYPE_DSC25                233
#define MACH_TYPE_OMAP_INNOVATOR       234
#define MACH_TYPE_RAMSES               235
#define MACH_TYPE_S28X                 236
#define MACH_TYPE_MPORT3               237
#define MACH_TYPE_PXA_EAGLE250         238
#define MACH_TYPE_PDB                  239
#define MACH_TYPE_BLUE_2G              240
#define MACH_TYPE_BLUEARCH             241
#define MACH_TYPE_IXDP2400             242
#define MACH_TYPE_IXDP2800             243
#define MACH_TYPE_EXPLORER             244
#define MACH_TYPE_IXDP425              245
#define MACH_TYPE_CHIMP                246
#define MACH_TYPE_STORK_NEST           247
#define MACH_TYPE_STORK_EGG            248
#define MACH_TYPE_WISMO                249
#define MACH_TYPE_EZLINX               250
#define MACH_TYPE_AT91RM9200           251
#define MACH_TYPE_ORION                252
#define MACH_TYPE_NEPTUNE              253
#define MACH_TYPE_HACKKIT              254
#define MACH_TYPE_PXA_WINS30           255
#define MACH_TYPE_LAVINNA              256
#define MACH_TYPE_PXA_UENGINE          257
#define MACH_TYPE_INNOKOM              258
#define MACH_TYPE_BMS                  259
#define MACH_TYPE_IXCDP1100            260
#define MACH_TYPE_PRPMC1100            261
#define MACH_TYPE_AT91RM9200DK         262
#define MACH_TYPE_ARMSTICK             263
#define MACH_TYPE_ARMONIE              264
#define MACH_TYPE_MPORT1               265
#define MACH_TYPE_S3C5410              266
#define MACH_TYPE_ZCP320A              267
#define MACH_TYPE_I_BOX                268
#define MACH_TYPE_STLC1502             269
#define MACH_TYPE_SIREN                270
#define MACH_TYPE_GREENLAKE            271
#define MACH_TYPE_ARGUS                272
#define MACH_TYPE_COMBADGE             273
#define MACH_TYPE_ROKEPXA              274
#define MACH_TYPE_CINTEGRATOR          275
#define MACH_TYPE_GUIDEA07             276
#define MACH_TYPE_TAT257               277
#define MACH_TYPE_IGP2425              278
#define MACH_TYPE_BLUEGRAMMA           279
#define MACH_TYPE_IPOD                 280
#define MACH_TYPE_ADSBITSYX            281
#define MACH_TYPE_TRIZEPS2             282
#define MACH_TYPE_VIPER                283
#define MACH_TYPE_ADSBITSYPLUS         284
#define MACH_TYPE_ADSAGC               285
#define MACH_TYPE_STP7312              286
#define MACH_TYPE_NX_PHNX              287
#define MACH_TYPE_WEP_EP250            288
#define MACH_TYPE_INHANDELF3           289
#define MACH_TYPE_ADI_COYOTE           290
#define MACH_TYPE_IYONIX               291
#define MACH_TYPE_DAMICAM_SA1110       292
#define MACH_TYPE_MEG03                293
#define MACH_TYPE_PXA_WHITECHAPEL      294
#define MACH_TYPE_NWSC                 295
#define MACH_TYPE_NWLARM               296
#define MACH_TYPE_IXP425_MGUARD        297
#define MACH_TYPE_PXA_NETDCU4          298
#define MACH_TYPE_IXDP2401             299
#define MACH_TYPE_IXDP2801             300
#define MACH_TYPE_ZODIAC               301
#define MACH_TYPE_ARMMODUL             302
#define MACH_TYPE_KETOP                303
#define MACH_TYPE_AV7200               304
#define MACH_TYPE_ARCH_TI925           305
#define MACH_TYPE_ACQ200               306
#define MACH_TYPE_PT_DAFIT             307
#define MACH_TYPE_IHBA                 308
#define MACH_TYPE_QUINQUE              309
#define MACH_TYPE_NIMBRAONE            310
#define MACH_TYPE_NIMBRA29X            311
#define MACH_TYPE_NIMBRA210            312
#define MACH_TYPE_HHP_D95XX            313
#define MACH_TYPE_LABARM               314
#define MACH_TYPE_M825XX               315
#define MACH_TYPE_M7100                316
#define MACH_TYPE_NIPC2                317
#define MACH_TYPE_FU7202               318
#define MACH_TYPE_ADSAGX               319
#define MACH_TYPE_PXA_POOH             320
#define MACH_TYPE_BANDON               321
#define MACH_TYPE_PCM7210              322
#define MACH_TYPE_NMS9200              323
#define MACH_TYPE_LOGODL               324
#define MACH_TYPE_M7140                325
#define MACH_TYPE_KOREBOT              326
#define MACH_TYPE_IQ31244              327
#define MACH_TYPE_KOAN393              328
#define MACH_TYPE_INHANDFTIP3          329
#define MACH_TYPE_GONZO                330
#define MACH_TYPE_BAST                 331
#define MACH_TYPE_SCANPASS             332
#define MACH_TYPE_EP7312_POOH          333
#define MACH_TYPE_TA7S                 334
#define MACH_TYPE_TA7V                 335
#define MACH_TYPE_ICARUS               336
#define MACH_TYPE_H1900                337
#define MACH_TYPE_GEMINI               338
#define MACH_TYPE_AXIM                 339
#define MACH_TYPE_AUDIOTRON            340
#define MACH_TYPE_H2200                341
#define MACH_TYPE_LOOX600              342
#define MACH_TYPE_NIOP                 343
#define MACH_TYPE_DM310                344
#define MACH_TYPE_SEEDPXA_C2           345
#define MACH_TYPE_IXP4XX_MGUARD_PCI    346
#define MACH_TYPE_H1940                347
#define MACH_TYPE_SCORPIO              348
#define MACH_TYPE_VIVA                 349
#define MACH_TYPE_PXA_XCARD            350
#define MACH_TYPE_CSB335               351
#define MACH_TYPE_IXRD425              352
#define MACH_TYPE_IQ80315              353
#define MACH_TYPE_NMP7312              354
#define MACH_TYPE_CX861XX              355
#define MACH_TYPE_ENP2611              356
#define MACH_TYPE_XDA                  357
#define MACH_TYPE_CSIR_IMS             358
#define MACH_TYPE_IXP421_DNAEETH       359
#define MACH_TYPE_POCKETSERV9200       360
#define MACH_TYPE_TOTO                 361
#define MACH_TYPE_S3C2440              362
#define MACH_TYPE_KS8695P              363
#define MACH_TYPE_SE4000               364
#define MACH_TYPE_QUADRICEPS           365
#define MACH_TYPE_BRONCO               366
#define MACH_TYPE_ESL_SOFCOMP          368
#define MACH_TYPE_S5C7375              369
#define MACH_TYPE_SPEARHEAD            370
#define MACH_TYPE_PANTERA              371
#define MACH_TYPE_PRAYOGLITE           372
#define MACH_TYPE_GUMSTIK              373
#define MACH_TYPE_RCUBE                374
#define MACH_TYPE_REA_OLV              375
#define MACH_TYPE_PXA_IPHONE           376
#define MACH_TYPE_S3C3410              377
#define MACH_TYPE_ESPD_4510B           378
#define MACH_TYPE_MP1X                 379
#define MACH_TYPE_AT91RM9200TB         380
#define MACH_TYPE_ADSVGX               381
#define MACH_TYPE_OMAP_H2              382
#define MACH_TYPE_PELEE                383
#define MACH_TYPE_E740                 384
#define MACH_TYPE_IQ80331              385
#define MACH_TYPE_VERSATILE_PB         387
#define MACH_TYPE_KEV7A400             388
#define MACH_TYPE_LPD7A400             389
#define MACH_TYPE_LPD7A404             390
#define MACH_TYPE_FUJITSU_CAMELOT      391
#define MACH_TYPE_JANUS2M              392
#define MACH_TYPE_EMBTF                393
#define MACH_TYPE_HPM                  394
#define MACH_TYPE_SMDK2410TK           395
#define MACH_TYPE_SMDK2410AJ           396
#define MACH_TYPE_STREETRACER          397
#define MACH_TYPE_EFRAME               398
#define MACH_TYPE_CSB337               399
#define MACH_TYPE_PXA_LARK             400
#define MACH_TYPE_PNP2110              401
#define MACH_TYPE_TCC72X               402
#define MACH_TYPE_ALTAIR               403
#define MACH_TYPE_KC3                  404
#define MACH_TYPE_SINTEFTD             405
#define MACH_TYPE_MAINSTONE            406
#define MACH_TYPE_ADAY4X               407
#define MACH_TYPE_LITE300              408
#define MACH_TYPE_S5C7376              409
#define MACH_TYPE_MT02                 410
#define MACH_TYPE_MPORT3S              411
#define MACH_TYPE_RA_ALPHA             412
#define MACH_TYPE_XCEP                 413
#define MACH_TYPE_ARCOM_VULCAN         414
#define MACH_TYPE_STARGATE             415
#define MACH_TYPE_ARMADILLOJ           416
#define MACH_TYPE_ELROY_JACK           417
#define MACH_TYPE_BACKEND              418
#define MACH_TYPE_S5LINBOX             419
#define MACH_TYPE_NOMADIK              420
#define MACH_TYPE_IA_CPU_9200          421
#define MACH_TYPE_AT91_BJA1            422
#define MACH_TYPE_CORGI                423
#define MACH_TYPE_POODLE               424
#define MACH_TYPE_TEN                  425
#define MACH_TYPE_ROVERP5P             426
#define MACH_TYPE_SC2700               427
#define MACH_TYPE_EX_EAGLE             428
#define MACH_TYPE_NX_PXA12             429
#define MACH_TYPE_NX_PXA5              430
#define MACH_TYPE_BLACKBOARD2          431
#define MACH_TYPE_I819                 432
#define MACH_TYPE_IXMB995E             433
#define MACH_TYPE_SKYRIDER             434
#define MACH_TYPE_SKYHAWK              435
#define MACH_TYPE_ENTERPRISE           436
#define MACH_TYPE_DEP2410              437
#define MACH_TYPE_ARMCORE              438
#define MACH_TYPE_HOBBIT               439
#define MACH_TYPE_H7210                440
#define MACH_TYPE_PXA_NETDCU5          441
#define MACH_TYPE_ACC                  442
#define MACH_TYPE_ESL_SARVA            443
#define MACH_TYPE_XM250                444
#define MACH_TYPE_T6TC1XB              445
#define MACH_TYPE_ESS710               446
#define MACH_TYPE_MX31ADS              447
#define MACH_TYPE_HIMALAYA             448
#define MACH_TYPE_BOLFENK              449
#define MACH_TYPE_AT91RM9200KR         450
#define MACH_TYPE_EDB9312              451
#define MACH_TYPE_OMAP_GENERIC         452
#define MACH_TYPE_AXIMX3               453
#define MACH_TYPE_EB67XDIP             454
#define MACH_TYPE_WEBTXS               455
#define MACH_TYPE_HAWK                 456
#define MACH_TYPE_CCAT91SBC001         457
#define MACH_TYPE_EXPRESSO             458
#define MACH_TYPE_H4000                459
#define MACH_TYPE_DINO                 460
#define MACH_TYPE_ML675K               461
#define MACH_TYPE_EDB9301              462
#define MACH_TYPE_EDB9315              463
#define MACH_TYPE_RECIVA_TT            464
#define MACH_TYPE_CSTCB01              465
#define MACH_TYPE_CSTCB1               466
#define MACH_TYPE_SHADWELL             467
#define MACH_TYPE_GOEPEL263            468
#define MACH_TYPE_ACQ100               469
#define MACH_TYPE_MX1FS2               470
#define MACH_TYPE_HIPTOP_G1            471
#define MACH_TYPE_SPARKY               472
#define MACH_TYPE_NS9750               473
#define MACH_TYPE_PHOENIX              474
#define MACH_TYPE_VR1000               475
#define MACH_TYPE_DEISTERPXA           476
#define MACH_TYPE_BCM1160              477
#define MACH_TYPE_PCM022               478
#define MACH_TYPE_ADSGCX               479
#define MACH_TYPE_DREADNAUGHT          480
#define MACH_TYPE_DM320                481
#define MACH_TYPE_MARKOV               482
#define MACH_TYPE_COS7A400             483
#define MACH_TYPE_MILANO               484
#define MACH_TYPE_UE9328               485
#define MACH_TYPE_UEX255               486
#define MACH_TYPE_UE2410               487
#define MACH_TYPE_A620                 488
#define MACH_TYPE_OCELOT               489
#define MACH_TYPE_CHEETAH              490
#define MACH_TYPE_OMAP_PERSEUS2        491
#define MACH_TYPE_ZVUE                 492
#define MACH_TYPE_ROVERP1              493
#define MACH_TYPE_ASIDIAL2             494
#define MACH_TYPE_S3C24A0              495
#define MACH_TYPE_E800                 496
#define MACH_TYPE_E750                 497
#define MACH_TYPE_S3C5500              498
#define MACH_TYPE_SMDK5500             499
#define MACH_TYPE_SIGNALSYNC           500
#define MACH_TYPE_NBC                  501
#define MACH_TYPE_KODIAK               502
#define MACH_TYPE_NETBOOKPRO           503
#define MACH_TYPE_HW90200              504
#define MACH_TYPE_CONDOR               505
#define MACH_TYPE_CUP                  506
#define MACH_TYPE_KITE                 507
#define MACH_TYPE_SCB9328              508
#define MACH_TYPE_OMAP_H3              509
#define MACH_TYPE_OMAP_H4              510
#define MACH_TYPE_N10                  511
#define MACH_TYPE_MONTAJADE            512
#define MACH_TYPE_SG560                513
#define MACH_TYPE_DP1000               514
#define MACH_TYPE_OMAP_OSK             515
#define MACH_TYPE_RG100V3              516
#define MACH_TYPE_MX2ADS               517
#define MACH_TYPE_PXA_KILO             518
#define MACH_TYPE_IXP4XX_EAGLE         519
#define MACH_TYPE_TOSA                 520
#define MACH_TYPE_MB2520F              521
#define MACH_TYPE_EMC1000              522
#define MACH_TYPE_TIDSC25              523
#define MACH_TYPE_AKCPMXL              524
#define MACH_TYPE_AV3XX                525
#define MACH_TYPE_AVILA                526
#define MACH_TYPE_PXA_MPM10            527
#define MACH_TYPE_PXA_KYANITE          528
#define MACH_TYPE_SGOLD                529
#define MACH_TYPE_OSCAR                530
#define MACH_TYPE_EPXA4USB2            531
#define MACH_TYPE_XSENGINE             532
#define MACH_TYPE_IP600                533
#define MACH_TYPE_MCAN2                534
#define MACH_TYPE_DDI_BLUERIDGE        535
#define MACH_TYPE_SKYMINDER            536
#define MACH_TYPE_LPD79520             537
#define MACH_TYPE_EDB9302              538
#define MACH_TYPE_HW90340              539
#define MACH_TYPE_CIP_BOX              540
#define MACH_TYPE_IVPN                 541
#define MACH_TYPE_RSOC2                542
#define MACH_TYPE_HUSKY                543
#define MACH_TYPE_BOXER                544
#define MACH_TYPE_SHEPHERD             545
#define MACH_TYPE_AML42800AA           546
#define MACH_TYPE_MACH_TYPE_ML674001   547
#define MACH_TYPE_LPC2294              548
#define MACH_TYPE_SWITCHGRASS          549
#define MACH_TYPE_ENS_CMU              550
#define MACH_TYPE_MM6_SDB              551
#define MACH_TYPE_SATURN               552
#define MACH_TYPE_I30030EVB            553
#define MACH_TYPE_MXC27530EVB          554
#define MACH_TYPE_SMDK2800             555
#define MACH_TYPE_MTWILSON             556
#define MACH_TYPE_ZITI                 557
#define MACH_TYPE_GRANDFATHER          558
#define MACH_TYPE_TENGINE              559
#define MACH_TYPE_S3C2460              560
#define MACH_TYPE_PDM                  561
#define MACH_TYPE_H4700                562
#define MACH_TYPE_H6300                563
#define MACH_TYPE_RZ1700               564
#define MACH_TYPE_A716                 565
#define MACH_TYPE_ESTK2440A            566
#define MACH_TYPE_ATWIXP425            567
#define MACH_TYPE_CSB336               568
#define MACH_TYPE_RIRM2                569
#define MACH_TYPE_CX23518              570
#define MACH_TYPE_CX2351X              571
#define MACH_TYPE_COMPUTIME            572
#define MACH_TYPE_IZARUS               573
#define MACH_TYPE_RTS                  574
#define MACH_TYPE_SE5100               575
#define MACH_TYPE_S3C2510              576
#define MACH_TYPE_CSB437TL             577
#define MACH_TYPE_SLAUSON              578
#define MACH_TYPE_PEARLRIVER           579
#define MACH_TYPE_TDC_P210             580
#define MACH_TYPE_SG580                581
#define MACH_TYPE_WRSBCARM7            582
#define MACH_TYPE_IPD                  583
#define MACH_TYPE_PXA_DNP2110          584
#define MACH_TYPE_XAENIAX              585
#define MACH_TYPE_SOMN4250             586
#define MACH_TYPE_PLEB2                587
#define MACH_TYPE_CORNWALLIS           588
#define MACH_TYPE_GURNEY_DRV           589
#define MACH_TYPE_CHAFFEE              590
#define MACH_TYPE_RMS101               591
#define MACH_TYPE_RX3715               592
#define MACH_TYPE_SWIFT                593
#define MACH_TYPE_ROVERP7              594
#define MACH_TYPE_PR818S               595
#define MACH_TYPE_TRXPRO               596
#define MACH_TYPE_NSLU2                597
#define MACH_TYPE_E400                 598
#define MACH_TYPE_TRAB                 599
#define MACH_TYPE_CMC_PU2              600
#define MACH_TYPE_FULCRUM              601
#define MACH_TYPE_NETGATE42X           602
#define MACH_TYPE_STR710               603
#define MACH_TYPE_IXDPG425             604
#define MACH_TYPE_TOMTOMGO             605
#define MACH_TYPE_VERSATILE_AB         606
#define MACH_TYPE_EDB9307              607
#define MACH_TYPE_SG565                608
#define MACH_TYPE_LPD79524             609
#define MACH_TYPE_LPD79525             610
#define MACH_TYPE_RMS100               611
#define MACH_TYPE_KB9200               612
#define MACH_TYPE_SX1                  613
#define MACH_TYPE_HMS39C7092           614
#define MACH_TYPE_ARMADILLO            615
#define MACH_TYPE_IPCU                 616
#define MACH_TYPE_LOOX720              617
#define MACH_TYPE_IXDP465              618
#define MACH_TYPE_IXDP2351             619
#define MACH_TYPE_ADSVIX               620
#define MACH_TYPE_DM270                621
#define MACH_TYPE_SOCLTPLUS            622
#define MACH_TYPE_ECIA                 623
#define MACH_TYPE_CM4008               624
#define MACH_TYPE_P2001                625
#define MACH_TYPE_TWISTER              626
#define MACH_TYPE_MUDSHARK             627
#define MACH_TYPE_HB2                  628
#define MACH_TYPE_IQ80332              629
#define MACH_TYPE_SENDT                630
#define MACH_TYPE_MX2JAZZ              631
#define MACH_TYPE_MULTIIO              632
#define MACH_TYPE_HRDISPLAY            633
#define MACH_TYPE_MXC27530ADS          634
#define MACH_TYPE_TRIZEPS3             635
#define MACH_TYPE_ZEFEERDZA            636
#define MACH_TYPE_ZEFEERDZB            637
#define MACH_TYPE_ZEFEERDZG            638
#define MACH_TYPE_ZEFEERDZN            639
#define MACH_TYPE_ZEFEERDZQ            640
#define MACH_TYPE_GTWX5715             641
#define MACH_TYPE_ASTRO_JACK           643
#define MACH_TYPE_TIP03                644
#define MACH_TYPE_A9200EC              645
#define MACH_TYPE_PNX0105              646
#define MACH_TYPE_ADCPOECPU            647
#define MACH_TYPE_CSB637               648
#define MACH_TYPE_ML69Q6203            649
#define MACH_TYPE_MB9200               650
#define MACH_TYPE_KULUN                651
#define MACH_TYPE_SNAPPER              652
#define MACH_TYPE_OPTIMA               653
#define MACH_TYPE_DLHSBC               654
#define MACH_TYPE_X30                  655
#define MACH_TYPE_N30                  656
#define MACH_TYPE_MANGA_KS8695         657
#define MACH_TYPE_AJAX                 658
#define MACH_TYPE_NEC_MP900            659
#define MACH_TYPE_VVTK1000             661
#define MACH_TYPE_KAFA                 662
#define MACH_TYPE_VVTK3000             663
#define MACH_TYPE_PIMX1                664
#define MACH_TYPE_OLLIE                665
#define MACH_TYPE_SKYMAX               666
#define MACH_TYPE_JAZZ                 667
#define MACH_TYPE_TEL_T3               668
#define MACH_TYPE_AISINO_FCR255        669
#define MACH_TYPE_BTWEB                670
#define MACH_TYPE_DBG_LH79520          671
#define MACH_TYPE_CM41XX               672
#define MACH_TYPE_TS72XX               673
#define MACH_TYPE_NGGPXA               674
#define MACH_TYPE_CSB535               675
#define MACH_TYPE_CSB536               676
#define MACH_TYPE_PXA_TRAKPOD          677
#define MACH_TYPE_PRAXIS               678
#define MACH_TYPE_LH75411              679
#define MACH_TYPE_OTOM                 680
#define MACH_TYPE_NEXCODER_2440        681
#define MACH_TYPE_LOOX410              682
#define MACH_TYPE_WESTLAKE             683
#define MACH_TYPE_NSB                  684
#define MACH_TYPE_ESL_SARVA_STN        685
#define MACH_TYPE_ESL_SARVA_TFT        686
#define MACH_TYPE_ESL_SARVA_IAD        687
#define MACH_TYPE_ESL_SARVA_ACC        688
#define MACH_TYPE_TYPHOON              689
#define MACH_TYPE_CNAV                 690
#define MACH_TYPE_A730                 691
#define MACH_TYPE_NETSTAR              692
#define MACH_TYPE_PHASEFALE_SUPERCON   693
#define MACH_TYPE_SHIVA1100            694
#define MACH_TYPE_ETEXSC               695
#define MACH_TYPE_IXDPG465             696
#define MACH_TYPE_A9M2410              697
#define MACH_TYPE_A9M2440              698
#define MACH_TYPE_A9M9750              699
#define MACH_TYPE_A9M9360              700
#define MACH_TYPE_UNC90                701
#define MACH_TYPE_ECO920               702
#define MACH_TYPE_SATVIEW              703
#define MACH_TYPE_ROADRUNNER           704
#define MACH_TYPE_AT91RM9200EK         705
#define MACH_TYPE_GP32                 706
#define MACH_TYPE_GEM                  707
#define MACH_TYPE_I858                 708
#define MACH_TYPE_HX2750               709
#define MACH_TYPE_MXC91131EVB          710
#define MACH_TYPE_P700                 711
#define MACH_TYPE_CPE                  712
#define MACH_TYPE_SPITZ                713
#define MACH_TYPE_NIMBRA340            714
#define MACH_TYPE_LPC22XX              715
#define MACH_TYPE_COMET3               716
#define MACH_TYPE_COMET4               717
#define MACH_TYPE_CSB625               718
#define MACH_TYPE_FORTUNET2            719
#define MACH_TYPE_S5H2200              720
#define MACH_TYPE_OPTORM920            721
#define MACH_TYPE_ADSBITSYXB           722
#define MACH_TYPE_ADSSPHERE            723
#define MACH_TYPE_ADSPORTAL            724
#define MACH_TYPE_LN2410SBC            725
#define MACH_TYPE_CB3RUFC              726
#define MACH_TYPE_MP2USB               727
#define MACH_TYPE_WMT                  728
#define MACH_TYPE_COLIBRI              729
#define MACH_TYPE_PCM7220              730
#define MACH_TYPE_GATEWAY7001          731
#define MACH_TYPE_PCM027               732
#define MACH_TYPE_CMPXA                733
#define MACH_TYPE_ANUBIS               734
#define MACH_TYPE_ITE8152              735
#define MACH_TYPE_LPC3XXX              736
#define MACH_TYPE_PUPPETEER            737
#define MACH_TYPE_MACH_VADATECH        738
#define MACH_TYPE_E570                 739
#define MACH_TYPE_X50                  740
#define MACH_TYPE_RECON                741
#define MACH_TYPE_XBOARDGP8            742
#define MACH_TYPE_FPIC2                743
#define MACH_TYPE_AKITA                744
#define MACH_TYPE_A81                  745
#define MACH_TYPE_SVM_SC25X            746
#define MACH_TYPE_VADATECH020          747
#define MACH_TYPE_TLI                  748
#define MACH_TYPE_EDB9315LC            749
#define MACH_TYPE_PASSEC               750
#define MACH_TYPE_DS_TIGER             751
#define MACH_TYPE_E310                 752
#define MACH_TYPE_E330                 753
#define MACH_TYPE_RT3000               754
#define MACH_TYPE_NOKIA770             755
#define MACH_TYPE_PNX0106              756
#define MACH_TYPE_HX21XX               757
#define MACH_TYPE_FARADAY              758
#define MACH_TYPE_SBC9312              759
#define MACH_TYPE_BATMAN               760
#define MACH_TYPE_JPD201               761
#define MACH_TYPE_MIPSA                762
#define MACH_TYPE_KACOM                763
#define MACH_TYPE_SWARCOCPU            764
#define MACH_TYPE_SWARCODSL            765
#define MACH_TYPE_BLUEANGEL            766
#define MACH_TYPE_HAIRYGRAMA           767
#define MACH_TYPE_BANFF                768
#define MACH_TYPE_CARMEVA              769
#define MACH_TYPE_SAM255               770
#define MACH_TYPE_PPM10                771
#define MACH_TYPE_EDB9315A             772
#define MACH_TYPE_SUNSET               773
#define MACH_TYPE_STARGATE2            774
#define MACH_TYPE_INTELMOTE2           775
#define MACH_TYPE_TRIZEPS4             776
#define MACH_TYPE_MAINSTONE2           777
#define MACH_TYPE_EZ_IXP42X            778
#define MACH_TYPE_TAPWAVE_ZODIAC       779
#define MACH_TYPE_UNIVERSALMETER       780
#define MACH_TYPE_HICOARM9             781
#define MACH_TYPE_PNX4008              782
#define MACH_TYPE_KWS6000              783
#define MACH_TYPE_PORTUX920T           784
#define MACH_TYPE_EZ_X5                785
#define MACH_TYPE_OMAP_RUDOLPH         786
#define MACH_TYPE_CPUAT91              787
#define MACH_TYPE_REA9200              788
#define MACH_TYPE_ACTS_PUNE_SA1110     789
#define MACH_TYPE_IXP425               790
#define MACH_TYPE_I30030ADS            791
#define MACH_TYPE_PERCH                792
#define MACH_TYPE_EIS05R1              793
#define MACH_TYPE_PEPPERPAD            794
#define MACH_TYPE_SB3010               795
#define MACH_TYPE_RM9200               796
#define MACH_TYPE_DMA03                797
#define MACH_TYPE_ROAD_S101            798
#define MACH_TYPE_IQ_NEXTGEN_A         799
#define MACH_TYPE_IQ_NEXTGEN_B         800
#define MACH_TYPE_IQ_NEXTGEN_C         801
#define MACH_TYPE_IQ_NEXTGEN_D         802
#define MACH_TYPE_IQ_NEXTGEN_E         803
#define MACH_TYPE_MALLOW_AT91          804
#define MACH_TYPE_CYBERTRACKER_I       805
#define MACH_TYPE_GESBC931X            806
#define MACH_TYPE_CENTIPAD             807
#define MACH_TYPE_ARMSOC               808
#define MACH_TYPE_SE4200               809
#define MACH_TYPE_EMS197A              810
#define MACH_TYPE_MICRO9               811
#define MACH_TYPE_MICRO9L              812
#define MACH_TYPE_UC5471DSP            813
#define MACH_TYPE_SJ5471ENG            814
#define MACH_TYPE_CMPXA26X             815
#define MACH_TYPE_NC                   816
#define MACH_TYPE_OMAP_PALMTE          817
#define MACH_TYPE_AJAX52X              818
#define MACH_TYPE_SIRIUSTAR            819
#define MACH_TYPE_IODATA_HDLG          820
#define MACH_TYPE_AT91RM9200UTL        821
#define MACH_TYPE_BIOSAFE              822
#define MACH_TYPE_MP1000               823
#define MACH_TYPE_PARSY                824
#define MACH_TYPE_CCXP                 825
#define MACH_TYPE_OMAP_GSAMPLE         826
#define MACH_TYPE_REALVIEW_EB          827
#define MACH_TYPE_SAMOA                828
#define MACH_TYPE_T3XSCALE             829
#define MACH_TYPE_I878                 830
#define MACH_TYPE_BORZOI               831
#define MACH_TYPE_GECKO                832
#define MACH_TYPE_DS101                833
#define MACH_TYPE_OMAP_PALMTT2         834
#define MACH_TYPE_XSCALE_PALMLD        835
#define MACH_TYPE_CC9C                 836
#define MACH_TYPE_SBC1670              837
#define MACH_TYPE_IXDP28X5             838
#define MACH_TYPE_OMAP_PALMTT          839
#define MACH_TYPE_ML696K               840
#define MACH_TYPE_ARCOM_ZEUS           841
#define MACH_TYPE_OSIRIS               842
#define MACH_TYPE_MAESTRO              843
#define MACH_TYPE_TUNGE2               844
#define MACH_TYPE_IXBBM                845
#define MACH_TYPE_MX27ADS              846
#define MACH_TYPE_AX8004               847
#define MACH_TYPE_AT91SAM9261EK        848
#define MACH_TYPE_LOFT                 849
#define MACH_TYPE_MAGPIE               850
#define MACH_TYPE_MX21ADS              851
#define MACH_TYPE_MB87M3400            852
#define MACH_TYPE_MGUARD_DELTA         853
#define MACH_TYPE_DAVINCI_DVDP         854
#define MACH_TYPE_HTCUNIVERSAL         855
#define MACH_TYPE_TPAD                 856
#define MACH_TYPE_ROVERP3              857
#define MACH_TYPE_JORNADA928           858
#define MACH_TYPE_MV88FXX81            859
#define MACH_TYPE_STMP36XX             860
#define MACH_TYPE_SXNI79524            861
#define MACH_TYPE_AMS_DELTA            862
#define MACH_TYPE_URANIUM              863
#define MACH_TYPE_UCON                 864
#define MACH_TYPE_NAS100D              865
#define MACH_TYPE_L083_1000            866
#define MACH_TYPE_EZX                  867
#define MACH_TYPE_PNX5220              868
#define MACH_TYPE_BUTTE                869
#define MACH_TYPE_SRM2                 870
#define MACH_TYPE_DSBR                 871
#define MACH_TYPE_CRYSTALBALL          872
#define MACH_TYPE_TINYPXA27X           873
#define MACH_TYPE_HERBIE               874
#define MACH_TYPE_MAGICIAN             875
#define MACH_TYPE_CM4002               876
#define MACH_TYPE_B4                   877
#define MACH_TYPE_MAUI                 878
#define MACH_TYPE_CYBERTRACKER_G       879
#define MACH_TYPE_NXDKN                880
#define MACH_TYPE_MIO8390              881
#define MACH_TYPE_OMI_BOARD            882
#define MACH_TYPE_MX21CIV              883
#define MACH_TYPE_MAHI_CDAC            884
#define MACH_TYPE_XSCALE_PALMTX        885
#define MACH_TYPE_S3C2413              887
#define MACH_TYPE_SAMSYS_EP0           888
#define MACH_TYPE_WG302V1              889
#define MACH_TYPE_WG302V2              890
#define MACH_TYPE_EB42X                891
#define MACH_TYPE_IQ331ES              892
#define MACH_TYPE_COSYDSP              893
#define MACH_TYPE_UPLAT7D              894
#define MACH_TYPE_PTDAVINCI            895
#define MACH_TYPE_MBUS                 896
#define MACH_TYPE_NADIA2VB             897
#define MACH_TYPE_R1000                898
#define MACH_TYPE_HW90250              899
#define MACH_TYPE_OMAP_2430SDP         900
#define MACH_TYPE_DAVINCI_EVM          901
#define MACH_TYPE_OMAP_TORNADO         902
#define MACH_TYPE_OLOCREEK             903
#define MACH_TYPE_PALMZ72              904
#define MACH_TYPE_NXDB500              905
#define MACH_TYPE_APF9328              906
#define MACH_TYPE_OMAP_WIPOQ           907
#define MACH_TYPE_OMAP_TWIP            908
#define MACH_TYPE_XSCALE_PALMTREO650   909
#define MACH_TYPE_ACUMEN               910
#define MACH_TYPE_XP100                911
#define MACH_TYPE_FS2410               912
#define MACH_TYPE_PXA270_CERF          913
#define MACH_TYPE_SQ2FTLPALM           914
#define MACH_TYPE_BSEMSERVER           915
#define MACH_TYPE_NETCLIENT            916
#define MACH_TYPE_XSCALE_PALMTT5       917
#define MACH_TYPE_OMAP_PALMTC          918
#define MACH_TYPE_OMAP_APOLLON         919
#define MACH_TYPE_MXC30030EVB          920
#define MACH_TYPE_REA_2D               921
#define MACH_TYPE_TI3E524              922
#define MACH_TYPE_ATEB9200             923
#define MACH_TYPE_AUCKLAND             924
#define MACH_TYPE_AK3320M              925
#define MACH_TYPE_DURAMAX              926
#define MACH_TYPE_N35                  927
#define MACH_TYPE_PRONGHORN            928
#define MACH_TYPE_FUNDY                929
#define MACH_TYPE_LOGICPD_PXA270       930
#define MACH_TYPE_CPU777               931
#define MACH_TYPE_SIMICON9201          932
#define MACH_TYPE_LEAP2_HPM            933
#define MACH_TYPE_CM922TXA10           934
#define MACH_TYPE_PXA                  935
#define MACH_TYPE_SANDGATE2            936
#define MACH_TYPE_SANDGATE2G           937
#define MACH_TYPE_SANDGATE2P           938
#define MACH_TYPE_FRED_JACK            939
#define MACH_TYPE_TTG_COLOR1           940
#define MACH_TYPE_NXEB500HMI           941
#define MACH_TYPE_NETDCU8              942
#define MACH_TYPE_ML675050_CPU_BOA     943
#define MACH_TYPE_NG_FVX538            944
#define MACH_TYPE_NG_FVS338            945
#define MACH_TYPE_PNX4103              946
#define MACH_TYPE_HESDB                947
#define MACH_TYPE_XSILO                948
#define MACH_TYPE_ESPRESSO             949
#define MACH_TYPE_EMLC                 950
#define MACH_TYPE_SISTERON             951
#define MACH_TYPE_RX1950               952
#define MACH_TYPE_TSC_VENUS            953
#define MACH_TYPE_DS101J               954
#define MACH_TYPE_MXC30030ADS          955
#define MACH_TYPE_FUJITSU_WIMAXSOC     956
#define MACH_TYPE_DUALPCMODEM          957
#define MACH_TYPE_GESBC9312            958
#define MACH_TYPE_HTCAPACHE            959
#define MACH_TYPE_IXDP435              960
#define MACH_TYPE_CATPROVT100          961
#define MACH_TYPE_PICOTUX1XX           962
#define MACH_TYPE_PICOTUX2XX           963
#define MACH_TYPE_DSMG600              964
#define MACH_TYPE_EMPC2                965
#define MACH_TYPE_VENTURA              966
#define MACH_TYPE_PHIDGET_SBC          967
#define MACH_TYPE_IJ3K                 968
#define MACH_TYPE_PISGAH               969
#define MACH_TYPE_OMAP_FSAMPLE         970
#define MACH_TYPE_SG720                971
#define MACH_TYPE_REDFOX               972
#define MACH_TYPE_MYSH_EP9315_1        973
#define MACH_TYPE_TPF106               974
#define MACH_TYPE_AT91RM9200KG         975
#define MACH_TYPE_SLEDB                976
#define MACH_TYPE_ONTRACK              977
#define MACH_TYPE_PM1200               978
#define MACH_TYPE_ESS24XXX             979
#define MACH_TYPE_COREMP7              980
#define MACH_TYPE_NEXCODER_6446        981
#define MACH_TYPE_STVC8380             982
#define MACH_TYPE_TEKLYNX              983
#define MACH_TYPE_CARBONADO            984
#define MACH_TYPE_SYSMOS_MP730         985
#define MACH_TYPE_SNAPPER_CL15         986
#define MACH_TYPE_PGIGIM               987
#define MACH_TYPE_PTX9160P2            988
#define MACH_TYPE_DCORE1               989
#define MACH_TYPE_VICTORPXA            990
#define MACH_TYPE_MX2DTB               991
#define MACH_TYPE_PXA_IREX_ER0100      992
#define MACH_TYPE_OMAP_PALMZ71         993
#define MACH_TYPE_BARTEC_DEG           994
#define MACH_TYPE_HW50251              995
#define MACH_TYPE_IBOX                 996
#define MACH_TYPE_ATLASLH7A404         997
#define MACH_TYPE_PT2026               998
#define MACH_TYPE_HTCALPINE            999
#define MACH_TYPE_BARTEC_VTU           1000
#define MACH_TYPE_VCOREII              1001
#define MACH_TYPE_PDNB3                1002
#define MACH_TYPE_HTCBEETLES           1003
#define MACH_TYPE_S3C6400              1004
#define MACH_TYPE_S3C2443              1005
#define MACH_TYPE_OMAP_LDK             1006
#define MACH_TYPE_SMDK2460             1007
#define MACH_TYPE_SMDK2440             1008
#define MACH_TYPE_SMDK2412             1009
#define MACH_TYPE_WEBBOX               1010
#define MACH_TYPE_CWWNDP               1011
#define MACH_TYPE_DRAGON               1012
#define MACH_TYPE_OPENDO_CPU_BOARD     1013
#define MACH_TYPE_CCM2200              1014
#define MACH_TYPE_ETWARM               1015
#define MACH_TYPE_M93030               1016
#define MACH_TYPE_CC7U                 1017
#define MACH_TYPE_MTT_RANGER           1018
#define MACH_TYPE_NEXUS                1019
#define MACH_TYPE_DESMAN               1020
#define MACH_TYPE_BKDE303              1021
#define MACH_TYPE_SMDK2413             1022
#define MACH_TYPE_AML_M7200            1023
#define MACH_TYPE_AML_M5900            1024
#define MACH_TYPE_SG640                1025
#define MACH_TYPE_EDG79524             1026
#define MACH_TYPE_AI2410               1027
#define MACH_TYPE_IXP465               1028
#define MACH_TYPE_BALLOON3             1029
#define MACH_TYPE_HEINS                1030
#define MACH_TYPE_MPLUSEVA             1031
#define MACH_TYPE_RT042                1032
#define MACH_TYPE_CWIEM                1033
#define MACH_TYPE_CM_X270              1034
#define MACH_TYPE_CM_X255              1035
#define MACH_TYPE_ESH_AT91             1036
#define MACH_TYPE_SANDGATE3            1037
#define MACH_TYPE_PRIMO                1038
#define MACH_TYPE_GEMSTONE             1039
#define MACH_TYPE_PRONGHORNMETRO       1040
#define MACH_TYPE_SIDEWINDER           1041
#define MACH_TYPE_PICOMOD1             1042
#define MACH_TYPE_SG590                1043
#define MACH_TYPE_AKAI9307             1044
#define MACH_TYPE_FONTAINE             1045
#define MACH_TYPE_WOMBAT               1046
#define MACH_TYPE_ACQ300               1047
#define MACH_TYPE_MOD_270              1048
#define MACH_TYPE_VC0820               1049
#define MACH_TYPE_ANI_AIM              1050
#define MACH_TYPE_JELLYFISH            1051
#define MACH_TYPE_AMANITA              1052
#define MACH_TYPE_VLINK                1053
#define MACH_TYPE_DEXFLEX              1054
#define MACH_TYPE_EIGEN_TTQ            1055
#define MACH_TYPE_ARCOM_TITAN          1056
#define MACH_TYPE_TABLA                1057
#define MACH_TYPE_MDIRAC3              1058
#define MACH_TYPE_MRHFBP2              1059
#define MACH_TYPE_AT91RM9200RB         1060
#define MACH_TYPE_ANI_APM              1061
#define MACH_TYPE_ELLA1                1062
#define MACH_TYPE_INHAND_PXA27X        1063
#define MACH_TYPE_INHAND_PXA25X        1064
#define MACH_TYPE_EMPOS_XM             1065
#define MACH_TYPE_EMPOS                1066
#define MACH_TYPE_EMPOS_TINY           1067
#define MACH_TYPE_EMPOS_SM             1068
#define MACH_TYPE_EGRET                1069
#define MACH_TYPE_OSTRICH              1070
#define MACH_TYPE_N50                  1071
#define MACH_TYPE_ECBAT91              1072
#define MACH_TYPE_STAREAST             1073
#define MACH_TYPE_DSPG_DW              1074
#define MACH_TYPE_ONEARM               1075
#define MACH_TYPE_MRG110_6             1076
#define MACH_TYPE_WRT300NV2            1077
#define MACH_TYPE_XM_BULVERDE          1078
#define MACH_TYPE_MSM6100              1079
#define MACH_TYPE_ETI_B1               1080
#define MACH_TYPE_ZILOG_ZA9L           1081
#define MACH_TYPE_BIT2440              1082
#define MACH_TYPE_NBI                  1083
#define MACH_TYPE_SMDK2443             1084
#define MACH_TYPE_VDAVINCI             1085
#define MACH_TYPE_ATC6                 1086
#define MACH_TYPE_MULTMDW              1087
#define MACH_TYPE_MBA2440              1088
#define MACH_TYPE_ECSD                 1089
#define MACH_TYPE_ZIRE31               1090
#define MACH_TYPE_FSG                  1091
#define MACH_TYPE_RAZOR101             1092
#define MACH_TYPE_OPERA_TDM            1093
#define MACH_TYPE_COMCERTO             1094
#define MACH_TYPE_TB0319               1095
#define MACH_TYPE_KWS8000              1096
#define MACH_TYPE_B2                   1097
#define MACH_TYPE_LCL54                1098
#define MACH_TYPE_AT91SAM9260EK        1099
#define MACH_TYPE_GLANTANK             1100
#define MACH_TYPE_N2100                1101
#define MACH_TYPE_N4100                1102
#define MACH_TYPE_VERTICAL_RSC4        1103
#define MACH_TYPE_SG8100               1104
#define MACH_TYPE_IM42XX               1105
#define MACH_TYPE_FTXX                 1106
#define MACH_TYPE_LWFUSION             1107
#define MACH_TYPE_QT2410               1108
#define MACH_TYPE_KIXRP435             1109
#define MACH_TYPE_CCW9C                1110
#define MACH_TYPE_DABHS                1111
#define MACH_TYPE_GZMX                 1112
#define MACH_TYPE_IPNW100AP            1113
#define MACH_TYPE_CC9P9360DEV          1114
#define MACH_TYPE_CC9P9750DEV          1115
#define MACH_TYPE_CC9P9360VAL          1116
#define MACH_TYPE_CC9P9750VAL          1117
#define MACH_TYPE_NX70V                1118
#define MACH_TYPE_AT91RM9200DF         1119
#define MACH_TYPE_SE_PILOT2            1120
#define MACH_TYPE_MTCN_T800            1121
#define MACH_TYPE_VCMX212              1122
#define MACH_TYPE_LYNX                 1123
#define MACH_TYPE_AT91SAM9260ID        1124
#define MACH_TYPE_HW86052              1125
#define MACH_TYPE_PILZ_PMI3            1126
#define MACH_TYPE_EDB9302A             1127
#define MACH_TYPE_EDB9307A             1128
#define MACH_TYPE_CT_DFS               1129
#define MACH_TYPE_PILZ_PMI4            1130
#define MACH_TYPE_XCEEDNP_IXP          1131
#define MACH_TYPE_SMDK2442B            1132
#define MACH_TYPE_XNODE                1133
#define MACH_TYPE_AIDX270              1134
#define MACH_TYPE_REMA                 1135
#define MACH_TYPE_BPS1000              1136
#define MACH_TYPE_HW90350              1137
#define MACH_TYPE_OMAP_SDP3430         1138
#define MACH_TYPE_BLUETOUCH            1139
#define MACH_TYPE_VSTMS                1140
#define MACH_TYPE_XSBASE270            1141
#define MACH_TYPE_AT91SAM9260EK_CN     1142
#define MACH_TYPE_ADSTURBOXB           1143
#define MACH_TYPE_OTI4110              1144
#define MACH_TYPE_HME_PXA              1145
#define MACH_TYPE_DEISTERDCA           1146

#ifdef CONFIG_ARCH_EBSA110
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EBSA110
# endif
# define machine_is_ebsa110()	(machine_arch_type == MACH_TYPE_EBSA110)
#else
# define machine_is_ebsa110()	(0)
#endif

#ifdef CONFIG_ARCH_RPC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RISCPC
# endif
# define machine_is_riscpc()	(machine_arch_type == MACH_TYPE_RISCPC)
#else
# define machine_is_riscpc()	(0)
#endif

#ifdef CONFIG_ARCH_NEXUSPCI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NEXUSPCI
# endif
# define machine_is_nexuspci()	(machine_arch_type == MACH_TYPE_NEXUSPCI)
#else
# define machine_is_nexuspci()	(0)
#endif

#ifdef CONFIG_ARCH_EBSA285
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EBSA285
# endif
# define machine_is_ebsa285()	(machine_arch_type == MACH_TYPE_EBSA285)
#else
# define machine_is_ebsa285()	(0)
#endif

#ifdef CONFIG_ARCH_NETWINDER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NETWINDER
# endif
# define machine_is_netwinder()	(machine_arch_type == MACH_TYPE_NETWINDER)
#else
# define machine_is_netwinder()	(0)
#endif

#ifdef CONFIG_ARCH_CATS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CATS
# endif
# define machine_is_cats()	(machine_arch_type == MACH_TYPE_CATS)
#else
# define machine_is_cats()	(0)
#endif

#ifdef CONFIG_ARCH_TBOX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TBOX
# endif
# define machine_is_tbox()	(machine_arch_type == MACH_TYPE_TBOX)
#else
# define machine_is_tbox()	(0)
#endif

#ifdef CONFIG_ARCH_CO285
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CO285
# endif
# define machine_is_co285()	(machine_arch_type == MACH_TYPE_CO285)
#else
# define machine_is_co285()	(0)
#endif

#ifdef CONFIG_ARCH_CLPS7110
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CLPS7110
# endif
# define machine_is_clps7110()	(machine_arch_type == MACH_TYPE_CLPS7110)
#else
# define machine_is_clps7110()	(0)
#endif

#ifdef CONFIG_ARCH_ARC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARCHIMEDES
# endif
# define machine_is_archimedes()	(machine_arch_type == MACH_TYPE_ARCHIMEDES)
#else
# define machine_is_archimedes()	(0)
#endif

#ifdef CONFIG_ARCH_A5K
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_A5K
# endif
# define machine_is_a5k()	(machine_arch_type == MACH_TYPE_A5K)
#else
# define machine_is_a5k()	(0)
#endif

#ifdef CONFIG_ARCH_ETOILE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ETOILE
# endif
# define machine_is_etoile()	(machine_arch_type == MACH_TYPE_ETOILE)
#else
# define machine_is_etoile()	(0)
#endif

#ifdef CONFIG_ARCH_LACIE_NAS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LACIE_NAS
# endif
# define machine_is_lacie_nas()	(machine_arch_type == MACH_TYPE_LACIE_NAS)
#else
# define machine_is_lacie_nas()	(0)
#endif

#ifdef CONFIG_ARCH_CLPS7500
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CLPS7500
# endif
# define machine_is_clps7500()	(machine_arch_type == MACH_TYPE_CLPS7500)
#else
# define machine_is_clps7500()	(0)
#endif

#ifdef CONFIG_ARCH_SHARK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SHARK
# endif
# define machine_is_shark()	(machine_arch_type == MACH_TYPE_SHARK)
#else
# define machine_is_shark()	(0)
#endif

#ifdef CONFIG_SA1100_BRUTUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BRUTUS
# endif
# define machine_is_brutus()	(machine_arch_type == MACH_TYPE_BRUTUS)
#else
# define machine_is_brutus()	(0)
#endif

#ifdef CONFIG_ARCH_PERSONAL_SERVER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PERSONAL_SERVER
# endif
# define machine_is_personal_server()	(machine_arch_type == MACH_TYPE_PERSONAL_SERVER)
#else
# define machine_is_personal_server()	(0)
#endif

#ifdef CONFIG_SA1100_ITSY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ITSY
# endif
# define machine_is_itsy()	(machine_arch_type == MACH_TYPE_ITSY)
#else
# define machine_is_itsy()	(0)
#endif

#ifdef CONFIG_ARCH_L7200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_L7200
# endif
# define machine_is_l7200()	(machine_arch_type == MACH_TYPE_L7200)
#else
# define machine_is_l7200()	(0)
#endif

#ifdef CONFIG_SA1100_PLEB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PLEB
# endif
# define machine_is_pleb()	(machine_arch_type == MACH_TYPE_PLEB)
#else
# define machine_is_pleb()	(0)
#endif

#ifdef CONFIG_ARCH_INTEGRATOR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_INTEGRATOR
# endif
# define machine_is_integrator()	(machine_arch_type == MACH_TYPE_INTEGRATOR)
#else
# define machine_is_integrator()	(0)
#endif

#ifdef CONFIG_SA1100_H3600
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H3600
# endif
# define machine_is_h3600()	(machine_arch_type == MACH_TYPE_H3600)
#else
# define machine_is_h3600()	(0)
#endif

#ifdef CONFIG_ARCH_IXP1200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXP1200
# endif
# define machine_is_ixp1200()	(machine_arch_type == MACH_TYPE_IXP1200)
#else
# define machine_is_ixp1200()	(0)
#endif

#ifdef CONFIG_ARCH_P720T
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_P720T
# endif
# define machine_is_p720t()	(machine_arch_type == MACH_TYPE_P720T)
#else
# define machine_is_p720t()	(0)
#endif

#ifdef CONFIG_SA1100_ASSABET
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ASSABET
# endif
# define machine_is_assabet()	(machine_arch_type == MACH_TYPE_ASSABET)
#else
# define machine_is_assabet()	(0)
#endif

#ifdef CONFIG_SA1100_VICTOR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VICTOR
# endif
# define machine_is_victor()	(machine_arch_type == MACH_TYPE_VICTOR)
#else
# define machine_is_victor()	(0)
#endif

#ifdef CONFIG_SA1100_LART
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LART
# endif
# define machine_is_lart()	(machine_arch_type == MACH_TYPE_LART)
#else
# define machine_is_lart()	(0)
#endif

#ifdef CONFIG_SA1100_RANGER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RANGER
# endif
# define machine_is_ranger()	(machine_arch_type == MACH_TYPE_RANGER)
#else
# define machine_is_ranger()	(0)
#endif

#ifdef CONFIG_SA1100_GRAPHICSCLIENT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GRAPHICSCLIENT
# endif
# define machine_is_graphicsclient()	(machine_arch_type == MACH_TYPE_GRAPHICSCLIENT)
#else
# define machine_is_graphicsclient()	(0)
#endif

#ifdef CONFIG_SA1100_XP860
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XP860
# endif
# define machine_is_xp860()	(machine_arch_type == MACH_TYPE_XP860)
#else
# define machine_is_xp860()	(0)
#endif

#ifdef CONFIG_SA1100_CERF
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CERF
# endif
# define machine_is_cerf()	(machine_arch_type == MACH_TYPE_CERF)
#else
# define machine_is_cerf()	(0)
#endif

#ifdef CONFIG_SA1100_NANOENGINE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NANOENGINE
# endif
# define machine_is_nanoengine()	(machine_arch_type == MACH_TYPE_NANOENGINE)
#else
# define machine_is_nanoengine()	(0)
#endif

#ifdef CONFIG_SA1100_FPIC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FPIC
# endif
# define machine_is_fpic()	(machine_arch_type == MACH_TYPE_FPIC)
#else
# define machine_is_fpic()	(0)
#endif

#ifdef CONFIG_SA1100_EXTENEX1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EXTENEX1
# endif
# define machine_is_extenex1()	(machine_arch_type == MACH_TYPE_EXTENEX1)
#else
# define machine_is_extenex1()	(0)
#endif

#ifdef CONFIG_SA1100_SHERMAN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SHERMAN
# endif
# define machine_is_sherman()	(machine_arch_type == MACH_TYPE_SHERMAN)
#else
# define machine_is_sherman()	(0)
#endif

#ifdef CONFIG_SA1100_ACCELENT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACCELENT_SA
# endif
# define machine_is_accelent_sa()	(machine_arch_type == MACH_TYPE_ACCELENT_SA)
#else
# define machine_is_accelent_sa()	(0)
#endif

#ifdef CONFIG_ARCH_L7200_ACCELENT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACCELENT_L7200
# endif
# define machine_is_accelent_l7200()	(machine_arch_type == MACH_TYPE_ACCELENT_L7200)
#else
# define machine_is_accelent_l7200()	(0)
#endif

#ifdef CONFIG_SA1100_NETPORT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NETPORT
# endif
# define machine_is_netport()	(machine_arch_type == MACH_TYPE_NETPORT)
#else
# define machine_is_netport()	(0)
#endif

#ifdef CONFIG_SA1100_PANGOLIN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PANGOLIN
# endif
# define machine_is_pangolin()	(machine_arch_type == MACH_TYPE_PANGOLIN)
#else
# define machine_is_pangolin()	(0)
#endif

#ifdef CONFIG_SA1100_YOPY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_YOPY
# endif
# define machine_is_yopy()	(machine_arch_type == MACH_TYPE_YOPY)
#else
# define machine_is_yopy()	(0)
#endif

#ifdef CONFIG_SA1100_COOLIDGE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COOLIDGE
# endif
# define machine_is_coolidge()	(machine_arch_type == MACH_TYPE_COOLIDGE)
#else
# define machine_is_coolidge()	(0)
#endif

#ifdef CONFIG_SA1100_HUW_WEBPANEL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HUW_WEBPANEL
# endif
# define machine_is_huw_webpanel()	(machine_arch_type == MACH_TYPE_HUW_WEBPANEL)
#else
# define machine_is_huw_webpanel()	(0)
#endif

#ifdef CONFIG_ARCH_SPOTME
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPOTME
# endif
# define machine_is_spotme()	(machine_arch_type == MACH_TYPE_SPOTME)
#else
# define machine_is_spotme()	(0)
#endif

#ifdef CONFIG_ARCH_FREEBIRD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FREEBIRD
# endif
# define machine_is_freebird()	(machine_arch_type == MACH_TYPE_FREEBIRD)
#else
# define machine_is_freebird()	(0)
#endif

#ifdef CONFIG_ARCH_TI925
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TI925
# endif
# define machine_is_ti925()	(machine_arch_type == MACH_TYPE_TI925)
#else
# define machine_is_ti925()	(0)
#endif

#ifdef CONFIG_ARCH_RISCSTATION
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RISCSTATION
# endif
# define machine_is_riscstation()	(machine_arch_type == MACH_TYPE_RISCSTATION)
#else
# define machine_is_riscstation()	(0)
#endif

#ifdef CONFIG_SA1100_CAVY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CAVY
# endif
# define machine_is_cavy()	(machine_arch_type == MACH_TYPE_CAVY)
#else
# define machine_is_cavy()	(0)
#endif

#ifdef CONFIG_SA1100_JORNADA720
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_JORNADA720
# endif
# define machine_is_jornada720()	(machine_arch_type == MACH_TYPE_JORNADA720)
#else
# define machine_is_jornada720()	(0)
#endif

#ifdef CONFIG_SA1100_OMNIMETER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMNIMETER
# endif
# define machine_is_omnimeter()	(machine_arch_type == MACH_TYPE_OMNIMETER)
#else
# define machine_is_omnimeter()	(0)
#endif

#ifdef CONFIG_ARCH_EDB7211
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB7211
# endif
# define machine_is_edb7211()	(machine_arch_type == MACH_TYPE_EDB7211)
#else
# define machine_is_edb7211()	(0)
#endif

#ifdef CONFIG_SA1100_CITYGO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CITYGO
# endif
# define machine_is_citygo()	(machine_arch_type == MACH_TYPE_CITYGO)
#else
# define machine_is_citygo()	(0)
#endif

#ifdef CONFIG_SA1100_PFS168
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PFS168
# endif
# define machine_is_pfs168()	(machine_arch_type == MACH_TYPE_PFS168)
#else
# define machine_is_pfs168()	(0)
#endif

#ifdef CONFIG_SA1100_SPOT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPOT
# endif
# define machine_is_spot()	(machine_arch_type == MACH_TYPE_SPOT)
#else
# define machine_is_spot()	(0)
#endif

#ifdef CONFIG_SA1100_FLEXANET
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FLEXANET
# endif
# define machine_is_flexanet()	(machine_arch_type == MACH_TYPE_FLEXANET)
#else
# define machine_is_flexanet()	(0)
#endif

#ifdef CONFIG_ARCH_WEBPAL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WEBPAL
# endif
# define machine_is_webpal()	(machine_arch_type == MACH_TYPE_WEBPAL)
#else
# define machine_is_webpal()	(0)
#endif

#ifdef CONFIG_SA1100_LINPDA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LINPDA
# endif
# define machine_is_linpda()	(machine_arch_type == MACH_TYPE_LINPDA)
#else
# define machine_is_linpda()	(0)
#endif

#ifdef CONFIG_ARCH_ANAKIN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ANAKIN
# endif
# define machine_is_anakin()	(machine_arch_type == MACH_TYPE_ANAKIN)
#else
# define machine_is_anakin()	(0)
#endif

#ifdef CONFIG_SA1100_MVI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MVI
# endif
# define machine_is_mvi()	(machine_arch_type == MACH_TYPE_MVI)
#else
# define machine_is_mvi()	(0)
#endif

#ifdef CONFIG_SA1100_JUPITER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_JUPITER
# endif
# define machine_is_jupiter()	(machine_arch_type == MACH_TYPE_JUPITER)
#else
# define machine_is_jupiter()	(0)
#endif

#ifdef CONFIG_ARCH_PSIONW
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PSIONW
# endif
# define machine_is_psionw()	(machine_arch_type == MACH_TYPE_PSIONW)
#else
# define machine_is_psionw()	(0)
#endif

#ifdef CONFIG_SA1100_ALN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ALN
# endif
# define machine_is_aln()	(machine_arch_type == MACH_TYPE_ALN)
#else
# define machine_is_aln()	(0)
#endif

#ifdef CONFIG_ARCH_CAMELOT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CAMELOT
# endif
# define machine_is_epxa()	(machine_arch_type == MACH_TYPE_CAMELOT)
#else
# define machine_is_epxa()	(0)
#endif

#ifdef CONFIG_SA1100_GDS2200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GDS2200
# endif
# define machine_is_gds2200()	(machine_arch_type == MACH_TYPE_GDS2200)
#else
# define machine_is_gds2200()	(0)
#endif

#ifdef CONFIG_SA1100_PSION_SERIES7
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PSION_SERIES7
# endif
# define machine_is_psion_series7()	(machine_arch_type == MACH_TYPE_PSION_SERIES7)
#else
# define machine_is_psion_series7()	(0)
#endif

#ifdef CONFIG_SA1100_XFILE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XFILE
# endif
# define machine_is_xfile()	(machine_arch_type == MACH_TYPE_XFILE)
#else
# define machine_is_xfile()	(0)
#endif

#ifdef CONFIG_ARCH_ACCELENT_EP9312
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACCELENT_EP9312
# endif
# define machine_is_accelent_ep9312()	(machine_arch_type == MACH_TYPE_ACCELENT_EP9312)
#else
# define machine_is_accelent_ep9312()	(0)
#endif

#ifdef CONFIG_ARCH_IC200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IC200
# endif
# define machine_is_ic200()	(machine_arch_type == MACH_TYPE_IC200)
#else
# define machine_is_ic200()	(0)
#endif

#ifdef CONFIG_SA1100_CREDITLART
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CREDITLART
# endif
# define machine_is_creditlart()	(machine_arch_type == MACH_TYPE_CREDITLART)
#else
# define machine_is_creditlart()	(0)
#endif

#ifdef CONFIG_SA1100_HTM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HTM
# endif
# define machine_is_htm()	(machine_arch_type == MACH_TYPE_HTM)
#else
# define machine_is_htm()	(0)
#endif

#ifdef CONFIG_ARCH_IQ80310
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IQ80310
# endif
# define machine_is_iq80310()	(machine_arch_type == MACH_TYPE_IQ80310)
#else
# define machine_is_iq80310()	(0)
#endif

#ifdef CONFIG_SA1100_FREEBOT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FREEBOT
# endif
# define machine_is_freebot()	(machine_arch_type == MACH_TYPE_FREEBOT)
#else
# define machine_is_freebot()	(0)
#endif

#ifdef CONFIG_ARCH_ENTEL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ENTEL
# endif
# define machine_is_entel()	(machine_arch_type == MACH_TYPE_ENTEL)
#else
# define machine_is_entel()	(0)
#endif

#ifdef CONFIG_ARCH_ENP3510
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ENP3510
# endif
# define machine_is_enp3510()	(machine_arch_type == MACH_TYPE_ENP3510)
#else
# define machine_is_enp3510()	(0)
#endif

#ifdef CONFIG_SA1100_TRIZEPS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TRIZEPS
# endif
# define machine_is_trizeps()	(machine_arch_type == MACH_TYPE_TRIZEPS)
#else
# define machine_is_trizeps()	(0)
#endif

#ifdef CONFIG_SA1100_NESA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NESA
# endif
# define machine_is_nesa()	(machine_arch_type == MACH_TYPE_NESA)
#else
# define machine_is_nesa()	(0)
#endif

#ifdef CONFIG_ARCH_VENUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VENUS
# endif
# define machine_is_venus()	(machine_arch_type == MACH_TYPE_VENUS)
#else
# define machine_is_venus()	(0)
#endif

#ifdef CONFIG_ARCH_TARDIS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TARDIS
# endif
# define machine_is_tardis()	(machine_arch_type == MACH_TYPE_TARDIS)
#else
# define machine_is_tardis()	(0)
#endif

#ifdef CONFIG_ARCH_MERCURY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MERCURY
# endif
# define machine_is_mercury()	(machine_arch_type == MACH_TYPE_MERCURY)
#else
# define machine_is_mercury()	(0)
#endif

#ifdef CONFIG_SA1100_EMPEG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EMPEG
# endif
# define machine_is_empeg()	(machine_arch_type == MACH_TYPE_EMPEG)
#else
# define machine_is_empeg()	(0)
#endif

#ifdef CONFIG_ARCH_I80200FCC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_I80200FCC
# endif
# define machine_is_adi_evb()	(machine_arch_type == MACH_TYPE_I80200FCC)
#else
# define machine_is_adi_evb()	(0)
#endif

#ifdef CONFIG_SA1100_ITT_CPB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ITT_CPB
# endif
# define machine_is_itt_cpb()	(machine_arch_type == MACH_TYPE_ITT_CPB)
#else
# define machine_is_itt_cpb()	(0)
#endif

#ifdef CONFIG_SA1100_SVC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SVC
# endif
# define machine_is_svc()	(machine_arch_type == MACH_TYPE_SVC)
#else
# define machine_is_svc()	(0)
#endif

#ifdef CONFIG_SA1100_ALPHA2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ALPHA2
# endif
# define machine_is_alpha2()	(machine_arch_type == MACH_TYPE_ALPHA2)
#else
# define machine_is_alpha2()	(0)
#endif

#ifdef CONFIG_SA1100_ALPHA1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ALPHA1
# endif
# define machine_is_alpha1()	(machine_arch_type == MACH_TYPE_ALPHA1)
#else
# define machine_is_alpha1()	(0)
#endif

#ifdef CONFIG_ARCH_NETARM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NETARM
# endif
# define machine_is_netarm()	(machine_arch_type == MACH_TYPE_NETARM)
#else
# define machine_is_netarm()	(0)
#endif

#ifdef CONFIG_SA1100_SIMPAD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SIMPAD
# endif
# define machine_is_simpad()	(machine_arch_type == MACH_TYPE_SIMPAD)
#else
# define machine_is_simpad()	(0)
#endif

#ifdef CONFIG_ARCH_PDA1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PDA1
# endif
# define machine_is_pda1()	(machine_arch_type == MACH_TYPE_PDA1)
#else
# define machine_is_pda1()	(0)
#endif

#ifdef CONFIG_ARCH_LUBBOCK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LUBBOCK
# endif
# define machine_is_lubbock()	(machine_arch_type == MACH_TYPE_LUBBOCK)
#else
# define machine_is_lubbock()	(0)
#endif

#ifdef CONFIG_ARCH_ANIKO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ANIKO
# endif
# define machine_is_aniko()	(machine_arch_type == MACH_TYPE_ANIKO)
#else
# define machine_is_aniko()	(0)
#endif

#ifdef CONFIG_ARCH_CLEP7212
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CLEP7212
# endif
# define machine_is_clep7212()	(machine_arch_type == MACH_TYPE_CLEP7212)
#else
# define machine_is_clep7212()	(0)
#endif

#ifdef CONFIG_ARCH_CS89712
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CS89712
# endif
# define machine_is_cs89712()	(machine_arch_type == MACH_TYPE_CS89712)
#else
# define machine_is_cs89712()	(0)
#endif

#ifdef CONFIG_SA1100_WEARARM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WEARARM
# endif
# define machine_is_weararm()	(machine_arch_type == MACH_TYPE_WEARARM)
#else
# define machine_is_weararm()	(0)
#endif

#ifdef CONFIG_SA1100_POSSIO_PX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_POSSIO_PX
# endif
# define machine_is_possio_px()	(machine_arch_type == MACH_TYPE_POSSIO_PX)
#else
# define machine_is_possio_px()	(0)
#endif

#ifdef CONFIG_SA1100_SIDEARM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SIDEARM
# endif
# define machine_is_sidearm()	(machine_arch_type == MACH_TYPE_SIDEARM)
#else
# define machine_is_sidearm()	(0)
#endif

#ifdef CONFIG_SA1100_STORK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STORK
# endif
# define machine_is_stork()	(machine_arch_type == MACH_TYPE_STORK)
#else
# define machine_is_stork()	(0)
#endif

#ifdef CONFIG_SA1100_SHANNON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SHANNON
# endif
# define machine_is_shannon()	(machine_arch_type == MACH_TYPE_SHANNON)
#else
# define machine_is_shannon()	(0)
#endif

#ifdef CONFIG_ARCH_ACE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACE
# endif
# define machine_is_ace()	(machine_arch_type == MACH_TYPE_ACE)
#else
# define machine_is_ace()	(0)
#endif

#ifdef CONFIG_SA1100_BALLYARM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BALLYARM
# endif
# define machine_is_ballyarm()	(machine_arch_type == MACH_TYPE_BALLYARM)
#else
# define machine_is_ballyarm()	(0)
#endif

#ifdef CONFIG_SA1100_SIMPUTER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SIMPUTER
# endif
# define machine_is_simputer()	(machine_arch_type == MACH_TYPE_SIMPUTER)
#else
# define machine_is_simputer()	(0)
#endif

#ifdef CONFIG_SA1100_NEXTERM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NEXTERM
# endif
# define machine_is_nexterm()	(machine_arch_type == MACH_TYPE_NEXTERM)
#else
# define machine_is_nexterm()	(0)
#endif

#ifdef CONFIG_SA1100_SA1100_ELF
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SA1100_ELF
# endif
# define machine_is_sa1100_elf()	(machine_arch_type == MACH_TYPE_SA1100_ELF)
#else
# define machine_is_sa1100_elf()	(0)
#endif

#ifdef CONFIG_SA1100_GATOR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GATOR
# endif
# define machine_is_gator()	(machine_arch_type == MACH_TYPE_GATOR)
#else
# define machine_is_gator()	(0)
#endif

#ifdef CONFIG_ARCH_GRANITE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GRANITE
# endif
# define machine_is_granite()	(machine_arch_type == MACH_TYPE_GRANITE)
#else
# define machine_is_granite()	(0)
#endif

#ifdef CONFIG_SA1100_CONSUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CONSUS
# endif
# define machine_is_consus()	(machine_arch_type == MACH_TYPE_CONSUS)
#else
# define machine_is_consus()	(0)
#endif

#ifdef CONFIG_ARCH_AAED2000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AAED2000
# endif
# define machine_is_aaed2000()	(machine_arch_type == MACH_TYPE_AAED2000)
#else
# define machine_is_aaed2000()	(0)
#endif

#ifdef CONFIG_ARCH_CDB89712
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CDB89712
# endif
# define machine_is_cdb89712()	(machine_arch_type == MACH_TYPE_CDB89712)
#else
# define machine_is_cdb89712()	(0)
#endif

#ifdef CONFIG_SA1100_GRAPHICSMASTER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GRAPHICSMASTER
# endif
# define machine_is_graphicsmaster()	(machine_arch_type == MACH_TYPE_GRAPHICSMASTER)
#else
# define machine_is_graphicsmaster()	(0)
#endif

#ifdef CONFIG_SA1100_ADSBITSY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADSBITSY
# endif
# define machine_is_adsbitsy()	(machine_arch_type == MACH_TYPE_ADSBITSY)
#else
# define machine_is_adsbitsy()	(0)
#endif

#ifdef CONFIG_ARCH_PXA_IDP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_IDP
# endif
# define machine_is_pxa_idp()	(machine_arch_type == MACH_TYPE_PXA_IDP)
#else
# define machine_is_pxa_idp()	(0)
#endif

#ifdef CONFIG_ARCH_PLCE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PLCE
# endif
# define machine_is_plce()	(machine_arch_type == MACH_TYPE_PLCE)
#else
# define machine_is_plce()	(0)
#endif

#ifdef CONFIG_SA1100_PT_SYSTEM3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PT_SYSTEM3
# endif
# define machine_is_pt_system3()	(machine_arch_type == MACH_TYPE_PT_SYSTEM3)
#else
# define machine_is_pt_system3()	(0)
#endif

#ifdef CONFIG_ARCH_MEDALB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MEDALB
# endif
# define machine_is_murphy()	(machine_arch_type == MACH_TYPE_MEDALB)
#else
# define machine_is_murphy()	(0)
#endif

#ifdef CONFIG_ARCH_EAGLE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EAGLE
# endif
# define machine_is_eagle()	(machine_arch_type == MACH_TYPE_EAGLE)
#else
# define machine_is_eagle()	(0)
#endif

#ifdef CONFIG_ARCH_DSC21
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DSC21
# endif
# define machine_is_dsc21()	(machine_arch_type == MACH_TYPE_DSC21)
#else
# define machine_is_dsc21()	(0)
#endif

#ifdef CONFIG_ARCH_DSC24
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DSC24
# endif
# define machine_is_dsc24()	(machine_arch_type == MACH_TYPE_DSC24)
#else
# define machine_is_dsc24()	(0)
#endif

#ifdef CONFIG_ARCH_TI5472
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TI5472
# endif
# define machine_is_ti5472()	(machine_arch_type == MACH_TYPE_TI5472)
#else
# define machine_is_ti5472()	(0)
#endif

#ifdef CONFIG_ARCH_AUTCPU12
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AUTCPU12
# endif
# define machine_is_autcpu12()	(machine_arch_type == MACH_TYPE_AUTCPU12)
#else
# define machine_is_autcpu12()	(0)
#endif

#ifdef CONFIG_ARCH_UENGINE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_UENGINE
# endif
# define machine_is_uengine()	(machine_arch_type == MACH_TYPE_UENGINE)
#else
# define machine_is_uengine()	(0)
#endif

#ifdef CONFIG_SA1100_BLUESTEM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BLUESTEM
# endif
# define machine_is_bluestem()	(machine_arch_type == MACH_TYPE_BLUESTEM)
#else
# define machine_is_bluestem()	(0)
#endif

#ifdef CONFIG_ARCH_XINGU8
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XINGU8
# endif
# define machine_is_xingu8()	(machine_arch_type == MACH_TYPE_XINGU8)
#else
# define machine_is_xingu8()	(0)
#endif

#ifdef CONFIG_ARCH_BUSHSTB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BUSHSTB
# endif
# define machine_is_bushstb()	(machine_arch_type == MACH_TYPE_BUSHSTB)
#else
# define machine_is_bushstb()	(0)
#endif

#ifdef CONFIG_SA1100_EPSILON1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EPSILON1
# endif
# define machine_is_epsilon1()	(machine_arch_type == MACH_TYPE_EPSILON1)
#else
# define machine_is_epsilon1()	(0)
#endif

#ifdef CONFIG_SA1100_BALLOON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BALLOON
# endif
# define machine_is_balloon()	(machine_arch_type == MACH_TYPE_BALLOON)
#else
# define machine_is_balloon()	(0)
#endif

#ifdef CONFIG_ARCH_PUPPY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PUPPY
# endif
# define machine_is_puppy()	(machine_arch_type == MACH_TYPE_PUPPY)
#else
# define machine_is_puppy()	(0)
#endif

#ifdef CONFIG_SA1100_ELROY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ELROY
# endif
# define machine_is_elroy()	(machine_arch_type == MACH_TYPE_ELROY)
#else
# define machine_is_elroy()	(0)
#endif

#ifdef CONFIG_ARCH_GMS720
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GMS720
# endif
# define machine_is_gms720()	(machine_arch_type == MACH_TYPE_GMS720)
#else
# define machine_is_gms720()	(0)
#endif

#ifdef CONFIG_ARCH_S24X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S24X
# endif
# define machine_is_s24x()	(machine_arch_type == MACH_TYPE_S24X)
#else
# define machine_is_s24x()	(0)
#endif

#ifdef CONFIG_ARCH_JTEL_CLEP7312
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_JTEL_CLEP7312
# endif
# define machine_is_jtel_clep7312()	(machine_arch_type == MACH_TYPE_JTEL_CLEP7312)
#else
# define machine_is_jtel_clep7312()	(0)
#endif

#ifdef CONFIG_ARCH_CX821XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CX821XX
# endif
# define machine_is_cx821xx()	(machine_arch_type == MACH_TYPE_CX821XX)
#else
# define machine_is_cx821xx()	(0)
#endif

#ifdef CONFIG_ARCH_EDB7312
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB7312
# endif
# define machine_is_edb7312()	(machine_arch_type == MACH_TYPE_EDB7312)
#else
# define machine_is_edb7312()	(0)
#endif

#ifdef CONFIG_SA1100_BSA1110
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BSA1110
# endif
# define machine_is_bsa1110()	(machine_arch_type == MACH_TYPE_BSA1110)
#else
# define machine_is_bsa1110()	(0)
#endif

#ifdef CONFIG_ARCH_POWERPIN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_POWERPIN
# endif
# define machine_is_powerpin()	(machine_arch_type == MACH_TYPE_POWERPIN)
#else
# define machine_is_powerpin()	(0)
#endif

#ifdef CONFIG_ARCH_OPENARM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OPENARM
# endif
# define machine_is_openarm()	(machine_arch_type == MACH_TYPE_OPENARM)
#else
# define machine_is_openarm()	(0)
#endif

#ifdef CONFIG_SA1100_WHITECHAPEL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WHITECHAPEL
# endif
# define machine_is_whitechapel()	(machine_arch_type == MACH_TYPE_WHITECHAPEL)
#else
# define machine_is_whitechapel()	(0)
#endif

#ifdef CONFIG_SA1100_H3100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H3100
# endif
# define machine_is_h3100()	(machine_arch_type == MACH_TYPE_H3100)
#else
# define machine_is_h3100()	(0)
#endif

#ifdef CONFIG_SA1100_H3800
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H3800
# endif
# define machine_is_h3800()	(machine_arch_type == MACH_TYPE_H3800)
#else
# define machine_is_h3800()	(0)
#endif

#ifdef CONFIG_ARCH_BLUE_V1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BLUE_V1
# endif
# define machine_is_blue_v1()	(machine_arch_type == MACH_TYPE_BLUE_V1)
#else
# define machine_is_blue_v1()	(0)
#endif

#ifdef CONFIG_ARCH_PXA_CERF
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_CERF
# endif
# define machine_is_pxa_cerf()	(machine_arch_type == MACH_TYPE_PXA_CERF)
#else
# define machine_is_pxa_cerf()	(0)
#endif

#ifdef CONFIG_ARCH_ARM7TEVB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARM7TEVB
# endif
# define machine_is_arm7tevb()	(machine_arch_type == MACH_TYPE_ARM7TEVB)
#else
# define machine_is_arm7tevb()	(0)
#endif

#ifdef CONFIG_SA1100_D7400
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_D7400
# endif
# define machine_is_d7400()	(machine_arch_type == MACH_TYPE_D7400)
#else
# define machine_is_d7400()	(0)
#endif

#ifdef CONFIG_ARCH_PIRANHA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PIRANHA
# endif
# define machine_is_piranha()	(machine_arch_type == MACH_TYPE_PIRANHA)
#else
# define machine_is_piranha()	(0)
#endif

#ifdef CONFIG_SA1100_SBCAMELOT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SBCAMELOT
# endif
# define machine_is_sbcamelot()	(machine_arch_type == MACH_TYPE_SBCAMELOT)
#else
# define machine_is_sbcamelot()	(0)
#endif

#ifdef CONFIG_SA1100_KINGS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KINGS
# endif
# define machine_is_kings()	(machine_arch_type == MACH_TYPE_KINGS)
#else
# define machine_is_kings()	(0)
#endif

#ifdef CONFIG_ARCH_SMDK2400
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK2400
# endif
# define machine_is_smdk2400()	(machine_arch_type == MACH_TYPE_SMDK2400)
#else
# define machine_is_smdk2400()	(0)
#endif

#ifdef CONFIG_SA1100_COLLIE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COLLIE
# endif
# define machine_is_collie()	(machine_arch_type == MACH_TYPE_COLLIE)
#else
# define machine_is_collie()	(0)
#endif

#ifdef CONFIG_ARCH_IDR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IDR
# endif
# define machine_is_idr()	(machine_arch_type == MACH_TYPE_IDR)
#else
# define machine_is_idr()	(0)
#endif

#ifdef CONFIG_SA1100_BADGE4
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BADGE4
# endif
# define machine_is_badge4()	(machine_arch_type == MACH_TYPE_BADGE4)
#else
# define machine_is_badge4()	(0)
#endif

#ifdef CONFIG_ARCH_WEBNET
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WEBNET
# endif
# define machine_is_webnet()	(machine_arch_type == MACH_TYPE_WEBNET)
#else
# define machine_is_webnet()	(0)
#endif

#ifdef CONFIG_SA1100_D7300
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_D7300
# endif
# define machine_is_d7300()	(machine_arch_type == MACH_TYPE_D7300)
#else
# define machine_is_d7300()	(0)
#endif

#ifdef CONFIG_SA1100_CEP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CEP
# endif
# define machine_is_cep()	(machine_arch_type == MACH_TYPE_CEP)
#else
# define machine_is_cep()	(0)
#endif

#ifdef CONFIG_ARCH_FORTUNET
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FORTUNET
# endif
# define machine_is_fortunet()	(machine_arch_type == MACH_TYPE_FORTUNET)
#else
# define machine_is_fortunet()	(0)
#endif

#ifdef CONFIG_ARCH_VC547X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VC547X
# endif
# define machine_is_vc547x()	(machine_arch_type == MACH_TYPE_VC547X)
#else
# define machine_is_vc547x()	(0)
#endif

#ifdef CONFIG_SA1100_FILEWALKER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FILEWALKER
# endif
# define machine_is_filewalker()	(machine_arch_type == MACH_TYPE_FILEWALKER)
#else
# define machine_is_filewalker()	(0)
#endif

#ifdef CONFIG_SA1100_NETGATEWAY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NETGATEWAY
# endif
# define machine_is_netgateway()	(machine_arch_type == MACH_TYPE_NETGATEWAY)
#else
# define machine_is_netgateway()	(0)
#endif

#ifdef CONFIG_SA1100_SYMBOL2800
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SYMBOL2800
# endif
# define machine_is_symbol2800()	(machine_arch_type == MACH_TYPE_SYMBOL2800)
#else
# define machine_is_symbol2800()	(0)
#endif

#ifdef CONFIG_SA1100_SUNS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SUNS
# endif
# define machine_is_suns()	(machine_arch_type == MACH_TYPE_SUNS)
#else
# define machine_is_suns()	(0)
#endif

#ifdef CONFIG_SA1100_FRODO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FRODO
# endif
# define machine_is_frodo()	(machine_arch_type == MACH_TYPE_FRODO)
#else
# define machine_is_frodo()	(0)
#endif

#ifdef CONFIG_SA1100_MACH_TYTE_MS301
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MACH_TYTE_MS301
# endif
# define machine_is_ms301()	(machine_arch_type == MACH_TYPE_MACH_TYTE_MS301)
#else
# define machine_is_ms301()	(0)
#endif

#ifdef CONFIG_ARCH_MX1ADS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX1ADS
# endif
# define machine_is_mx1ads()	(machine_arch_type == MACH_TYPE_MX1ADS)
#else
# define machine_is_mx1ads()	(0)
#endif

#ifdef CONFIG_ARCH_H7201
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H7201
# endif
# define machine_is_h7201()	(machine_arch_type == MACH_TYPE_H7201)
#else
# define machine_is_h7201()	(0)
#endif

#ifdef CONFIG_ARCH_H7202
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H7202
# endif
# define machine_is_h7202()	(machine_arch_type == MACH_TYPE_H7202)
#else
# define machine_is_h7202()	(0)
#endif

#ifdef CONFIG_ARCH_AMICO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AMICO
# endif
# define machine_is_amico()	(machine_arch_type == MACH_TYPE_AMICO)
#else
# define machine_is_amico()	(0)
#endif

#ifdef CONFIG_SA1100_IAM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IAM
# endif
# define machine_is_iam()	(machine_arch_type == MACH_TYPE_IAM)
#else
# define machine_is_iam()	(0)
#endif

#ifdef CONFIG_SA1100_TT530
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TT530
# endif
# define machine_is_tt530()	(machine_arch_type == MACH_TYPE_TT530)
#else
# define machine_is_tt530()	(0)
#endif

#ifdef CONFIG_ARCH_SAM2400
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SAM2400
# endif
# define machine_is_sam2400()	(machine_arch_type == MACH_TYPE_SAM2400)
#else
# define machine_is_sam2400()	(0)
#endif

#ifdef CONFIG_SA1100_JORNADA56X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_JORNADA56X
# endif
# define machine_is_jornada56x()	(machine_arch_type == MACH_TYPE_JORNADA56X)
#else
# define machine_is_jornada56x()	(0)
#endif

#ifdef CONFIG_SA1100_ACTIVE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACTIVE
# endif
# define machine_is_active()	(machine_arch_type == MACH_TYPE_ACTIVE)
#else
# define machine_is_active()	(0)
#endif

#ifdef CONFIG_ARCH_IQ80321
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IQ80321
# endif
# define machine_is_iq80321()	(machine_arch_type == MACH_TYPE_IQ80321)
#else
# define machine_is_iq80321()	(0)
#endif

#ifdef CONFIG_SA1100_WID
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WID
# endif
# define machine_is_wid()	(machine_arch_type == MACH_TYPE_WID)
#else
# define machine_is_wid()	(0)
#endif

#ifdef CONFIG_ARCH_SABINAL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SABINAL
# endif
# define machine_is_sabinal()	(machine_arch_type == MACH_TYPE_SABINAL)
#else
# define machine_is_sabinal()	(0)
#endif

#ifdef CONFIG_ARCH_IXP425_MATACUMBE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXP425_MATACUMBE
# endif
# define machine_is_ixp425_matacumbe()	(machine_arch_type == MACH_TYPE_IXP425_MATACUMBE)
#else
# define machine_is_ixp425_matacumbe()	(0)
#endif

#ifdef CONFIG_SA1100_MINIPRINT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MINIPRINT
# endif
# define machine_is_miniprint()	(machine_arch_type == MACH_TYPE_MINIPRINT)
#else
# define machine_is_miniprint()	(0)
#endif

#ifdef CONFIG_ARCH_ADM510X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADM510X
# endif
# define machine_is_adm510x()	(machine_arch_type == MACH_TYPE_ADM510X)
#else
# define machine_is_adm510x()	(0)
#endif

#ifdef CONFIG_SA1100_SVS200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SVS200
# endif
# define machine_is_svs200()	(machine_arch_type == MACH_TYPE_SVS200)
#else
# define machine_is_svs200()	(0)
#endif

#ifdef CONFIG_ARCH_ATG_TCU
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ATG_TCU
# endif
# define machine_is_atg_tcu()	(machine_arch_type == MACH_TYPE_ATG_TCU)
#else
# define machine_is_atg_tcu()	(0)
#endif

#ifdef CONFIG_SA1100_JORNADA820
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_JORNADA820
# endif
# define machine_is_jornada820()	(machine_arch_type == MACH_TYPE_JORNADA820)
#else
# define machine_is_jornada820()	(0)
#endif

#ifdef CONFIG_ARCH_S3C44B0
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S3C44B0
# endif
# define machine_is_s3c44b0()	(machine_arch_type == MACH_TYPE_S3C44B0)
#else
# define machine_is_s3c44b0()	(0)
#endif

#ifdef CONFIG_ARCH_MARGIS2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MARGIS2
# endif
# define machine_is_margis2()	(machine_arch_type == MACH_TYPE_MARGIS2)
#else
# define machine_is_margis2()	(0)
#endif

#ifdef CONFIG_ARCH_KS8695
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KS8695
# endif
# define machine_is_ks8695()	(machine_arch_type == MACH_TYPE_KS8695)
#else
# define machine_is_ks8695()	(0)
#endif

#ifdef CONFIG_ARCH_BRH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BRH
# endif
# define machine_is_brh()	(machine_arch_type == MACH_TYPE_BRH)
#else
# define machine_is_brh()	(0)
#endif

#ifdef CONFIG_ARCH_S3C2410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S3C2410
# endif
# define machine_is_s3c2410()	(machine_arch_type == MACH_TYPE_S3C2410)
#else
# define machine_is_s3c2410()	(0)
#endif

#ifdef CONFIG_ARCH_POSSIO_PX30
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_POSSIO_PX30
# endif
# define machine_is_possio_px30()	(machine_arch_type == MACH_TYPE_POSSIO_PX30)
#else
# define machine_is_possio_px30()	(0)
#endif

#ifdef CONFIG_ARCH_S3C2800
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S3C2800
# endif
# define machine_is_s3c2800()	(machine_arch_type == MACH_TYPE_S3C2800)
#else
# define machine_is_s3c2800()	(0)
#endif

#ifdef CONFIG_SA1100_FLEETWOOD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FLEETWOOD
# endif
# define machine_is_fleetwood()	(machine_arch_type == MACH_TYPE_FLEETWOOD)
#else
# define machine_is_fleetwood()	(0)
#endif

#ifdef CONFIG_ARCH_OMAHA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAHA
# endif
# define machine_is_omaha()	(machine_arch_type == MACH_TYPE_OMAHA)
#else
# define machine_is_omaha()	(0)
#endif

#ifdef CONFIG_ARCH_TA7
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TA7
# endif
# define machine_is_ta7()	(machine_arch_type == MACH_TYPE_TA7)
#else
# define machine_is_ta7()	(0)
#endif

#ifdef CONFIG_SA1100_NOVA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NOVA
# endif
# define machine_is_nova()	(machine_arch_type == MACH_TYPE_NOVA)
#else
# define machine_is_nova()	(0)
#endif

#ifdef CONFIG_ARCH_HMK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HMK
# endif
# define machine_is_hmk()	(machine_arch_type == MACH_TYPE_HMK)
#else
# define machine_is_hmk()	(0)
#endif

#ifdef CONFIG_ARCH_KARO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KARO
# endif
# define machine_is_karo()	(machine_arch_type == MACH_TYPE_KARO)
#else
# define machine_is_karo()	(0)
#endif

#ifdef CONFIG_SA1100_FESTER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FESTER
# endif
# define machine_is_fester()	(machine_arch_type == MACH_TYPE_FESTER)
#else
# define machine_is_fester()	(0)
#endif

#ifdef CONFIG_ARCH_GPI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GPI
# endif
# define machine_is_gpi()	(machine_arch_type == MACH_TYPE_GPI)
#else
# define machine_is_gpi()	(0)
#endif

#ifdef CONFIG_ARCH_SMDK2410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK2410
# endif
# define machine_is_smdk2410()	(machine_arch_type == MACH_TYPE_SMDK2410)
#else
# define machine_is_smdk2410()	(0)
#endif

#ifdef CONFIG_ARCH_I519
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_I519
# endif
# define machine_is_i519()	(machine_arch_type == MACH_TYPE_I519)
#else
# define machine_is_i519()	(0)
#endif

#ifdef CONFIG_SA1100_NEXIO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NEXIO
# endif
# define machine_is_nexio()	(machine_arch_type == MACH_TYPE_NEXIO)
#else
# define machine_is_nexio()	(0)
#endif

#ifdef CONFIG_SA1100_BITBOX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BITBOX
# endif
# define machine_is_bitbox()	(machine_arch_type == MACH_TYPE_BITBOX)
#else
# define machine_is_bitbox()	(0)
#endif

#ifdef CONFIG_SA1100_G200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_G200
# endif
# define machine_is_g200()	(machine_arch_type == MACH_TYPE_G200)
#else
# define machine_is_g200()	(0)
#endif

#ifdef CONFIG_SA1100_GILL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GILL
# endif
# define machine_is_gill()	(machine_arch_type == MACH_TYPE_GILL)
#else
# define machine_is_gill()	(0)
#endif

#ifdef CONFIG_ARCH_PXA_MERCURY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_MERCURY
# endif
# define machine_is_pxa_mercury()	(machine_arch_type == MACH_TYPE_PXA_MERCURY)
#else
# define machine_is_pxa_mercury()	(0)
#endif

#ifdef CONFIG_ARCH_CEIVA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CEIVA
# endif
# define machine_is_ceiva()	(machine_arch_type == MACH_TYPE_CEIVA)
#else
# define machine_is_ceiva()	(0)
#endif

#ifdef CONFIG_SA1100_FRET
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FRET
# endif
# define machine_is_fret()	(machine_arch_type == MACH_TYPE_FRET)
#else
# define machine_is_fret()	(0)
#endif

#ifdef CONFIG_SA1100_EMAILPHONE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EMAILPHONE
# endif
# define machine_is_emailphone()	(machine_arch_type == MACH_TYPE_EMAILPHONE)
#else
# define machine_is_emailphone()	(0)
#endif

#ifdef CONFIG_ARCH_H3900
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H3900
# endif
# define machine_is_h3900()	(machine_arch_type == MACH_TYPE_H3900)
#else
# define machine_is_h3900()	(0)
#endif

#ifdef CONFIG_ARCH_PXA1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA1
# endif
# define machine_is_pxa1()	(machine_arch_type == MACH_TYPE_PXA1)
#else
# define machine_is_pxa1()	(0)
#endif

#ifdef CONFIG_SA1100_KOAN369
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KOAN369
# endif
# define machine_is_koan369()	(machine_arch_type == MACH_TYPE_KOAN369)
#else
# define machine_is_koan369()	(0)
#endif

#ifdef CONFIG_ARCH_COGENT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COGENT
# endif
# define machine_is_cogent()	(machine_arch_type == MACH_TYPE_COGENT)
#else
# define machine_is_cogent()	(0)
#endif

#ifdef CONFIG_ARCH_ESL_SIMPUTER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESL_SIMPUTER
# endif
# define machine_is_esl_simputer()	(machine_arch_type == MACH_TYPE_ESL_SIMPUTER)
#else
# define machine_is_esl_simputer()	(0)
#endif

#ifdef CONFIG_ARCH_ESL_SIMPUTER_CLR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESL_SIMPUTER_CLR
# endif
# define machine_is_esl_simputer_clr()	(machine_arch_type == MACH_TYPE_ESL_SIMPUTER_CLR)
#else
# define machine_is_esl_simputer_clr()	(0)
#endif

#ifdef CONFIG_ARCH_ESL_SIMPUTER_BW
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESL_SIMPUTER_BW
# endif
# define machine_is_esl_simputer_bw()	(machine_arch_type == MACH_TYPE_ESL_SIMPUTER_BW)
#else
# define machine_is_esl_simputer_bw()	(0)
#endif

#ifdef CONFIG_ARCH_HHP_CRADLE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HHP_CRADLE
# endif
# define machine_is_hhp_cradle()	(machine_arch_type == MACH_TYPE_HHP_CRADLE)
#else
# define machine_is_hhp_cradle()	(0)
#endif

#ifdef CONFIG_ARCH_HE500
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HE500
# endif
# define machine_is_he500()	(machine_arch_type == MACH_TYPE_HE500)
#else
# define machine_is_he500()	(0)
#endif

#ifdef CONFIG_SA1100_INHANDELF2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_INHANDELF2
# endif
# define machine_is_inhandelf2()	(machine_arch_type == MACH_TYPE_INHANDELF2)
#else
# define machine_is_inhandelf2()	(0)
#endif

#ifdef CONFIG_SA1100_INHANDFTIP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_INHANDFTIP
# endif
# define machine_is_inhandftip()	(machine_arch_type == MACH_TYPE_INHANDFTIP)
#else
# define machine_is_inhandftip()	(0)
#endif

#ifdef CONFIG_SA1100_DNP1110
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DNP1110
# endif
# define machine_is_dnp1110()	(machine_arch_type == MACH_TYPE_DNP1110)
#else
# define machine_is_dnp1110()	(0)
#endif

#ifdef CONFIG_SA1100_PNP1110
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PNP1110
# endif
# define machine_is_pnp1110()	(machine_arch_type == MACH_TYPE_PNP1110)
#else
# define machine_is_pnp1110()	(0)
#endif

#ifdef CONFIG_ARCH_CSB226
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CSB226
# endif
# define machine_is_csb226()	(machine_arch_type == MACH_TYPE_CSB226)
#else
# define machine_is_csb226()	(0)
#endif

#ifdef CONFIG_SA1100_ARNOLD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARNOLD
# endif
# define machine_is_arnold()	(machine_arch_type == MACH_TYPE_ARNOLD)
#else
# define machine_is_arnold()	(0)
#endif

#ifdef CONFIG_MACH_VOICEBLUE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VOICEBLUE
# endif
# define machine_is_voiceblue()	(machine_arch_type == MACH_TYPE_VOICEBLUE)
#else
# define machine_is_voiceblue()	(0)
#endif

#ifdef CONFIG_ARCH_JZ8028
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_JZ8028
# endif
# define machine_is_jz8028()	(machine_arch_type == MACH_TYPE_JZ8028)
#else
# define machine_is_jz8028()	(0)
#endif

#ifdef CONFIG_ARCH_H5400
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H5400
# endif
# define machine_is_h5400()	(machine_arch_type == MACH_TYPE_H5400)
#else
# define machine_is_h5400()	(0)
#endif

#ifdef CONFIG_SA1100_FORTE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FORTE
# endif
# define machine_is_forte()	(machine_arch_type == MACH_TYPE_FORTE)
#else
# define machine_is_forte()	(0)
#endif

#ifdef CONFIG_SA1100_ACAM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACAM
# endif
# define machine_is_acam()	(machine_arch_type == MACH_TYPE_ACAM)
#else
# define machine_is_acam()	(0)
#endif

#ifdef CONFIG_SA1100_ABOX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ABOX
# endif
# define machine_is_abox()	(machine_arch_type == MACH_TYPE_ABOX)
#else
# define machine_is_abox()	(0)
#endif

#ifdef CONFIG_ARCH_ATMEL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ATMEL
# endif
# define machine_is_atmel()	(machine_arch_type == MACH_TYPE_ATMEL)
#else
# define machine_is_atmel()	(0)
#endif

#ifdef CONFIG_ARCH_SITSANG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SITSANG
# endif
# define machine_is_sitsang()	(machine_arch_type == MACH_TYPE_SITSANG)
#else
# define machine_is_sitsang()	(0)
#endif

#ifdef CONFIG_SA1100_CPU1110LCDNET
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CPU1110LCDNET
# endif
# define machine_is_cpu1110lcdnet()	(machine_arch_type == MACH_TYPE_CPU1110LCDNET)
#else
# define machine_is_cpu1110lcdnet()	(0)
#endif

#ifdef CONFIG_ARCH_MPL_VCMA9
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MPL_VCMA9
# endif
# define machine_is_mpl_vcma9()	(machine_arch_type == MACH_TYPE_MPL_VCMA9)
#else
# define machine_is_mpl_vcma9()	(0)
#endif

#ifdef CONFIG_ARCH_OPUS_A1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OPUS_A1
# endif
# define machine_is_opus_a1()	(machine_arch_type == MACH_TYPE_OPUS_A1)
#else
# define machine_is_opus_a1()	(0)
#endif

#ifdef CONFIG_ARCH_DAYTONA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DAYTONA
# endif
# define machine_is_daytona()	(machine_arch_type == MACH_TYPE_DAYTONA)
#else
# define machine_is_daytona()	(0)
#endif

#ifdef CONFIG_SA1100_KILLBEAR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KILLBEAR
# endif
# define machine_is_killbear()	(machine_arch_type == MACH_TYPE_KILLBEAR)
#else
# define machine_is_killbear()	(0)
#endif

#ifdef CONFIG_ARCH_YOHO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_YOHO
# endif
# define machine_is_yoho()	(machine_arch_type == MACH_TYPE_YOHO)
#else
# define machine_is_yoho()	(0)
#endif

#ifdef CONFIG_ARCH_JASPER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_JASPER
# endif
# define machine_is_jasper()	(machine_arch_type == MACH_TYPE_JASPER)
#else
# define machine_is_jasper()	(0)
#endif

#ifdef CONFIG_ARCH_DSC25
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DSC25
# endif
# define machine_is_dsc25()	(machine_arch_type == MACH_TYPE_DSC25)
#else
# define machine_is_dsc25()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_INNOVATOR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_INNOVATOR
# endif
# define machine_is_omap_innovator()	(machine_arch_type == MACH_TYPE_OMAP_INNOVATOR)
#else
# define machine_is_omap_innovator()	(0)
#endif

#ifdef CONFIG_ARCH_RAMSES
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RAMSES
# endif
# define machine_is_mnci()	(machine_arch_type == MACH_TYPE_RAMSES)
#else
# define machine_is_mnci()	(0)
#endif

#ifdef CONFIG_ARCH_S28X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S28X
# endif
# define machine_is_s28x()	(machine_arch_type == MACH_TYPE_S28X)
#else
# define machine_is_s28x()	(0)
#endif

#ifdef CONFIG_ARCH_MPORT3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MPORT3
# endif
# define machine_is_mport3()	(machine_arch_type == MACH_TYPE_MPORT3)
#else
# define machine_is_mport3()	(0)
#endif

#ifdef CONFIG_ARCH_PXA_EAGLE250
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_EAGLE250
# endif
# define machine_is_pxa_eagle250()	(machine_arch_type == MACH_TYPE_PXA_EAGLE250)
#else
# define machine_is_pxa_eagle250()	(0)
#endif

#ifdef CONFIG_ARCH_PDB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PDB
# endif
# define machine_is_pdb()	(machine_arch_type == MACH_TYPE_PDB)
#else
# define machine_is_pdb()	(0)
#endif

#ifdef CONFIG_SA1100_BLUE_2G
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BLUE_2G
# endif
# define machine_is_blue_2g()	(machine_arch_type == MACH_TYPE_BLUE_2G)
#else
# define machine_is_blue_2g()	(0)
#endif

#ifdef CONFIG_SA1100_BLUEARCH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BLUEARCH
# endif
# define machine_is_bluearch()	(machine_arch_type == MACH_TYPE_BLUEARCH)
#else
# define machine_is_bluearch()	(0)
#endif

#ifdef CONFIG_ARCH_IXDP2400
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDP2400
# endif
# define machine_is_ixdp2400()	(machine_arch_type == MACH_TYPE_IXDP2400)
#else
# define machine_is_ixdp2400()	(0)
#endif

#ifdef CONFIG_ARCH_IXDP2800
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDP2800
# endif
# define machine_is_ixdp2800()	(machine_arch_type == MACH_TYPE_IXDP2800)
#else
# define machine_is_ixdp2800()	(0)
#endif

#ifdef CONFIG_SA1100_EXPLORER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EXPLORER
# endif
# define machine_is_explorer()	(machine_arch_type == MACH_TYPE_EXPLORER)
#else
# define machine_is_explorer()	(0)
#endif

#ifdef CONFIG_ARCH_IXDP425
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDP425
# endif
# define machine_is_ixdp425()	(machine_arch_type == MACH_TYPE_IXDP425)
#else
# define machine_is_ixdp425()	(0)
#endif

#ifdef CONFIG_ARCH_CHIMP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CHIMP
# endif
# define machine_is_chimp()	(machine_arch_type == MACH_TYPE_CHIMP)
#else
# define machine_is_chimp()	(0)
#endif

#ifdef CONFIG_ARCH_STORK_NEST
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STORK_NEST
# endif
# define machine_is_stork_nest()	(machine_arch_type == MACH_TYPE_STORK_NEST)
#else
# define machine_is_stork_nest()	(0)
#endif

#ifdef CONFIG_ARCH_STORK_EGG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STORK_EGG
# endif
# define machine_is_stork_egg()	(machine_arch_type == MACH_TYPE_STORK_EGG)
#else
# define machine_is_stork_egg()	(0)
#endif

#ifdef CONFIG_SA1100_WISMO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WISMO
# endif
# define machine_is_wismo()	(machine_arch_type == MACH_TYPE_WISMO)
#else
# define machine_is_wismo()	(0)
#endif

#ifdef CONFIG_ARCH_EZLINX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EZLINX
# endif
# define machine_is_ezlinx()	(machine_arch_type == MACH_TYPE_EZLINX)
#else
# define machine_is_ezlinx()	(0)
#endif

#ifdef CONFIG_ARCH_AT91RM9200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91RM9200
# endif
# define machine_is_at91rm9200()	(machine_arch_type == MACH_TYPE_AT91RM9200)
#else
# define machine_is_at91rm9200()	(0)
#endif

#ifdef CONFIG_ARCH_ORION
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ORION
# endif
# define machine_is_orion()	(machine_arch_type == MACH_TYPE_ORION)
#else
# define machine_is_orion()	(0)
#endif

#ifdef CONFIG_ARCH_NEPTUNE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NEPTUNE
# endif
# define machine_is_neptune()	(machine_arch_type == MACH_TYPE_NEPTUNE)
#else
# define machine_is_neptune()	(0)
#endif

#ifdef CONFIG_SA1100_HACKKIT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HACKKIT
# endif
# define machine_is_hackkit()	(machine_arch_type == MACH_TYPE_HACKKIT)
#else
# define machine_is_hackkit()	(0)
#endif

#ifdef CONFIG_ARCH_PXA_WINS30
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_WINS30
# endif
# define machine_is_pxa_wins30()	(machine_arch_type == MACH_TYPE_PXA_WINS30)
#else
# define machine_is_pxa_wins30()	(0)
#endif

#ifdef CONFIG_SA1100_LAVINNA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LAVINNA
# endif
# define machine_is_lavinna()	(machine_arch_type == MACH_TYPE_LAVINNA)
#else
# define machine_is_lavinna()	(0)
#endif

#ifdef CONFIG_ARCH_PXA_UENGINE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_UENGINE
# endif
# define machine_is_pxa_uengine()	(machine_arch_type == MACH_TYPE_PXA_UENGINE)
#else
# define machine_is_pxa_uengine()	(0)
#endif

#ifdef CONFIG_ARCH_INNOKOM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_INNOKOM
# endif
# define machine_is_innokom()	(machine_arch_type == MACH_TYPE_INNOKOM)
#else
# define machine_is_innokom()	(0)
#endif

#ifdef CONFIG_ARCH_BMS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BMS
# endif
# define machine_is_bms()	(machine_arch_type == MACH_TYPE_BMS)
#else
# define machine_is_bms()	(0)
#endif

#ifdef CONFIG_ARCH_IXCDP1100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXCDP1100
# endif
# define machine_is_ixcdp1100()	(machine_arch_type == MACH_TYPE_IXCDP1100)
#else
# define machine_is_ixcdp1100()	(0)
#endif

#ifdef CONFIG_ARCH_PRPMC1100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PRPMC1100
# endif
# define machine_is_prpmc1100()	(machine_arch_type == MACH_TYPE_PRPMC1100)
#else
# define machine_is_prpmc1100()	(0)
#endif

#ifdef CONFIG_ARCH_AT91RM9200DK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91RM9200DK
# endif
# define machine_is_at91rm9200dk()	(machine_arch_type == MACH_TYPE_AT91RM9200DK)
#else
# define machine_is_at91rm9200dk()	(0)
#endif

#ifdef CONFIG_ARCH_ARMSTICK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARMSTICK
# endif
# define machine_is_armstick()	(machine_arch_type == MACH_TYPE_ARMSTICK)
#else
# define machine_is_armstick()	(0)
#endif

#ifdef CONFIG_ARCH_ARMONIE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARMONIE
# endif
# define machine_is_armonie()	(machine_arch_type == MACH_TYPE_ARMONIE)
#else
# define machine_is_armonie()	(0)
#endif

#ifdef CONFIG_ARCH_MPORT1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MPORT1
# endif
# define machine_is_mport1()	(machine_arch_type == MACH_TYPE_MPORT1)
#else
# define machine_is_mport1()	(0)
#endif

#ifdef CONFIG_ARCH_S3C5410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S3C5410
# endif
# define machine_is_s3c5410()	(machine_arch_type == MACH_TYPE_S3C5410)
#else
# define machine_is_s3c5410()	(0)
#endif

#ifdef CONFIG_ARCH_ZCP320A
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ZCP320A
# endif
# define machine_is_zcp320a()	(machine_arch_type == MACH_TYPE_ZCP320A)
#else
# define machine_is_zcp320a()	(0)
#endif

#ifdef CONFIG_ARCH_I_BOX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_I_BOX
# endif
# define machine_is_i_box()	(machine_arch_type == MACH_TYPE_I_BOX)
#else
# define machine_is_i_box()	(0)
#endif

#ifdef CONFIG_ARCH_STLC1502
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STLC1502
# endif
# define machine_is_stlc1502()	(machine_arch_type == MACH_TYPE_STLC1502)
#else
# define machine_is_stlc1502()	(0)
#endif

#ifdef CONFIG_ARCH_SIREN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SIREN
# endif
# define machine_is_siren()	(machine_arch_type == MACH_TYPE_SIREN)
#else
# define machine_is_siren()	(0)
#endif

#ifdef CONFIG_ARCH_GREENLAKE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GREENLAKE
# endif
# define machine_is_greenlake()	(machine_arch_type == MACH_TYPE_GREENLAKE)
#else
# define machine_is_greenlake()	(0)
#endif

#ifdef CONFIG_ARCH_ARGUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARGUS
# endif
# define machine_is_argus()	(machine_arch_type == MACH_TYPE_ARGUS)
#else
# define machine_is_argus()	(0)
#endif

#ifdef CONFIG_SA1100_COMBADGE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COMBADGE
# endif
# define machine_is_combadge()	(machine_arch_type == MACH_TYPE_COMBADGE)
#else
# define machine_is_combadge()	(0)
#endif

#ifdef CONFIG_ARCH_ROKEPXA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ROKEPXA
# endif
# define machine_is_rokepxa()	(machine_arch_type == MACH_TYPE_ROKEPXA)
#else
# define machine_is_rokepxa()	(0)
#endif

#ifdef CONFIG_ARCH_CINTEGRATOR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CINTEGRATOR
# endif
# define machine_is_cintegrator()	(machine_arch_type == MACH_TYPE_CINTEGRATOR)
#else
# define machine_is_cintegrator()	(0)
#endif

#ifdef CONFIG_ARCH_GUIDEA07
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GUIDEA07
# endif
# define machine_is_guidea07()	(machine_arch_type == MACH_TYPE_GUIDEA07)
#else
# define machine_is_guidea07()	(0)
#endif

#ifdef CONFIG_ARCH_TAT257
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TAT257
# endif
# define machine_is_tat257()	(machine_arch_type == MACH_TYPE_TAT257)
#else
# define machine_is_tat257()	(0)
#endif

#ifdef CONFIG_ARCH_IGP2425
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IGP2425
# endif
# define machine_is_igp2425()	(machine_arch_type == MACH_TYPE_IGP2425)
#else
# define machine_is_igp2425()	(0)
#endif

#ifdef CONFIG_ARCH_BLUEGRAMMA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BLUEGRAMMA
# endif
# define machine_is_bluegrama()	(machine_arch_type == MACH_TYPE_BLUEGRAMMA)
#else
# define machine_is_bluegrama()	(0)
#endif

#ifdef CONFIG_ARCH_IPOD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IPOD
# endif
# define machine_is_ipod()	(machine_arch_type == MACH_TYPE_IPOD)
#else
# define machine_is_ipod()	(0)
#endif

#ifdef CONFIG_ARCH_ADSBITSYX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADSBITSYX
# endif
# define machine_is_adsbitsyx()	(machine_arch_type == MACH_TYPE_ADSBITSYX)
#else
# define machine_is_adsbitsyx()	(0)
#endif

#ifdef CONFIG_ARCH_TRIZEPS2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TRIZEPS2
# endif
# define machine_is_trizeps2()	(machine_arch_type == MACH_TYPE_TRIZEPS2)
#else
# define machine_is_trizeps2()	(0)
#endif

#ifdef CONFIG_ARCH_VIPER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VIPER
# endif
# define machine_is_viper()	(machine_arch_type == MACH_TYPE_VIPER)
#else
# define machine_is_viper()	(0)
#endif

#ifdef CONFIG_SA1100_ADSBITSYPLUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADSBITSYPLUS
# endif
# define machine_is_adsbitsyplus()	(machine_arch_type == MACH_TYPE_ADSBITSYPLUS)
#else
# define machine_is_adsbitsyplus()	(0)
#endif

#ifdef CONFIG_SA1100_ADSAGC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADSAGC
# endif
# define machine_is_adsagc()	(machine_arch_type == MACH_TYPE_ADSAGC)
#else
# define machine_is_adsagc()	(0)
#endif

#ifdef CONFIG_ARCH_STP7312
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STP7312
# endif
# define machine_is_stp7312()	(machine_arch_type == MACH_TYPE_STP7312)
#else
# define machine_is_stp7312()	(0)
#endif

#ifdef CONFIG_MACH_NX_PHNX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NX_PHNX
# endif
# define machine_is_nx_phnx()	(machine_arch_type == MACH_TYPE_NX_PHNX)
#else
# define machine_is_nx_phnx()	(0)
#endif

#ifdef CONFIG_ARCH_WEP_EP250
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WEP_EP250
# endif
# define machine_is_wep_ep250()	(machine_arch_type == MACH_TYPE_WEP_EP250)
#else
# define machine_is_wep_ep250()	(0)
#endif

#ifdef CONFIG_ARCH_INHANDELF3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_INHANDELF3
# endif
# define machine_is_inhandelf3()	(machine_arch_type == MACH_TYPE_INHANDELF3)
#else
# define machine_is_inhandelf3()	(0)
#endif

#ifdef CONFIG_ARCH_ADI_COYOTE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADI_COYOTE
# endif
# define machine_is_adi_coyote()	(machine_arch_type == MACH_TYPE_ADI_COYOTE)
#else
# define machine_is_adi_coyote()	(0)
#endif

#ifdef CONFIG_ARCH_IYONIX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IYONIX
# endif
# define machine_is_iyonix()	(machine_arch_type == MACH_TYPE_IYONIX)
#else
# define machine_is_iyonix()	(0)
#endif

#ifdef CONFIG_ARCH_DAMICAM_SA1110
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DAMICAM_SA1110
# endif
# define machine_is_damicam1()	(machine_arch_type == MACH_TYPE_DAMICAM_SA1110)
#else
# define machine_is_damicam1()	(0)
#endif

#ifdef CONFIG_ARCH_MEG03
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MEG03
# endif
# define machine_is_meg03()	(machine_arch_type == MACH_TYPE_MEG03)
#else
# define machine_is_meg03()	(0)
#endif

#ifdef CONFIG_ARCH_PXA_WHITECHAPEL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_WHITECHAPEL
# endif
# define machine_is_pxa_whitechapel()	(machine_arch_type == MACH_TYPE_PXA_WHITECHAPEL)
#else
# define machine_is_pxa_whitechapel()	(0)
#endif

#ifdef CONFIG_ARCH_NWSC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NWSC
# endif
# define machine_is_nwsc()	(machine_arch_type == MACH_TYPE_NWSC)
#else
# define machine_is_nwsc()	(0)
#endif

#ifdef CONFIG_ARCH_NWLARM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NWLARM
# endif
# define machine_is_nwlarm()	(machine_arch_type == MACH_TYPE_NWLARM)
#else
# define machine_is_nwlarm()	(0)
#endif

#ifdef CONFIG_ARCH_IXP425_MGUARD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXP425_MGUARD
# endif
# define machine_is_ixp425_mguard()	(machine_arch_type == MACH_TYPE_IXP425_MGUARD)
#else
# define machine_is_ixp425_mguard()	(0)
#endif

#ifdef CONFIG_ARCH_PXA_NETDCU4
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_NETDCU4
# endif
# define machine_is_pxa_netdcu4()	(machine_arch_type == MACH_TYPE_PXA_NETDCU4)
#else
# define machine_is_pxa_netdcu4()	(0)
#endif

#ifdef CONFIG_ARCH_IXDP2401
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDP2401
# endif
# define machine_is_ixdp2401()	(machine_arch_type == MACH_TYPE_IXDP2401)
#else
# define machine_is_ixdp2401()	(0)
#endif

#ifdef CONFIG_ARCH_IXDP2801
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDP2801
# endif
# define machine_is_ixdp2801()	(machine_arch_type == MACH_TYPE_IXDP2801)
#else
# define machine_is_ixdp2801()	(0)
#endif

#ifdef CONFIG_ARCH_ZODIAC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ZODIAC
# endif
# define machine_is_zodiac()	(machine_arch_type == MACH_TYPE_ZODIAC)
#else
# define machine_is_zodiac()	(0)
#endif

#ifdef CONFIG_ARCH_ARMMODUL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARMMODUL
# endif
# define machine_is_armmodul()	(machine_arch_type == MACH_TYPE_ARMMODUL)
#else
# define machine_is_armmodul()	(0)
#endif

#ifdef CONFIG_SA1100_KETOP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KETOP
# endif
# define machine_is_ketop()	(machine_arch_type == MACH_TYPE_KETOP)
#else
# define machine_is_ketop()	(0)
#endif

#ifdef CONFIG_ARCH_AV7200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AV7200
# endif
# define machine_is_av7200()	(machine_arch_type == MACH_TYPE_AV7200)
#else
# define machine_is_av7200()	(0)
#endif

#ifdef CONFIG_ARCH_ARCH_TI925
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARCH_TI925
# endif
# define machine_is_arch_ti925()	(machine_arch_type == MACH_TYPE_ARCH_TI925)
#else
# define machine_is_arch_ti925()	(0)
#endif

#ifdef CONFIG_ARCH_ACQ200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACQ200
# endif
# define machine_is_acq200()	(machine_arch_type == MACH_TYPE_ACQ200)
#else
# define machine_is_acq200()	(0)
#endif

#ifdef CONFIG_SA1100_PT_DAFIT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PT_DAFIT
# endif
# define machine_is_pt_dafit()	(machine_arch_type == MACH_TYPE_PT_DAFIT)
#else
# define machine_is_pt_dafit()	(0)
#endif

#ifdef CONFIG_ARCH_IHBA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IHBA
# endif
# define machine_is_ihba()	(machine_arch_type == MACH_TYPE_IHBA)
#else
# define machine_is_ihba()	(0)
#endif

#ifdef CONFIG_ARCH_QUINQUE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_QUINQUE
# endif
# define machine_is_quinque()	(machine_arch_type == MACH_TYPE_QUINQUE)
#else
# define machine_is_quinque()	(0)
#endif

#ifdef CONFIG_ARCH_NIMBRAONE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NIMBRAONE
# endif
# define machine_is_nimbraone()	(machine_arch_type == MACH_TYPE_NIMBRAONE)
#else
# define machine_is_nimbraone()	(0)
#endif

#ifdef CONFIG_ARCH_NIMBRA29X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NIMBRA29X
# endif
# define machine_is_nimbra29x()	(machine_arch_type == MACH_TYPE_NIMBRA29X)
#else
# define machine_is_nimbra29x()	(0)
#endif

#ifdef CONFIG_ARCH_NIMBRA210
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NIMBRA210
# endif
# define machine_is_nimbra210()	(machine_arch_type == MACH_TYPE_NIMBRA210)
#else
# define machine_is_nimbra210()	(0)
#endif

#ifdef CONFIG_ARCH_HHP_D95XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HHP_D95XX
# endif
# define machine_is_hhp_d95xx()	(machine_arch_type == MACH_TYPE_HHP_D95XX)
#else
# define machine_is_hhp_d95xx()	(0)
#endif

#ifdef CONFIG_ARCH_LABARM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LABARM
# endif
# define machine_is_labarm()	(machine_arch_type == MACH_TYPE_LABARM)
#else
# define machine_is_labarm()	(0)
#endif

#ifdef CONFIG_ARCH_M825XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_M825XX
# endif
# define machine_is_m825xx()	(machine_arch_type == MACH_TYPE_M825XX)
#else
# define machine_is_m825xx()	(0)
#endif

#ifdef CONFIG_SA1100_M7100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_M7100
# endif
# define machine_is_m7100()	(machine_arch_type == MACH_TYPE_M7100)
#else
# define machine_is_m7100()	(0)
#endif

#ifdef CONFIG_ARCH_NIPC2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NIPC2
# endif
# define machine_is_nipc2()	(machine_arch_type == MACH_TYPE_NIPC2)
#else
# define machine_is_nipc2()	(0)
#endif

#ifdef CONFIG_ARCH_FU7202
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FU7202
# endif
# define machine_is_fu7202()	(machine_arch_type == MACH_TYPE_FU7202)
#else
# define machine_is_fu7202()	(0)
#endif

#ifdef CONFIG_ARCH_ADSAGX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADSAGX
# endif
# define machine_is_adsagx()	(machine_arch_type == MACH_TYPE_ADSAGX)
#else
# define machine_is_adsagx()	(0)
#endif

#ifdef CONFIG_ARCH_PXA_POOH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_POOH
# endif
# define machine_is_pxa_pooh()	(machine_arch_type == MACH_TYPE_PXA_POOH)
#else
# define machine_is_pxa_pooh()	(0)
#endif

#ifdef CONFIG_ARCH_BANDON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BANDON
# endif
# define machine_is_bandon()	(machine_arch_type == MACH_TYPE_BANDON)
#else
# define machine_is_bandon()	(0)
#endif

#ifdef CONFIG_ARCH_PCM7210
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PCM7210
# endif
# define machine_is_pcm7210()	(machine_arch_type == MACH_TYPE_PCM7210)
#else
# define machine_is_pcm7210()	(0)
#endif

#ifdef CONFIG_ARCH_NMS9200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NMS9200
# endif
# define machine_is_nms9200()	(machine_arch_type == MACH_TYPE_NMS9200)
#else
# define machine_is_nms9200()	(0)
#endif

#ifdef CONFIG_ARCH_LOGODL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LOGODL
# endif
# define machine_is_logodl()	(machine_arch_type == MACH_TYPE_LOGODL)
#else
# define machine_is_logodl()	(0)
#endif

#ifdef CONFIG_SA1100_M7140
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_M7140
# endif
# define machine_is_m7140()	(machine_arch_type == MACH_TYPE_M7140)
#else
# define machine_is_m7140()	(0)
#endif

#ifdef CONFIG_ARCH_KOREBOT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KOREBOT
# endif
# define machine_is_korebot()	(machine_arch_type == MACH_TYPE_KOREBOT)
#else
# define machine_is_korebot()	(0)
#endif

#ifdef CONFIG_ARCH_IQ31244
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IQ31244
# endif
# define machine_is_iq31244()	(machine_arch_type == MACH_TYPE_IQ31244)
#else
# define machine_is_iq31244()	(0)
#endif

#ifdef CONFIG_SA1100_KOAN393
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KOAN393
# endif
# define machine_is_koan393()	(machine_arch_type == MACH_TYPE_KOAN393)
#else
# define machine_is_koan393()	(0)
#endif

#ifdef CONFIG_ARCH_INHANDFTIP3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_INHANDFTIP3
# endif
# define machine_is_inhandftip3()	(machine_arch_type == MACH_TYPE_INHANDFTIP3)
#else
# define machine_is_inhandftip3()	(0)
#endif

#ifdef CONFIG_ARCH_GONZO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GONZO
# endif
# define machine_is_gonzo()	(machine_arch_type == MACH_TYPE_GONZO)
#else
# define machine_is_gonzo()	(0)
#endif

#ifdef CONFIG_ARCH_BAST
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BAST
# endif
# define machine_is_bast()	(machine_arch_type == MACH_TYPE_BAST)
#else
# define machine_is_bast()	(0)
#endif

#ifdef CONFIG_ARCH_SCANPASS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SCANPASS
# endif
# define machine_is_scanpass()	(machine_arch_type == MACH_TYPE_SCANPASS)
#else
# define machine_is_scanpass()	(0)
#endif

#ifdef CONFIG_ARCH_EP7312_POOH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EP7312_POOH
# endif
# define machine_is_ep7312_pooh()	(machine_arch_type == MACH_TYPE_EP7312_POOH)
#else
# define machine_is_ep7312_pooh()	(0)
#endif

#ifdef CONFIG_ARCH_TA7S
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TA7S
# endif
# define machine_is_ta7s()	(machine_arch_type == MACH_TYPE_TA7S)
#else
# define machine_is_ta7s()	(0)
#endif

#ifdef CONFIG_ARCH_TA7V
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TA7V
# endif
# define machine_is_ta7v()	(machine_arch_type == MACH_TYPE_TA7V)
#else
# define machine_is_ta7v()	(0)
#endif

#ifdef CONFIG_SA1100_ICARUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ICARUS
# endif
# define machine_is_icarus()	(machine_arch_type == MACH_TYPE_ICARUS)
#else
# define machine_is_icarus()	(0)
#endif

#ifdef CONFIG_ARCH_H1900
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H1900
# endif
# define machine_is_h1900()	(machine_arch_type == MACH_TYPE_H1900)
#else
# define machine_is_h1900()	(0)
#endif

#ifdef CONFIG_SA1100_GEMINI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GEMINI
# endif
# define machine_is_gemini()	(machine_arch_type == MACH_TYPE_GEMINI)
#else
# define machine_is_gemini()	(0)
#endif

#ifdef CONFIG_ARCH_AXIM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AXIM
# endif
# define machine_is_axim()	(machine_arch_type == MACH_TYPE_AXIM)
#else
# define machine_is_axim()	(0)
#endif

#ifdef CONFIG_ARCH_AUDIOTRON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AUDIOTRON
# endif
# define machine_is_audiotron()	(machine_arch_type == MACH_TYPE_AUDIOTRON)
#else
# define machine_is_audiotron()	(0)
#endif

#ifdef CONFIG_ARCH_H2200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H2200
# endif
# define machine_is_h2200()	(machine_arch_type == MACH_TYPE_H2200)
#else
# define machine_is_h2200()	(0)
#endif

#ifdef CONFIG_ARCH_LOOX600
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LOOX600
# endif
# define machine_is_loox600()	(machine_arch_type == MACH_TYPE_LOOX600)
#else
# define machine_is_loox600()	(0)
#endif

#ifdef CONFIG_ARCH_NIOP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NIOP
# endif
# define machine_is_niop()	(machine_arch_type == MACH_TYPE_NIOP)
#else
# define machine_is_niop()	(0)
#endif

#ifdef CONFIG_ARCH_DM310
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DM310
# endif
# define machine_is_dm310()	(machine_arch_type == MACH_TYPE_DM310)
#else
# define machine_is_dm310()	(0)
#endif

#ifdef CONFIG_ARCH_SEEDPXA_C2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SEEDPXA_C2
# endif
# define machine_is_seedpxa_c2()	(machine_arch_type == MACH_TYPE_SEEDPXA_C2)
#else
# define machine_is_seedpxa_c2()	(0)
#endif

#ifdef CONFIG_ARCH_IXP4XX_MGUARD_PCI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXP4XX_MGUARD_PCI
# endif
# define machine_is_ixp4xx_mguardpci()	(machine_arch_type == MACH_TYPE_IXP4XX_MGUARD_PCI)
#else
# define machine_is_ixp4xx_mguardpci()	(0)
#endif

#ifdef CONFIG_ARCH_H1940
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H1940
# endif
# define machine_is_h1940()	(machine_arch_type == MACH_TYPE_H1940)
#else
# define machine_is_h1940()	(0)
#endif

#ifdef CONFIG_ARCH_SCORPIO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SCORPIO
# endif
# define machine_is_scorpio()	(machine_arch_type == MACH_TYPE_SCORPIO)
#else
# define machine_is_scorpio()	(0)
#endif

#ifdef CONFIG_ARCH_VIVA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VIVA
# endif
# define machine_is_viva()	(machine_arch_type == MACH_TYPE_VIVA)
#else
# define machine_is_viva()	(0)
#endif

#ifdef CONFIG_ARCH_PXA_XCARD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_XCARD
# endif
# define machine_is_pxa_xcard()	(machine_arch_type == MACH_TYPE_PXA_XCARD)
#else
# define machine_is_pxa_xcard()	(0)
#endif

#ifdef CONFIG_ARCH_CSB335
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CSB335
# endif
# define machine_is_csb335()	(machine_arch_type == MACH_TYPE_CSB335)
#else
# define machine_is_csb335()	(0)
#endif

#ifdef CONFIG_ARCH_IXRD425
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXRD425
# endif
# define machine_is_ixrd425()	(machine_arch_type == MACH_TYPE_IXRD425)
#else
# define machine_is_ixrd425()	(0)
#endif

#ifdef CONFIG_ARCH_IQ80315
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IQ80315
# endif
# define machine_is_iq80315()	(machine_arch_type == MACH_TYPE_IQ80315)
#else
# define machine_is_iq80315()	(0)
#endif

#ifdef CONFIG_ARCH_NMP7312
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NMP7312
# endif
# define machine_is_nmp7312()	(machine_arch_type == MACH_TYPE_NMP7312)
#else
# define machine_is_nmp7312()	(0)
#endif

#ifdef CONFIG_ARCH_CX861XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CX861XX
# endif
# define machine_is_cx861xx()	(machine_arch_type == MACH_TYPE_CX861XX)
#else
# define machine_is_cx861xx()	(0)
#endif

#ifdef CONFIG_ARCH_ENP2611
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ENP2611
# endif
# define machine_is_enp2611()	(machine_arch_type == MACH_TYPE_ENP2611)
#else
# define machine_is_enp2611()	(0)
#endif

#ifdef CONFIG_SA1100_XDA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XDA
# endif
# define machine_is_xda()	(machine_arch_type == MACH_TYPE_XDA)
#else
# define machine_is_xda()	(0)
#endif

#ifdef CONFIG_ARCH_CSIR_IMS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CSIR_IMS
# endif
# define machine_is_csir_ims()	(machine_arch_type == MACH_TYPE_CSIR_IMS)
#else
# define machine_is_csir_ims()	(0)
#endif

#ifdef CONFIG_ARCH_IXP421_DNAEETH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXP421_DNAEETH
# endif
# define machine_is_ixp421_dnaeeth()	(machine_arch_type == MACH_TYPE_IXP421_DNAEETH)
#else
# define machine_is_ixp421_dnaeeth()	(0)
#endif

#ifdef CONFIG_ARCH_POCKETSERV9200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_POCKETSERV9200
# endif
# define machine_is_pocketserv9200()	(machine_arch_type == MACH_TYPE_POCKETSERV9200)
#else
# define machine_is_pocketserv9200()	(0)
#endif

#ifdef CONFIG_ARCH_TOTO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TOTO
# endif
# define machine_is_toto()	(machine_arch_type == MACH_TYPE_TOTO)
#else
# define machine_is_toto()	(0)
#endif

#ifdef CONFIG_ARCH_S3C2440
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S3C2440
# endif
# define machine_is_s3c2440()	(machine_arch_type == MACH_TYPE_S3C2440)
#else
# define machine_is_s3c2440()	(0)
#endif

#ifdef CONFIG_ARCH_KS8695P
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KS8695P
# endif
# define machine_is_ks8695p()	(machine_arch_type == MACH_TYPE_KS8695P)
#else
# define machine_is_ks8695p()	(0)
#endif

#ifdef CONFIG_ARCH_SE4000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SE4000
# endif
# define machine_is_se4000()	(machine_arch_type == MACH_TYPE_SE4000)
#else
# define machine_is_se4000()	(0)
#endif

#ifdef CONFIG_ARCH_QUADRICEPS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_QUADRICEPS
# endif
# define machine_is_quadriceps()	(machine_arch_type == MACH_TYPE_QUADRICEPS)
#else
# define machine_is_quadriceps()	(0)
#endif

#ifdef CONFIG_ARCH_BRONCO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BRONCO
# endif
# define machine_is_bronco()	(machine_arch_type == MACH_TYPE_BRONCO)
#else
# define machine_is_bronco()	(0)
#endif

#ifdef CONFIG_ARCH_ESL_SOFCOMP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESL_SOFCOMP
# endif
# define machine_is_esl_sofcomp()	(machine_arch_type == MACH_TYPE_ESL_SOFCOMP)
#else
# define machine_is_esl_sofcomp()	(0)
#endif

#ifdef CONFIG_ARCH_S5C7375
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S5C7375
# endif
# define machine_is_s5c7375()	(machine_arch_type == MACH_TYPE_S5C7375)
#else
# define machine_is_s5c7375()	(0)
#endif

#ifdef CONFIG_ARCH_SPEARHEAD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPEARHEAD
# endif
# define machine_is_spearhead()	(machine_arch_type == MACH_TYPE_SPEARHEAD)
#else
# define machine_is_spearhead()	(0)
#endif

#ifdef CONFIG_ARCH_PANTERA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PANTERA
# endif
# define machine_is_pantera()	(machine_arch_type == MACH_TYPE_PANTERA)
#else
# define machine_is_pantera()	(0)
#endif

#ifdef CONFIG_ARCH_PRAYOGLITE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PRAYOGLITE
# endif
# define machine_is_prayoglite()	(machine_arch_type == MACH_TYPE_PRAYOGLITE)
#else
# define machine_is_prayoglite()	(0)
#endif

#ifdef CONFIG_ARCH_GUMSTIK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GUMSTIK
# endif
# define machine_is_gumstix()	(machine_arch_type == MACH_TYPE_GUMSTIK)
#else
# define machine_is_gumstix()	(0)
#endif

#ifdef CONFIG_ARCH_RCUBE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RCUBE
# endif
# define machine_is_rcube()	(machine_arch_type == MACH_TYPE_RCUBE)
#else
# define machine_is_rcube()	(0)
#endif

#ifdef CONFIG_ARCH_REA_OLV
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_REA_OLV
# endif
# define machine_is_rea_olv()	(machine_arch_type == MACH_TYPE_REA_OLV)
#else
# define machine_is_rea_olv()	(0)
#endif

#ifdef CONFIG_ARCH_PXA_IPHONE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_IPHONE
# endif
# define machine_is_pxa_iphone()	(machine_arch_type == MACH_TYPE_PXA_IPHONE)
#else
# define machine_is_pxa_iphone()	(0)
#endif

#ifdef CONFIG_ARCH_S3C3410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S3C3410
# endif
# define machine_is_s3c3410()	(machine_arch_type == MACH_TYPE_S3C3410)
#else
# define machine_is_s3c3410()	(0)
#endif

#ifdef CONFIG_ARCH_ESPD_4510B
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESPD_4510B
# endif
# define machine_is_espd_4510b()	(machine_arch_type == MACH_TYPE_ESPD_4510B)
#else
# define machine_is_espd_4510b()	(0)
#endif

#ifdef CONFIG_ARCH_MP1X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MP1X
# endif
# define machine_is_mp1x()	(machine_arch_type == MACH_TYPE_MP1X)
#else
# define machine_is_mp1x()	(0)
#endif

#ifdef CONFIG_ARCH_AT91RM9200TB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91RM9200TB
# endif
# define machine_is_at91rm9200tb()	(machine_arch_type == MACH_TYPE_AT91RM9200TB)
#else
# define machine_is_at91rm9200tb()	(0)
#endif

#ifdef CONFIG_ARCH_ADSVGX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADSVGX
# endif
# define machine_is_adsvgx()	(machine_arch_type == MACH_TYPE_ADSVGX)
#else
# define machine_is_adsvgx()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_H2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_H2
# endif
# define machine_is_omap_h2()	(machine_arch_type == MACH_TYPE_OMAP_H2)
#else
# define machine_is_omap_h2()	(0)
#endif

#ifdef CONFIG_ARCH_PELEE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PELEE
# endif
# define machine_is_pelee()	(machine_arch_type == MACH_TYPE_PELEE)
#else
# define machine_is_pelee()	(0)
#endif

#ifdef CONFIG_MACH_E740
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_E740
# endif
# define machine_is_e740()	(machine_arch_type == MACH_TYPE_E740)
#else
# define machine_is_e740()	(0)
#endif

#ifdef CONFIG_ARCH_IQ80331
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IQ80331
# endif
# define machine_is_iq80331()	(machine_arch_type == MACH_TYPE_IQ80331)
#else
# define machine_is_iq80331()	(0)
#endif

#ifdef CONFIG_ARCH_VERSATILE_PB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VERSATILE_PB
# endif
# define machine_is_versatile_pb()	(machine_arch_type == MACH_TYPE_VERSATILE_PB)
#else
# define machine_is_versatile_pb()	(0)
#endif

#ifdef CONFIG_MACH_KEV7A400
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KEV7A400
# endif
# define machine_is_kev7a400()	(machine_arch_type == MACH_TYPE_KEV7A400)
#else
# define machine_is_kev7a400()	(0)
#endif

#ifdef CONFIG_MACH_LPD7A400
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LPD7A400
# endif
# define machine_is_lpd7a400()	(machine_arch_type == MACH_TYPE_LPD7A400)
#else
# define machine_is_lpd7a400()	(0)
#endif

#ifdef CONFIG_MACH_LPD7A404
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LPD7A404
# endif
# define machine_is_lpd7a404()	(machine_arch_type == MACH_TYPE_LPD7A404)
#else
# define machine_is_lpd7a404()	(0)
#endif

#ifdef CONFIG_ARCH_FUJITSU_CAMELOT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FUJITSU_CAMELOT
# endif
# define machine_is_fujitsu_camelot()	(machine_arch_type == MACH_TYPE_FUJITSU_CAMELOT)
#else
# define machine_is_fujitsu_camelot()	(0)
#endif

#ifdef CONFIG_ARCH_JANUS2M
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_JANUS2M
# endif
# define machine_is_janus2m()	(machine_arch_type == MACH_TYPE_JANUS2M)
#else
# define machine_is_janus2m()	(0)
#endif

#ifdef CONFIG_MACH_EMBTF
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EMBTF
# endif
# define machine_is_embtf()	(machine_arch_type == MACH_TYPE_EMBTF)
#else
# define machine_is_embtf()	(0)
#endif

#ifdef CONFIG_MACH_HPM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HPM
# endif
# define machine_is_hpm()	(machine_arch_type == MACH_TYPE_HPM)
#else
# define machine_is_hpm()	(0)
#endif

#ifdef CONFIG_MACH_SMDK2410TK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK2410TK
# endif
# define machine_is_smdk2410tk()	(machine_arch_type == MACH_TYPE_SMDK2410TK)
#else
# define machine_is_smdk2410tk()	(0)
#endif

#ifdef CONFIG_MACH_SMDK2410AJ
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK2410AJ
# endif
# define machine_is_smdk2410aj()	(machine_arch_type == MACH_TYPE_SMDK2410AJ)
#else
# define machine_is_smdk2410aj()	(0)
#endif

#ifdef CONFIG_MACH_STREETRACER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STREETRACER
# endif
# define machine_is_streetracer()	(machine_arch_type == MACH_TYPE_STREETRACER)
#else
# define machine_is_streetracer()	(0)
#endif

#ifdef CONFIG_MACH_EFRAME
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EFRAME
# endif
# define machine_is_eframe()	(machine_arch_type == MACH_TYPE_EFRAME)
#else
# define machine_is_eframe()	(0)
#endif

#ifdef CONFIG_MACH_CSB337
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CSB337
# endif
# define machine_is_csb337()	(machine_arch_type == MACH_TYPE_CSB337)
#else
# define machine_is_csb337()	(0)
#endif

#ifdef CONFIG_MACH_PXA_LARK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_LARK
# endif
# define machine_is_pxa_lark()	(machine_arch_type == MACH_TYPE_PXA_LARK)
#else
# define machine_is_pxa_lark()	(0)
#endif

#ifdef CONFIG_MACH_PNP2110
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PNP2110
# endif
# define machine_is_pxa_pnp2110()	(machine_arch_type == MACH_TYPE_PNP2110)
#else
# define machine_is_pxa_pnp2110()	(0)
#endif

#ifdef CONFIG_MACH_TCC72X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TCC72X
# endif
# define machine_is_tcc72x()	(machine_arch_type == MACH_TYPE_TCC72X)
#else
# define machine_is_tcc72x()	(0)
#endif

#ifdef CONFIG_MACH_ALTAIR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ALTAIR
# endif
# define machine_is_altair()	(machine_arch_type == MACH_TYPE_ALTAIR)
#else
# define machine_is_altair()	(0)
#endif

#ifdef CONFIG_MACH_KC3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KC3
# endif
# define machine_is_kc3()	(machine_arch_type == MACH_TYPE_KC3)
#else
# define machine_is_kc3()	(0)
#endif

#ifdef CONFIG_MACH_SINTEFTD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SINTEFTD
# endif
# define machine_is_sinteftd()	(machine_arch_type == MACH_TYPE_SINTEFTD)
#else
# define machine_is_sinteftd()	(0)
#endif

#ifdef CONFIG_MACH_MAINSTONE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MAINSTONE
# endif
# define machine_is_mainstone()	(machine_arch_type == MACH_TYPE_MAINSTONE)
#else
# define machine_is_mainstone()	(0)
#endif

#ifdef CONFIG_MACH_ADAY4X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADAY4X
# endif
# define machine_is_aday4x()	(machine_arch_type == MACH_TYPE_ADAY4X)
#else
# define machine_is_aday4x()	(0)
#endif

#ifdef CONFIG_MACH_LITE300
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LITE300
# endif
# define machine_is_lite300()	(machine_arch_type == MACH_TYPE_LITE300)
#else
# define machine_is_lite300()	(0)
#endif

#ifdef CONFIG_MACH_S5C7376
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S5C7376
# endif
# define machine_is_s5c7376()	(machine_arch_type == MACH_TYPE_S5C7376)
#else
# define machine_is_s5c7376()	(0)
#endif

#ifdef CONFIG_MACH_MT02
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MT02
# endif
# define machine_is_mt02()	(machine_arch_type == MACH_TYPE_MT02)
#else
# define machine_is_mt02()	(0)
#endif

#ifdef CONFIG_MACH_MPORT3S
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MPORT3S
# endif
# define machine_is_mport3s()	(machine_arch_type == MACH_TYPE_MPORT3S)
#else
# define machine_is_mport3s()	(0)
#endif

#ifdef CONFIG_MACH_RA_ALPHA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RA_ALPHA
# endif
# define machine_is_ra_alpha()	(machine_arch_type == MACH_TYPE_RA_ALPHA)
#else
# define machine_is_ra_alpha()	(0)
#endif

#ifdef CONFIG_MACH_XCEP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XCEP
# endif
# define machine_is_xcep()	(machine_arch_type == MACH_TYPE_XCEP)
#else
# define machine_is_xcep()	(0)
#endif

#ifdef CONFIG_MACH_ARCOM_VULCAN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARCOM_VULCAN
# endif
# define machine_is_arcom_vulcan()	(machine_arch_type == MACH_TYPE_ARCOM_VULCAN)
#else
# define machine_is_arcom_vulcan()	(0)
#endif

#ifdef CONFIG_MACH_STARGATE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STARGATE
# endif
# define machine_is_stargate()	(machine_arch_type == MACH_TYPE_STARGATE)
#else
# define machine_is_stargate()	(0)
#endif

#ifdef CONFIG_MACH_ARMADILLOJ
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARMADILLOJ
# endif
# define machine_is_armadilloj()	(machine_arch_type == MACH_TYPE_ARMADILLOJ)
#else
# define machine_is_armadilloj()	(0)
#endif

#ifdef CONFIG_MACH_ELROY_JACK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ELROY_JACK
# endif
# define machine_is_elroy_jack()	(machine_arch_type == MACH_TYPE_ELROY_JACK)
#else
# define machine_is_elroy_jack()	(0)
#endif

#ifdef CONFIG_MACH_BACKEND
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BACKEND
# endif
# define machine_is_backend()	(machine_arch_type == MACH_TYPE_BACKEND)
#else
# define machine_is_backend()	(0)
#endif

#ifdef CONFIG_MACH_S5LINBOX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S5LINBOX
# endif
# define machine_is_s5linbox()	(machine_arch_type == MACH_TYPE_S5LINBOX)
#else
# define machine_is_s5linbox()	(0)
#endif

#ifdef CONFIG_MACH_NOMADIK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NOMADIK
# endif
# define machine_is_nomadik()	(machine_arch_type == MACH_TYPE_NOMADIK)
#else
# define machine_is_nomadik()	(0)
#endif

#ifdef CONFIG_MACH_IA_CPU_9200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IA_CPU_9200
# endif
# define machine_is_ia_cpu_9200()	(machine_arch_type == MACH_TYPE_IA_CPU_9200)
#else
# define machine_is_ia_cpu_9200()	(0)
#endif

#ifdef CONFIG_MACH_AT91_BJA1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91_BJA1
# endif
# define machine_is_at91_bja1()	(machine_arch_type == MACH_TYPE_AT91_BJA1)
#else
# define machine_is_at91_bja1()	(0)
#endif

#ifdef CONFIG_MACH_CORGI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CORGI
# endif
# define machine_is_corgi()	(machine_arch_type == MACH_TYPE_CORGI)
#else
# define machine_is_corgi()	(0)
#endif

#ifdef CONFIG_MACH_POODLE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_POODLE
# endif
# define machine_is_poodle()	(machine_arch_type == MACH_TYPE_POODLE)
#else
# define machine_is_poodle()	(0)
#endif

#ifdef CONFIG_MACH_TEN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TEN
# endif
# define machine_is_ten()	(machine_arch_type == MACH_TYPE_TEN)
#else
# define machine_is_ten()	(0)
#endif

#ifdef CONFIG_MACH_ROVERP5P
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ROVERP5P
# endif
# define machine_is_roverp5p()	(machine_arch_type == MACH_TYPE_ROVERP5P)
#else
# define machine_is_roverp5p()	(0)
#endif

#ifdef CONFIG_MACH_SC2700
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SC2700
# endif
# define machine_is_sc2700()	(machine_arch_type == MACH_TYPE_SC2700)
#else
# define machine_is_sc2700()	(0)
#endif

#ifdef CONFIG_MACH_EX_EAGLE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EX_EAGLE
# endif
# define machine_is_ex_eagle()	(machine_arch_type == MACH_TYPE_EX_EAGLE)
#else
# define machine_is_ex_eagle()	(0)
#endif

#ifdef CONFIG_MACH_NX_PXA12
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NX_PXA12
# endif
# define machine_is_nx_pxa12()	(machine_arch_type == MACH_TYPE_NX_PXA12)
#else
# define machine_is_nx_pxa12()	(0)
#endif

#ifdef CONFIG_MACH_NX_PXA5
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NX_PXA5
# endif
# define machine_is_nx_pxa5()	(machine_arch_type == MACH_TYPE_NX_PXA5)
#else
# define machine_is_nx_pxa5()	(0)
#endif

#ifdef CONFIG_MACH_BLACKBOARD2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BLACKBOARD2
# endif
# define machine_is_blackboard2()	(machine_arch_type == MACH_TYPE_BLACKBOARD2)
#else
# define machine_is_blackboard2()	(0)
#endif

#ifdef CONFIG_MACH_I819
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_I819
# endif
# define machine_is_i819()	(machine_arch_type == MACH_TYPE_I819)
#else
# define machine_is_i819()	(0)
#endif

#ifdef CONFIG_MACH_IXMB995E
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXMB995E
# endif
# define machine_is_ixmb995e()	(machine_arch_type == MACH_TYPE_IXMB995E)
#else
# define machine_is_ixmb995e()	(0)
#endif

#ifdef CONFIG_MACH_SKYRIDER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SKYRIDER
# endif
# define machine_is_skyrider()	(machine_arch_type == MACH_TYPE_SKYRIDER)
#else
# define machine_is_skyrider()	(0)
#endif

#ifdef CONFIG_MACH_SKYHAWK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SKYHAWK
# endif
# define machine_is_skyhawk()	(machine_arch_type == MACH_TYPE_SKYHAWK)
#else
# define machine_is_skyhawk()	(0)
#endif

#ifdef CONFIG_MACH_ENTERPRISE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ENTERPRISE
# endif
# define machine_is_enterprise()	(machine_arch_type == MACH_TYPE_ENTERPRISE)
#else
# define machine_is_enterprise()	(0)
#endif

#ifdef CONFIG_MACH_DEP2410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DEP2410
# endif
# define machine_is_dep2410()	(machine_arch_type == MACH_TYPE_DEP2410)
#else
# define machine_is_dep2410()	(0)
#endif

#ifdef CONFIG_MACH_ARMCORE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARMCORE
# endif
# define machine_is_armcore()	(machine_arch_type == MACH_TYPE_ARMCORE)
#else
# define machine_is_armcore()	(0)
#endif

#ifdef CONFIG_MACH_HOBBIT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HOBBIT
# endif
# define machine_is_hobbit()	(machine_arch_type == MACH_TYPE_HOBBIT)
#else
# define machine_is_hobbit()	(0)
#endif

#ifdef CONFIG_MACH_H7210
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H7210
# endif
# define machine_is_h7210()	(machine_arch_type == MACH_TYPE_H7210)
#else
# define machine_is_h7210()	(0)
#endif

#ifdef CONFIG_MACH_PXA_NETDCU5
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_NETDCU5
# endif
# define machine_is_pxa_netdcu5()	(machine_arch_type == MACH_TYPE_PXA_NETDCU5)
#else
# define machine_is_pxa_netdcu5()	(0)
#endif

#ifdef CONFIG_MACH_ACC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACC
# endif
# define machine_is_acc()	(machine_arch_type == MACH_TYPE_ACC)
#else
# define machine_is_acc()	(0)
#endif

#ifdef CONFIG_MACH_ESL_SARVA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESL_SARVA
# endif
# define machine_is_esl_sarva()	(machine_arch_type == MACH_TYPE_ESL_SARVA)
#else
# define machine_is_esl_sarva()	(0)
#endif

#ifdef CONFIG_MACH_XM250
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XM250
# endif
# define machine_is_xm250()	(machine_arch_type == MACH_TYPE_XM250)
#else
# define machine_is_xm250()	(0)
#endif

#ifdef CONFIG_MACH_T6TC1XB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_T6TC1XB
# endif
# define machine_is_t6tc1xb()	(machine_arch_type == MACH_TYPE_T6TC1XB)
#else
# define machine_is_t6tc1xb()	(0)
#endif

#ifdef CONFIG_MACH_ESS710
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESS710
# endif
# define machine_is_ess710()	(machine_arch_type == MACH_TYPE_ESS710)
#else
# define machine_is_ess710()	(0)
#endif

#ifdef CONFIG_MACH_MX31ADS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX31ADS
# endif
# define machine_is_mx31ads()	(machine_arch_type == MACH_TYPE_MX31ADS)
#else
# define machine_is_mx31ads()	(0)
#endif

#ifdef CONFIG_MACH_HIMALAYA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HIMALAYA
# endif
# define machine_is_himalaya()	(machine_arch_type == MACH_TYPE_HIMALAYA)
#else
# define machine_is_himalaya()	(0)
#endif

#ifdef CONFIG_MACH_BOLFENK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BOLFENK
# endif
# define machine_is_bolfenk()	(machine_arch_type == MACH_TYPE_BOLFENK)
#else
# define machine_is_bolfenk()	(0)
#endif

#ifdef CONFIG_MACH_AT91RM9200KR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91RM9200KR
# endif
# define machine_is_at91rm9200kr()	(machine_arch_type == MACH_TYPE_AT91RM9200KR)
#else
# define machine_is_at91rm9200kr()	(0)
#endif

#ifdef CONFIG_MACH_EDB9312
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB9312
# endif
# define machine_is_edb9312()	(machine_arch_type == MACH_TYPE_EDB9312)
#else
# define machine_is_edb9312()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_GENERIC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_GENERIC
# endif
# define machine_is_omap_generic()	(machine_arch_type == MACH_TYPE_OMAP_GENERIC)
#else
# define machine_is_omap_generic()	(0)
#endif

#ifdef CONFIG_MACH_AXIMX3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AXIMX3
# endif
# define machine_is_aximx3()	(machine_arch_type == MACH_TYPE_AXIMX3)
#else
# define machine_is_aximx3()	(0)
#endif

#ifdef CONFIG_MACH_EB67XDIP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EB67XDIP
# endif
# define machine_is_eb67xdip()	(machine_arch_type == MACH_TYPE_EB67XDIP)
#else
# define machine_is_eb67xdip()	(0)
#endif

#ifdef CONFIG_MACH_WEBTXS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WEBTXS
# endif
# define machine_is_webtxs()	(machine_arch_type == MACH_TYPE_WEBTXS)
#else
# define machine_is_webtxs()	(0)
#endif

#ifdef CONFIG_MACH_HAWK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HAWK
# endif
# define machine_is_hawk()	(machine_arch_type == MACH_TYPE_HAWK)
#else
# define machine_is_hawk()	(0)
#endif

#ifdef CONFIG_MACH_CCAT91SBC001
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CCAT91SBC001
# endif
# define machine_is_ccat91sbc001()	(machine_arch_type == MACH_TYPE_CCAT91SBC001)
#else
# define machine_is_ccat91sbc001()	(0)
#endif

#ifdef CONFIG_MACH_EXPRESSO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EXPRESSO
# endif
# define machine_is_expresso()	(machine_arch_type == MACH_TYPE_EXPRESSO)
#else
# define machine_is_expresso()	(0)
#endif

#ifdef CONFIG_MACH_H4000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H4000
# endif
# define machine_is_h4000()	(machine_arch_type == MACH_TYPE_H4000)
#else
# define machine_is_h4000()	(0)
#endif

#ifdef CONFIG_MACH_DINO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DINO
# endif
# define machine_is_dino()	(machine_arch_type == MACH_TYPE_DINO)
#else
# define machine_is_dino()	(0)
#endif

#ifdef CONFIG_MACH_ML675K
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ML675K
# endif
# define machine_is_ml675k()	(machine_arch_type == MACH_TYPE_ML675K)
#else
# define machine_is_ml675k()	(0)
#endif

#ifdef CONFIG_MACH_EDB9301
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB9301
# endif
# define machine_is_edb9301()	(machine_arch_type == MACH_TYPE_EDB9301)
#else
# define machine_is_edb9301()	(0)
#endif

#ifdef CONFIG_MACH_EDB9315
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB9315
# endif
# define machine_is_edb9315()	(machine_arch_type == MACH_TYPE_EDB9315)
#else
# define machine_is_edb9315()	(0)
#endif

#ifdef CONFIG_MACH_RECIVA_TT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RECIVA_TT
# endif
# define machine_is_reciva_tt()	(machine_arch_type == MACH_TYPE_RECIVA_TT)
#else
# define machine_is_reciva_tt()	(0)
#endif

#ifdef CONFIG_MACH_CSTCB01
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CSTCB01
# endif
# define machine_is_cstcb01()	(machine_arch_type == MACH_TYPE_CSTCB01)
#else
# define machine_is_cstcb01()	(0)
#endif

#ifdef CONFIG_MACH_CSTCB1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CSTCB1
# endif
# define machine_is_cstcb1()	(machine_arch_type == MACH_TYPE_CSTCB1)
#else
# define machine_is_cstcb1()	(0)
#endif

#ifdef CONFIG_MACH_SHADWELL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SHADWELL
# endif
# define machine_is_shadwell()	(machine_arch_type == MACH_TYPE_SHADWELL)
#else
# define machine_is_shadwell()	(0)
#endif

#ifdef CONFIG_MACH_GOEPEL263
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GOEPEL263
# endif
# define machine_is_goepel263()	(machine_arch_type == MACH_TYPE_GOEPEL263)
#else
# define machine_is_goepel263()	(0)
#endif

#ifdef CONFIG_MACH_ACQ100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACQ100
# endif
# define machine_is_acq100()	(machine_arch_type == MACH_TYPE_ACQ100)
#else
# define machine_is_acq100()	(0)
#endif

#ifdef CONFIG_MACH_MX1FS2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX1FS2
# endif
# define machine_is_mx1fs2()	(machine_arch_type == MACH_TYPE_MX1FS2)
#else
# define machine_is_mx1fs2()	(0)
#endif

#ifdef CONFIG_MACH_HIPTOP_G1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HIPTOP_G1
# endif
# define machine_is_hiptop_g1()	(machine_arch_type == MACH_TYPE_HIPTOP_G1)
#else
# define machine_is_hiptop_g1()	(0)
#endif

#ifdef CONFIG_MACH_SPARKY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPARKY
# endif
# define machine_is_sparky()	(machine_arch_type == MACH_TYPE_SPARKY)
#else
# define machine_is_sparky()	(0)
#endif

#ifdef CONFIG_MACH_NS9750
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NS9750
# endif
# define machine_is_ns9750()	(machine_arch_type == MACH_TYPE_NS9750)
#else
# define machine_is_ns9750()	(0)
#endif

#ifdef CONFIG_MACH_PHOENIX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PHOENIX
# endif
# define machine_is_phoenix()	(machine_arch_type == MACH_TYPE_PHOENIX)
#else
# define machine_is_phoenix()	(0)
#endif

#ifdef CONFIG_MACH_VR1000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VR1000
# endif
# define machine_is_vr1000()	(machine_arch_type == MACH_TYPE_VR1000)
#else
# define machine_is_vr1000()	(0)
#endif

#ifdef CONFIG_MACH_DEISTERPXA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DEISTERPXA
# endif
# define machine_is_deisterpxa()	(machine_arch_type == MACH_TYPE_DEISTERPXA)
#else
# define machine_is_deisterpxa()	(0)
#endif

#ifdef CONFIG_MACH_BCM1160
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BCM1160
# endif
# define machine_is_bcm1160()	(machine_arch_type == MACH_TYPE_BCM1160)
#else
# define machine_is_bcm1160()	(0)
#endif

#ifdef CONFIG_MACH_PCM022
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PCM022
# endif
# define machine_is_pcm022()	(machine_arch_type == MACH_TYPE_PCM022)
#else
# define machine_is_pcm022()	(0)
#endif

#ifdef CONFIG_MACH_ADSGCX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADSGCX
# endif
# define machine_is_adsgcx()	(machine_arch_type == MACH_TYPE_ADSGCX)
#else
# define machine_is_adsgcx()	(0)
#endif

#ifdef CONFIG_MACH_DREADNAUGHT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DREADNAUGHT
# endif
# define machine_is_dreadnaught()	(machine_arch_type == MACH_TYPE_DREADNAUGHT)
#else
# define machine_is_dreadnaught()	(0)
#endif

#ifdef CONFIG_MACH_DM320
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DM320
# endif
# define machine_is_dm320()	(machine_arch_type == MACH_TYPE_DM320)
#else
# define machine_is_dm320()	(0)
#endif

#ifdef CONFIG_MACH_MARKOV
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MARKOV
# endif
# define machine_is_markov()	(machine_arch_type == MACH_TYPE_MARKOV)
#else
# define machine_is_markov()	(0)
#endif

#ifdef CONFIG_MACH_COS7A400
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COS7A400
# endif
# define machine_is_cos7a400()	(machine_arch_type == MACH_TYPE_COS7A400)
#else
# define machine_is_cos7a400()	(0)
#endif

#ifdef CONFIG_MACH_MILANO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MILANO
# endif
# define machine_is_milano()	(machine_arch_type == MACH_TYPE_MILANO)
#else
# define machine_is_milano()	(0)
#endif

#ifdef CONFIG_MACH_UE9328
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_UE9328
# endif
# define machine_is_ue9328()	(machine_arch_type == MACH_TYPE_UE9328)
#else
# define machine_is_ue9328()	(0)
#endif

#ifdef CONFIG_MACH_UEX255
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_UEX255
# endif
# define machine_is_uex255()	(machine_arch_type == MACH_TYPE_UEX255)
#else
# define machine_is_uex255()	(0)
#endif

#ifdef CONFIG_MACH_UE2410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_UE2410
# endif
# define machine_is_ue2410()	(machine_arch_type == MACH_TYPE_UE2410)
#else
# define machine_is_ue2410()	(0)
#endif

#ifdef CONFIG_MACH_A620
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_A620
# endif
# define machine_is_a620()	(machine_arch_type == MACH_TYPE_A620)
#else
# define machine_is_a620()	(0)
#endif

#ifdef CONFIG_MACH_OCELOT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OCELOT
# endif
# define machine_is_ocelot()	(machine_arch_type == MACH_TYPE_OCELOT)
#else
# define machine_is_ocelot()	(0)
#endif

#ifdef CONFIG_MACH_CHEETAH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CHEETAH
# endif
# define machine_is_cheetah()	(machine_arch_type == MACH_TYPE_CHEETAH)
#else
# define machine_is_cheetah()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_PERSEUS2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_PERSEUS2
# endif
# define machine_is_omap_perseus2()	(machine_arch_type == MACH_TYPE_OMAP_PERSEUS2)
#else
# define machine_is_omap_perseus2()	(0)
#endif

#ifdef CONFIG_MACH_ZVUE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ZVUE
# endif
# define machine_is_zvue()	(machine_arch_type == MACH_TYPE_ZVUE)
#else
# define machine_is_zvue()	(0)
#endif

#ifdef CONFIG_MACH_ROVERP1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ROVERP1
# endif
# define machine_is_roverp1()	(machine_arch_type == MACH_TYPE_ROVERP1)
#else
# define machine_is_roverp1()	(0)
#endif

#ifdef CONFIG_MACH_ASIDIAL2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ASIDIAL2
# endif
# define machine_is_asidial2()	(machine_arch_type == MACH_TYPE_ASIDIAL2)
#else
# define machine_is_asidial2()	(0)
#endif

#ifdef CONFIG_MACH_S3C24A0
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S3C24A0
# endif
# define machine_is_s3c24a0()	(machine_arch_type == MACH_TYPE_S3C24A0)
#else
# define machine_is_s3c24a0()	(0)
#endif

#ifdef CONFIG_MACH_E800
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_E800
# endif
# define machine_is_e800()	(machine_arch_type == MACH_TYPE_E800)
#else
# define machine_is_e800()	(0)
#endif

#ifdef CONFIG_MACH_E750
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_E750
# endif
# define machine_is_e750()	(machine_arch_type == MACH_TYPE_E750)
#else
# define machine_is_e750()	(0)
#endif

#ifdef CONFIG_MACH_S3C5500
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S3C5500
# endif
# define machine_is_s3c5500()	(machine_arch_type == MACH_TYPE_S3C5500)
#else
# define machine_is_s3c5500()	(0)
#endif

#ifdef CONFIG_MACH_SMDK5500
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK5500
# endif
# define machine_is_smdk5500()	(machine_arch_type == MACH_TYPE_SMDK5500)
#else
# define machine_is_smdk5500()	(0)
#endif

#ifdef CONFIG_MACH_SIGNALSYNC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SIGNALSYNC
# endif
# define machine_is_signalsync()	(machine_arch_type == MACH_TYPE_SIGNALSYNC)
#else
# define machine_is_signalsync()	(0)
#endif

#ifdef CONFIG_MACH_NBC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NBC
# endif
# define machine_is_nbc()	(machine_arch_type == MACH_TYPE_NBC)
#else
# define machine_is_nbc()	(0)
#endif

#ifdef CONFIG_MACH_KODIAK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KODIAK
# endif
# define machine_is_kodiak()	(machine_arch_type == MACH_TYPE_KODIAK)
#else
# define machine_is_kodiak()	(0)
#endif

#ifdef CONFIG_MACH_NETBOOKPRO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NETBOOKPRO
# endif
# define machine_is_netbookpro()	(machine_arch_type == MACH_TYPE_NETBOOKPRO)
#else
# define machine_is_netbookpro()	(0)
#endif

#ifdef CONFIG_MACH_HW90200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HW90200
# endif
# define machine_is_hw90200()	(machine_arch_type == MACH_TYPE_HW90200)
#else
# define machine_is_hw90200()	(0)
#endif

#ifdef CONFIG_MACH_CONDOR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CONDOR
# endif
# define machine_is_condor()	(machine_arch_type == MACH_TYPE_CONDOR)
#else
# define machine_is_condor()	(0)
#endif

#ifdef CONFIG_MACH_CUP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CUP
# endif
# define machine_is_cup()	(machine_arch_type == MACH_TYPE_CUP)
#else
# define machine_is_cup()	(0)
#endif

#ifdef CONFIG_MACH_KITE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KITE
# endif
# define machine_is_kite()	(machine_arch_type == MACH_TYPE_KITE)
#else
# define machine_is_kite()	(0)
#endif

#ifdef CONFIG_MACH_SCB9328
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SCB9328
# endif
# define machine_is_scb9328()	(machine_arch_type == MACH_TYPE_SCB9328)
#else
# define machine_is_scb9328()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_H3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_H3
# endif
# define machine_is_omap_h3()	(machine_arch_type == MACH_TYPE_OMAP_H3)
#else
# define machine_is_omap_h3()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_H4
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_H4
# endif
# define machine_is_omap_h4()	(machine_arch_type == MACH_TYPE_OMAP_H4)
#else
# define machine_is_omap_h4()	(0)
#endif

#ifdef CONFIG_MACH_N10
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_N10
# endif
# define machine_is_n10()	(machine_arch_type == MACH_TYPE_N10)
#else
# define machine_is_n10()	(0)
#endif

#ifdef CONFIG_MACH_MONTAJADE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MONTAJADE
# endif
# define machine_is_montejade()	(machine_arch_type == MACH_TYPE_MONTAJADE)
#else
# define machine_is_montejade()	(0)
#endif

#ifdef CONFIG_MACH_SG560
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SG560
# endif
# define machine_is_sg560()	(machine_arch_type == MACH_TYPE_SG560)
#else
# define machine_is_sg560()	(0)
#endif

#ifdef CONFIG_MACH_DP1000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DP1000
# endif
# define machine_is_dp1000()	(machine_arch_type == MACH_TYPE_DP1000)
#else
# define machine_is_dp1000()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_OSK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_OSK
# endif
# define machine_is_omap_osk()	(machine_arch_type == MACH_TYPE_OMAP_OSK)
#else
# define machine_is_omap_osk()	(0)
#endif

#ifdef CONFIG_MACH_RG100V3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RG100V3
# endif
# define machine_is_rg100v3()	(machine_arch_type == MACH_TYPE_RG100V3)
#else
# define machine_is_rg100v3()	(0)
#endif

#ifdef CONFIG_MACH_MX2ADS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX2ADS
# endif
# define machine_is_mx2ads()	(machine_arch_type == MACH_TYPE_MX2ADS)
#else
# define machine_is_mx2ads()	(0)
#endif

#ifdef CONFIG_MACH_PXA_KILO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_KILO
# endif
# define machine_is_pxa_kilo()	(machine_arch_type == MACH_TYPE_PXA_KILO)
#else
# define machine_is_pxa_kilo()	(0)
#endif

#ifdef CONFIG_MACH_IXP4XX_EAGLE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXP4XX_EAGLE
# endif
# define machine_is_ixp4xx_eagle()	(machine_arch_type == MACH_TYPE_IXP4XX_EAGLE)
#else
# define machine_is_ixp4xx_eagle()	(0)
#endif

#ifdef CONFIG_MACH_TOSA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TOSA
# endif
# define machine_is_tosa()	(machine_arch_type == MACH_TYPE_TOSA)
#else
# define machine_is_tosa()	(0)
#endif

#ifdef CONFIG_MACH_MB2520F
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MB2520F
# endif
# define machine_is_mb2520f()	(machine_arch_type == MACH_TYPE_MB2520F)
#else
# define machine_is_mb2520f()	(0)
#endif

#ifdef CONFIG_MACH_EMC1000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EMC1000
# endif
# define machine_is_emc1000()	(machine_arch_type == MACH_TYPE_EMC1000)
#else
# define machine_is_emc1000()	(0)
#endif

#ifdef CONFIG_MACH_TIDSC25
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TIDSC25
# endif
# define machine_is_tidsc25()	(machine_arch_type == MACH_TYPE_TIDSC25)
#else
# define machine_is_tidsc25()	(0)
#endif

#ifdef CONFIG_MACH_AKCPMXL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AKCPMXL
# endif
# define machine_is_akcpmxl()	(machine_arch_type == MACH_TYPE_AKCPMXL)
#else
# define machine_is_akcpmxl()	(0)
#endif

#ifdef CONFIG_MACH_AV3XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AV3XX
# endif
# define machine_is_av3xx()	(machine_arch_type == MACH_TYPE_AV3XX)
#else
# define machine_is_av3xx()	(0)
#endif

#ifdef CONFIG_MACH_AVILA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AVILA
# endif
# define machine_is_avila()	(machine_arch_type == MACH_TYPE_AVILA)
#else
# define machine_is_avila()	(0)
#endif

#ifdef CONFIG_MACH_PXA_MPM10
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_MPM10
# endif
# define machine_is_pxa_mpm10()	(machine_arch_type == MACH_TYPE_PXA_MPM10)
#else
# define machine_is_pxa_mpm10()	(0)
#endif

#ifdef CONFIG_MACH_PXA_KYANITE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_KYANITE
# endif
# define machine_is_pxa_kyanite()	(machine_arch_type == MACH_TYPE_PXA_KYANITE)
#else
# define machine_is_pxa_kyanite()	(0)
#endif

#ifdef CONFIG_MACH_SGOLD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SGOLD
# endif
# define machine_is_sgold()	(machine_arch_type == MACH_TYPE_SGOLD)
#else
# define machine_is_sgold()	(0)
#endif

#ifdef CONFIG_MACH_OSCAR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OSCAR
# endif
# define machine_is_oscar()	(machine_arch_type == MACH_TYPE_OSCAR)
#else
# define machine_is_oscar()	(0)
#endif

#ifdef CONFIG_MACH_EPXA4USB2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EPXA4USB2
# endif
# define machine_is_epxa4usb2()	(machine_arch_type == MACH_TYPE_EPXA4USB2)
#else
# define machine_is_epxa4usb2()	(0)
#endif

#ifdef CONFIG_MACH_XSENGINE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XSENGINE
# endif
# define machine_is_xsengine()	(machine_arch_type == MACH_TYPE_XSENGINE)
#else
# define machine_is_xsengine()	(0)
#endif

#ifdef CONFIG_MACH_IP600
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IP600
# endif
# define machine_is_ip600()	(machine_arch_type == MACH_TYPE_IP600)
#else
# define machine_is_ip600()	(0)
#endif

#ifdef CONFIG_MACH_MCAN2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MCAN2
# endif
# define machine_is_mcan2()	(machine_arch_type == MACH_TYPE_MCAN2)
#else
# define machine_is_mcan2()	(0)
#endif

#ifdef CONFIG_MACH_DDI_BLUERIDGE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DDI_BLUERIDGE
# endif
# define machine_is_ddi_blueridge()	(machine_arch_type == MACH_TYPE_DDI_BLUERIDGE)
#else
# define machine_is_ddi_blueridge()	(0)
#endif

#ifdef CONFIG_MACH_SKYMINDER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SKYMINDER
# endif
# define machine_is_skyminder()	(machine_arch_type == MACH_TYPE_SKYMINDER)
#else
# define machine_is_skyminder()	(0)
#endif

#ifdef CONFIG_MACH_LPD79520
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LPD79520
# endif
# define machine_is_lpd79520()	(machine_arch_type == MACH_TYPE_LPD79520)
#else
# define machine_is_lpd79520()	(0)
#endif

#ifdef CONFIG_MACH_EDB9302
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB9302
# endif
# define machine_is_edb9302()	(machine_arch_type == MACH_TYPE_EDB9302)
#else
# define machine_is_edb9302()	(0)
#endif

#ifdef CONFIG_MACH_HW90340
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HW90340
# endif
# define machine_is_hw90340()	(machine_arch_type == MACH_TYPE_HW90340)
#else
# define machine_is_hw90340()	(0)
#endif

#ifdef CONFIG_MACH_CIP_BOX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CIP_BOX
# endif
# define machine_is_cip_box()	(machine_arch_type == MACH_TYPE_CIP_BOX)
#else
# define machine_is_cip_box()	(0)
#endif

#ifdef CONFIG_MACH_IVPN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IVPN
# endif
# define machine_is_ivpn()	(machine_arch_type == MACH_TYPE_IVPN)
#else
# define machine_is_ivpn()	(0)
#endif

#ifdef CONFIG_MACH_RSOC2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RSOC2
# endif
# define machine_is_rsoc2()	(machine_arch_type == MACH_TYPE_RSOC2)
#else
# define machine_is_rsoc2()	(0)
#endif

#ifdef CONFIG_MACH_HUSKY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HUSKY
# endif
# define machine_is_husky()	(machine_arch_type == MACH_TYPE_HUSKY)
#else
# define machine_is_husky()	(0)
#endif

#ifdef CONFIG_MACH_BOXER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BOXER
# endif
# define machine_is_boxer()	(machine_arch_type == MACH_TYPE_BOXER)
#else
# define machine_is_boxer()	(0)
#endif

#ifdef CONFIG_MACH_SHEPHERD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SHEPHERD
# endif
# define machine_is_shepherd()	(machine_arch_type == MACH_TYPE_SHEPHERD)
#else
# define machine_is_shepherd()	(0)
#endif

#ifdef CONFIG_MACH_AML42800AA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AML42800AA
# endif
# define machine_is_aml42800aa()	(machine_arch_type == MACH_TYPE_AML42800AA)
#else
# define machine_is_aml42800aa()	(0)
#endif

#ifdef CONFIG_MACH_MACH_TYPE_ML674001
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MACH_TYPE_ML674001
# endif
# define machine_is_ml674001()	(machine_arch_type == MACH_TYPE_MACH_TYPE_ML674001)
#else
# define machine_is_ml674001()	(0)
#endif

#ifdef CONFIG_MACH_LPC2294
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LPC2294
# endif
# define machine_is_lpc2294()	(machine_arch_type == MACH_TYPE_LPC2294)
#else
# define machine_is_lpc2294()	(0)
#endif

#ifdef CONFIG_MACH_SWITCHGRASS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SWITCHGRASS
# endif
# define machine_is_switchgrass()	(machine_arch_type == MACH_TYPE_SWITCHGRASS)
#else
# define machine_is_switchgrass()	(0)
#endif

#ifdef CONFIG_MACH_ENS_CMU
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ENS_CMU
# endif
# define machine_is_ens_cmu()	(machine_arch_type == MACH_TYPE_ENS_CMU)
#else
# define machine_is_ens_cmu()	(0)
#endif

#ifdef CONFIG_MACH_MM6_SDB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MM6_SDB
# endif
# define machine_is_mm6_sdb()	(machine_arch_type == MACH_TYPE_MM6_SDB)
#else
# define machine_is_mm6_sdb()	(0)
#endif

#ifdef CONFIG_MACH_SATURN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SATURN
# endif
# define machine_is_saturn()	(machine_arch_type == MACH_TYPE_SATURN)
#else
# define machine_is_saturn()	(0)
#endif

#ifdef CONFIG_MACH_I30030EVB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_I30030EVB
# endif
# define machine_is_i30030evb()	(machine_arch_type == MACH_TYPE_I30030EVB)
#else
# define machine_is_i30030evb()	(0)
#endif

#ifdef CONFIG_MACH_MXC27530EVB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MXC27530EVB
# endif
# define machine_is_mxc27530evb()	(machine_arch_type == MACH_TYPE_MXC27530EVB)
#else
# define machine_is_mxc27530evb()	(0)
#endif

#ifdef CONFIG_MACH_SMDK2800
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK2800
# endif
# define machine_is_smdk2800()	(machine_arch_type == MACH_TYPE_SMDK2800)
#else
# define machine_is_smdk2800()	(0)
#endif

#ifdef CONFIG_MACH_MTWILSON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MTWILSON
# endif
# define machine_is_mtwilson()	(machine_arch_type == MACH_TYPE_MTWILSON)
#else
# define machine_is_mtwilson()	(0)
#endif

#ifdef CONFIG_MACH_ZITI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ZITI
# endif
# define machine_is_ziti()	(machine_arch_type == MACH_TYPE_ZITI)
#else
# define machine_is_ziti()	(0)
#endif

#ifdef CONFIG_MACH_GRANDFATHER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GRANDFATHER
# endif
# define machine_is_grandfather()	(machine_arch_type == MACH_TYPE_GRANDFATHER)
#else
# define machine_is_grandfather()	(0)
#endif

#ifdef CONFIG_MACH_TENGINE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TENGINE
# endif
# define machine_is_tengine()	(machine_arch_type == MACH_TYPE_TENGINE)
#else
# define machine_is_tengine()	(0)
#endif

#ifdef CONFIG_MACH_S3C2460
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S3C2460
# endif
# define machine_is_s3c2460()	(machine_arch_type == MACH_TYPE_S3C2460)
#else
# define machine_is_s3c2460()	(0)
#endif

#ifdef CONFIG_MACH_PDM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PDM
# endif
# define machine_is_pdm()	(machine_arch_type == MACH_TYPE_PDM)
#else
# define machine_is_pdm()	(0)
#endif

#ifdef CONFIG_MACH_H4700
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H4700
# endif
# define machine_is_h4700()	(machine_arch_type == MACH_TYPE_H4700)
#else
# define machine_is_h4700()	(0)
#endif

#ifdef CONFIG_MACH_H6300
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H6300
# endif
# define machine_is_h6300()	(machine_arch_type == MACH_TYPE_H6300)
#else
# define machine_is_h6300()	(0)
#endif

#ifdef CONFIG_MACH_RZ1700
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RZ1700
# endif
# define machine_is_rz1700()	(machine_arch_type == MACH_TYPE_RZ1700)
#else
# define machine_is_rz1700()	(0)
#endif

#ifdef CONFIG_MACH_A716
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_A716
# endif
# define machine_is_a716()	(machine_arch_type == MACH_TYPE_A716)
#else
# define machine_is_a716()	(0)
#endif

#ifdef CONFIG_MACH_ESTK2440A
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESTK2440A
# endif
# define machine_is_estk2440a()	(machine_arch_type == MACH_TYPE_ESTK2440A)
#else
# define machine_is_estk2440a()	(0)
#endif

#ifdef CONFIG_MACH_ATWIXP425
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ATWIXP425
# endif
# define machine_is_atwixp425()	(machine_arch_type == MACH_TYPE_ATWIXP425)
#else
# define machine_is_atwixp425()	(0)
#endif

#ifdef CONFIG_MACH_CSB336
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CSB336
# endif
# define machine_is_csb336()	(machine_arch_type == MACH_TYPE_CSB336)
#else
# define machine_is_csb336()	(0)
#endif

#ifdef CONFIG_MACH_RIRM2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RIRM2
# endif
# define machine_is_rirm2()	(machine_arch_type == MACH_TYPE_RIRM2)
#else
# define machine_is_rirm2()	(0)
#endif

#ifdef CONFIG_MACH_CX23518
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CX23518
# endif
# define machine_is_cx23518()	(machine_arch_type == MACH_TYPE_CX23518)
#else
# define machine_is_cx23518()	(0)
#endif

#ifdef CONFIG_MACH_CX2351X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CX2351X
# endif
# define machine_is_cx2351x()	(machine_arch_type == MACH_TYPE_CX2351X)
#else
# define machine_is_cx2351x()	(0)
#endif

#ifdef CONFIG_MACH_COMPUTIME
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COMPUTIME
# endif
# define machine_is_computime()	(machine_arch_type == MACH_TYPE_COMPUTIME)
#else
# define machine_is_computime()	(0)
#endif

#ifdef CONFIG_MACH_IZARUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IZARUS
# endif
# define machine_is_izarus()	(machine_arch_type == MACH_TYPE_IZARUS)
#else
# define machine_is_izarus()	(0)
#endif

#ifdef CONFIG_MACH_RTS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RTS
# endif
# define machine_is_pxa_rts()	(machine_arch_type == MACH_TYPE_RTS)
#else
# define machine_is_pxa_rts()	(0)
#endif

#ifdef CONFIG_MACH_SE5100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SE5100
# endif
# define machine_is_se5100()	(machine_arch_type == MACH_TYPE_SE5100)
#else
# define machine_is_se5100()	(0)
#endif

#ifdef CONFIG_MACH_S3C2510
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S3C2510
# endif
# define machine_is_s3c2510()	(machine_arch_type == MACH_TYPE_S3C2510)
#else
# define machine_is_s3c2510()	(0)
#endif

#ifdef CONFIG_MACH_CSB437TL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CSB437TL
# endif
# define machine_is_csb437tl()	(machine_arch_type == MACH_TYPE_CSB437TL)
#else
# define machine_is_csb437tl()	(0)
#endif

#ifdef CONFIG_MACH_SLAUSON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SLAUSON
# endif
# define machine_is_slauson()	(machine_arch_type == MACH_TYPE_SLAUSON)
#else
# define machine_is_slauson()	(0)
#endif

#ifdef CONFIG_MACH_PEARLRIVER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PEARLRIVER
# endif
# define machine_is_pearlriver()	(machine_arch_type == MACH_TYPE_PEARLRIVER)
#else
# define machine_is_pearlriver()	(0)
#endif

#ifdef CONFIG_MACH_TDC_P210
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TDC_P210
# endif
# define machine_is_tdc_p210()	(machine_arch_type == MACH_TYPE_TDC_P210)
#else
# define machine_is_tdc_p210()	(0)
#endif

#ifdef CONFIG_MACH_SG580
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SG580
# endif
# define machine_is_sg580()	(machine_arch_type == MACH_TYPE_SG580)
#else
# define machine_is_sg580()	(0)
#endif

#ifdef CONFIG_MACH_WRSBCARM7
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WRSBCARM7
# endif
# define machine_is_wrsbcarm7()	(machine_arch_type == MACH_TYPE_WRSBCARM7)
#else
# define machine_is_wrsbcarm7()	(0)
#endif

#ifdef CONFIG_MACH_IPD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IPD
# endif
# define machine_is_ipd()	(machine_arch_type == MACH_TYPE_IPD)
#else
# define machine_is_ipd()	(0)
#endif

#ifdef CONFIG_MACH_PXA_DNP2110
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_DNP2110
# endif
# define machine_is_pxa_dnp2110()	(machine_arch_type == MACH_TYPE_PXA_DNP2110)
#else
# define machine_is_pxa_dnp2110()	(0)
#endif

#ifdef CONFIG_MACH_XAENIAX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XAENIAX
# endif
# define machine_is_xaeniax()	(machine_arch_type == MACH_TYPE_XAENIAX)
#else
# define machine_is_xaeniax()	(0)
#endif

#ifdef CONFIG_MACH_SOMN4250
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SOMN4250
# endif
# define machine_is_somn4250()	(machine_arch_type == MACH_TYPE_SOMN4250)
#else
# define machine_is_somn4250()	(0)
#endif

#ifdef CONFIG_MACH_PLEB2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PLEB2
# endif
# define machine_is_pleb2()	(machine_arch_type == MACH_TYPE_PLEB2)
#else
# define machine_is_pleb2()	(0)
#endif

#ifdef CONFIG_MACH_CORNWALLIS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CORNWALLIS
# endif
# define machine_is_cornwallis()	(machine_arch_type == MACH_TYPE_CORNWALLIS)
#else
# define machine_is_cornwallis()	(0)
#endif

#ifdef CONFIG_MACH_GURNEY_DRV
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GURNEY_DRV
# endif
# define machine_is_gurney_drv()	(machine_arch_type == MACH_TYPE_GURNEY_DRV)
#else
# define machine_is_gurney_drv()	(0)
#endif

#ifdef CONFIG_MACH_CHAFFEE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CHAFFEE
# endif
# define machine_is_chaffee()	(machine_arch_type == MACH_TYPE_CHAFFEE)
#else
# define machine_is_chaffee()	(0)
#endif

#ifdef CONFIG_MACH_RMS101
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RMS101
# endif
# define machine_is_rms101()	(machine_arch_type == MACH_TYPE_RMS101)
#else
# define machine_is_rms101()	(0)
#endif

#ifdef CONFIG_MACH_RX3715
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RX3715
# endif
# define machine_is_rx3715()	(machine_arch_type == MACH_TYPE_RX3715)
#else
# define machine_is_rx3715()	(0)
#endif

#ifdef CONFIG_MACH_SWIFT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SWIFT
# endif
# define machine_is_swift()	(machine_arch_type == MACH_TYPE_SWIFT)
#else
# define machine_is_swift()	(0)
#endif

#ifdef CONFIG_MACH_ROVERP7
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ROVERP7
# endif
# define machine_is_roverp7()	(machine_arch_type == MACH_TYPE_ROVERP7)
#else
# define machine_is_roverp7()	(0)
#endif

#ifdef CONFIG_MACH_PR818S
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PR818S
# endif
# define machine_is_pr818s()	(machine_arch_type == MACH_TYPE_PR818S)
#else
# define machine_is_pr818s()	(0)
#endif

#ifdef CONFIG_MACH_TRXPRO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TRXPRO
# endif
# define machine_is_trxpro()	(machine_arch_type == MACH_TYPE_TRXPRO)
#else
# define machine_is_trxpro()	(0)
#endif

#ifdef CONFIG_MACH_NSLU2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NSLU2
# endif
# define machine_is_nslu2()	(machine_arch_type == MACH_TYPE_NSLU2)
#else
# define machine_is_nslu2()	(0)
#endif

#ifdef CONFIG_MACH_E400
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_E400
# endif
# define machine_is_e400()	(machine_arch_type == MACH_TYPE_E400)
#else
# define machine_is_e400()	(0)
#endif

#ifdef CONFIG_MACH_TRAB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TRAB
# endif
# define machine_is_trab()	(machine_arch_type == MACH_TYPE_TRAB)
#else
# define machine_is_trab()	(0)
#endif

#ifdef CONFIG_MACH_CMC_PU2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CMC_PU2
# endif
# define machine_is_cmc_pu2()	(machine_arch_type == MACH_TYPE_CMC_PU2)
#else
# define machine_is_cmc_pu2()	(0)
#endif

#ifdef CONFIG_MACH_FULCRUM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FULCRUM
# endif
# define machine_is_fulcrum()	(machine_arch_type == MACH_TYPE_FULCRUM)
#else
# define machine_is_fulcrum()	(0)
#endif

#ifdef CONFIG_MACH_NETGATE42X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NETGATE42X
# endif
# define machine_is_netgate42x()	(machine_arch_type == MACH_TYPE_NETGATE42X)
#else
# define machine_is_netgate42x()	(0)
#endif

#ifdef CONFIG_MACH_STR710
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STR710
# endif
# define machine_is_str710()	(machine_arch_type == MACH_TYPE_STR710)
#else
# define machine_is_str710()	(0)
#endif

#ifdef CONFIG_MACH_IXDPG425
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDPG425
# endif
# define machine_is_ixdpg425()	(machine_arch_type == MACH_TYPE_IXDPG425)
#else
# define machine_is_ixdpg425()	(0)
#endif

#ifdef CONFIG_MACH_TOMTOMGO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TOMTOMGO
# endif
# define machine_is_tomtomgo()	(machine_arch_type == MACH_TYPE_TOMTOMGO)
#else
# define machine_is_tomtomgo()	(0)
#endif

#ifdef CONFIG_MACH_VERSATILE_AB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VERSATILE_AB
# endif
# define machine_is_versatile_ab()	(machine_arch_type == MACH_TYPE_VERSATILE_AB)
#else
# define machine_is_versatile_ab()	(0)
#endif

#ifdef CONFIG_MACH_EDB9307
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB9307
# endif
# define machine_is_edb9307()	(machine_arch_type == MACH_TYPE_EDB9307)
#else
# define machine_is_edb9307()	(0)
#endif

#ifdef CONFIG_MACH_SG565
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SG565
# endif
# define machine_is_sg565()	(machine_arch_type == MACH_TYPE_SG565)
#else
# define machine_is_sg565()	(0)
#endif

#ifdef CONFIG_MACH_LPD79524
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LPD79524
# endif
# define machine_is_lpd79524()	(machine_arch_type == MACH_TYPE_LPD79524)
#else
# define machine_is_lpd79524()	(0)
#endif

#ifdef CONFIG_MACH_LPD79525
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LPD79525
# endif
# define machine_is_lpd79525()	(machine_arch_type == MACH_TYPE_LPD79525)
#else
# define machine_is_lpd79525()	(0)
#endif

#ifdef CONFIG_MACH_RMS100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RMS100
# endif
# define machine_is_rms100()	(machine_arch_type == MACH_TYPE_RMS100)
#else
# define machine_is_rms100()	(0)
#endif

#ifdef CONFIG_MACH_KB9200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KB9200
# endif
# define machine_is_kb9200()	(machine_arch_type == MACH_TYPE_KB9200)
#else
# define machine_is_kb9200()	(0)
#endif

#ifdef CONFIG_MACH_SX1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SX1
# endif
# define machine_is_sx1()	(machine_arch_type == MACH_TYPE_SX1)
#else
# define machine_is_sx1()	(0)
#endif

#ifdef CONFIG_MACH_HMS39C7092
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HMS39C7092
# endif
# define machine_is_hms39c7092()	(machine_arch_type == MACH_TYPE_HMS39C7092)
#else
# define machine_is_hms39c7092()	(0)
#endif

#ifdef CONFIG_MACH_ARMADILLO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARMADILLO
# endif
# define machine_is_armadillo()	(machine_arch_type == MACH_TYPE_ARMADILLO)
#else
# define machine_is_armadillo()	(0)
#endif

#ifdef CONFIG_MACH_IPCU
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IPCU
# endif
# define machine_is_ipcu()	(machine_arch_type == MACH_TYPE_IPCU)
#else
# define machine_is_ipcu()	(0)
#endif

#ifdef CONFIG_MACH_LOOX720
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LOOX720
# endif
# define machine_is_loox720()	(machine_arch_type == MACH_TYPE_LOOX720)
#else
# define machine_is_loox720()	(0)
#endif

#ifdef CONFIG_MACH_IXDP465
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDP465
# endif
# define machine_is_ixdp465()	(machine_arch_type == MACH_TYPE_IXDP465)
#else
# define machine_is_ixdp465()	(0)
#endif

#ifdef CONFIG_MACH_IXDP2351
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDP2351
# endif
# define machine_is_ixdp2351()	(machine_arch_type == MACH_TYPE_IXDP2351)
#else
# define machine_is_ixdp2351()	(0)
#endif

#ifdef CONFIG_MACH_ADSVIX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADSVIX
# endif
# define machine_is_adsvix()	(machine_arch_type == MACH_TYPE_ADSVIX)
#else
# define machine_is_adsvix()	(0)
#endif

#ifdef CONFIG_MACH_DM270
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DM270
# endif
# define machine_is_dm270()	(machine_arch_type == MACH_TYPE_DM270)
#else
# define machine_is_dm270()	(0)
#endif

#ifdef CONFIG_MACH_SOCLTPLUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SOCLTPLUS
# endif
# define machine_is_socltplus()	(machine_arch_type == MACH_TYPE_SOCLTPLUS)
#else
# define machine_is_socltplus()	(0)
#endif

#ifdef CONFIG_MACH_ECIA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ECIA
# endif
# define machine_is_ecia()	(machine_arch_type == MACH_TYPE_ECIA)
#else
# define machine_is_ecia()	(0)
#endif

#ifdef CONFIG_MACH_CM4008
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CM4008
# endif
# define machine_is_cm4008()	(machine_arch_type == MACH_TYPE_CM4008)
#else
# define machine_is_cm4008()	(0)
#endif

#ifdef CONFIG_MACH_P2001
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_P2001
# endif
# define machine_is_p2001()	(machine_arch_type == MACH_TYPE_P2001)
#else
# define machine_is_p2001()	(0)
#endif

#ifdef CONFIG_MACH_TWISTER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TWISTER
# endif
# define machine_is_twister()	(machine_arch_type == MACH_TYPE_TWISTER)
#else
# define machine_is_twister()	(0)
#endif

#ifdef CONFIG_MACH_MUDSHARK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MUDSHARK
# endif
# define machine_is_mudshark()	(machine_arch_type == MACH_TYPE_MUDSHARK)
#else
# define machine_is_mudshark()	(0)
#endif

#ifdef CONFIG_MACH_HB2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HB2
# endif
# define machine_is_hb2()	(machine_arch_type == MACH_TYPE_HB2)
#else
# define machine_is_hb2()	(0)
#endif

#ifdef CONFIG_MACH_IQ80332
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IQ80332
# endif
# define machine_is_iq80332()	(machine_arch_type == MACH_TYPE_IQ80332)
#else
# define machine_is_iq80332()	(0)
#endif

#ifdef CONFIG_MACH_SENDT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SENDT
# endif
# define machine_is_sendt()	(machine_arch_type == MACH_TYPE_SENDT)
#else
# define machine_is_sendt()	(0)
#endif

#ifdef CONFIG_MACH_MX2JAZZ
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX2JAZZ
# endif
# define machine_is_mx2jazz()	(machine_arch_type == MACH_TYPE_MX2JAZZ)
#else
# define machine_is_mx2jazz()	(0)
#endif

#ifdef CONFIG_MACH_MULTIIO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MULTIIO
# endif
# define machine_is_multiio()	(machine_arch_type == MACH_TYPE_MULTIIO)
#else
# define machine_is_multiio()	(0)
#endif

#ifdef CONFIG_MACH_HRDISPLAY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HRDISPLAY
# endif
# define machine_is_hrdisplay()	(machine_arch_type == MACH_TYPE_HRDISPLAY)
#else
# define machine_is_hrdisplay()	(0)
#endif

#ifdef CONFIG_MACH_MXC27530ADS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MXC27530ADS
# endif
# define machine_is_mxc27530ads()	(machine_arch_type == MACH_TYPE_MXC27530ADS)
#else
# define machine_is_mxc27530ads()	(0)
#endif

#ifdef CONFIG_MACH_TRIZEPS3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TRIZEPS3
# endif
# define machine_is_trizeps3()	(machine_arch_type == MACH_TYPE_TRIZEPS3)
#else
# define machine_is_trizeps3()	(0)
#endif

#ifdef CONFIG_MACH_ZEFEERDZA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ZEFEERDZA
# endif
# define machine_is_zefeerdza()	(machine_arch_type == MACH_TYPE_ZEFEERDZA)
#else
# define machine_is_zefeerdza()	(0)
#endif

#ifdef CONFIG_MACH_ZEFEERDZB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ZEFEERDZB
# endif
# define machine_is_zefeerdzb()	(machine_arch_type == MACH_TYPE_ZEFEERDZB)
#else
# define machine_is_zefeerdzb()	(0)
#endif

#ifdef CONFIG_MACH_ZEFEERDZG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ZEFEERDZG
# endif
# define machine_is_zefeerdzg()	(machine_arch_type == MACH_TYPE_ZEFEERDZG)
#else
# define machine_is_zefeerdzg()	(0)
#endif

#ifdef CONFIG_MACH_ZEFEERDZN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ZEFEERDZN
# endif
# define machine_is_zefeerdzn()	(machine_arch_type == MACH_TYPE_ZEFEERDZN)
#else
# define machine_is_zefeerdzn()	(0)
#endif

#ifdef CONFIG_MACH_ZEFEERDZQ
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ZEFEERDZQ
# endif
# define machine_is_zefeerdzq()	(machine_arch_type == MACH_TYPE_ZEFEERDZQ)
#else
# define machine_is_zefeerdzq()	(0)
#endif

#ifdef CONFIG_MACH_GTWX5715
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GTWX5715
# endif
# define machine_is_gtwx5715()	(machine_arch_type == MACH_TYPE_GTWX5715)
#else
# define machine_is_gtwx5715()	(0)
#endif

#ifdef CONFIG_MACH_ASTRO_JACK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ASTRO_JACK
# endif
# define machine_is_astro_jack()	(machine_arch_type == MACH_TYPE_ASTRO_JACK)
#else
# define machine_is_astro_jack()	(0)
#endif

#ifdef CONFIG_MACH_TIP03
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TIP03
# endif
# define machine_is_tip03()	(machine_arch_type == MACH_TYPE_TIP03)
#else
# define machine_is_tip03()	(0)
#endif

#ifdef CONFIG_MACH_A9200EC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_A9200EC
# endif
# define machine_is_a9200ec()	(machine_arch_type == MACH_TYPE_A9200EC)
#else
# define machine_is_a9200ec()	(0)
#endif

#ifdef CONFIG_MACH_PNX0105
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PNX0105
# endif
# define machine_is_pnx0105()	(machine_arch_type == MACH_TYPE_PNX0105)
#else
# define machine_is_pnx0105()	(0)
#endif

#ifdef CONFIG_MACH_ADCPOECPU
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADCPOECPU
# endif
# define machine_is_adcpoecpu()	(machine_arch_type == MACH_TYPE_ADCPOECPU)
#else
# define machine_is_adcpoecpu()	(0)
#endif

#ifdef CONFIG_MACH_CSB637
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CSB637
# endif
# define machine_is_csb637()	(machine_arch_type == MACH_TYPE_CSB637)
#else
# define machine_is_csb637()	(0)
#endif

#ifdef CONFIG_MACH_ML69Q6203
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ML69Q6203
# endif
# define machine_is_ml69q6203()	(machine_arch_type == MACH_TYPE_ML69Q6203)
#else
# define machine_is_ml69q6203()	(0)
#endif

#ifdef CONFIG_MACH_MB9200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MB9200
# endif
# define machine_is_mb9200()	(machine_arch_type == MACH_TYPE_MB9200)
#else
# define machine_is_mb9200()	(0)
#endif

#ifdef CONFIG_MACH_KULUN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KULUN
# endif
# define machine_is_kulun()	(machine_arch_type == MACH_TYPE_KULUN)
#else
# define machine_is_kulun()	(0)
#endif

#ifdef CONFIG_MACH_SNAPPER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SNAPPER
# endif
# define machine_is_snapper()	(machine_arch_type == MACH_TYPE_SNAPPER)
#else
# define machine_is_snapper()	(0)
#endif

#ifdef CONFIG_MACH_OPTIMA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OPTIMA
# endif
# define machine_is_optima()	(machine_arch_type == MACH_TYPE_OPTIMA)
#else
# define machine_is_optima()	(0)
#endif

#ifdef CONFIG_MACH_DLHSBC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DLHSBC
# endif
# define machine_is_dlhsbc()	(machine_arch_type == MACH_TYPE_DLHSBC)
#else
# define machine_is_dlhsbc()	(0)
#endif

#ifdef CONFIG_MACH_X30
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_X30
# endif
# define machine_is_x30()	(machine_arch_type == MACH_TYPE_X30)
#else
# define machine_is_x30()	(0)
#endif

#ifdef CONFIG_MACH_N30
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_N30
# endif
# define machine_is_n30()	(machine_arch_type == MACH_TYPE_N30)
#else
# define machine_is_n30()	(0)
#endif

#ifdef CONFIG_MACH_MANGA_KS8695
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MANGA_KS8695
# endif
# define machine_is_manga_ks8695()	(machine_arch_type == MACH_TYPE_MANGA_KS8695)
#else
# define machine_is_manga_ks8695()	(0)
#endif

#ifdef CONFIG_MACH_AJAX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AJAX
# endif
# define machine_is_ajax()	(machine_arch_type == MACH_TYPE_AJAX)
#else
# define machine_is_ajax()	(0)
#endif

#ifdef CONFIG_MACH_NEC_MP900
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NEC_MP900
# endif
# define machine_is_nec_mp900()	(machine_arch_type == MACH_TYPE_NEC_MP900)
#else
# define machine_is_nec_mp900()	(0)
#endif

#ifdef CONFIG_MACH_VVTK1000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VVTK1000
# endif
# define machine_is_vvtk1000()	(machine_arch_type == MACH_TYPE_VVTK1000)
#else
# define machine_is_vvtk1000()	(0)
#endif

#ifdef CONFIG_MACH_KAFA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KAFA
# endif
# define machine_is_kafa()	(machine_arch_type == MACH_TYPE_KAFA)
#else
# define machine_is_kafa()	(0)
#endif

#ifdef CONFIG_MACH_VVTK3000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VVTK3000
# endif
# define machine_is_vvtk3000()	(machine_arch_type == MACH_TYPE_VVTK3000)
#else
# define machine_is_vvtk3000()	(0)
#endif

#ifdef CONFIG_MACH_PIMX1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PIMX1
# endif
# define machine_is_pimx1()	(machine_arch_type == MACH_TYPE_PIMX1)
#else
# define machine_is_pimx1()	(0)
#endif

#ifdef CONFIG_MACH_OLLIE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OLLIE
# endif
# define machine_is_ollie()	(machine_arch_type == MACH_TYPE_OLLIE)
#else
# define machine_is_ollie()	(0)
#endif

#ifdef CONFIG_MACH_SKYMAX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SKYMAX
# endif
# define machine_is_skymax()	(machine_arch_type == MACH_TYPE_SKYMAX)
#else
# define machine_is_skymax()	(0)
#endif

#ifdef CONFIG_MACH_JAZZ
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_JAZZ
# endif
# define machine_is_jazz()	(machine_arch_type == MACH_TYPE_JAZZ)
#else
# define machine_is_jazz()	(0)
#endif

#ifdef CONFIG_MACH_TEL_T3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TEL_T3
# endif
# define machine_is_tel_t3()	(machine_arch_type == MACH_TYPE_TEL_T3)
#else
# define machine_is_tel_t3()	(0)
#endif

#ifdef CONFIG_MACH_AISINO_FCR255
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AISINO_FCR255
# endif
# define machine_is_aisino_fcr255()	(machine_arch_type == MACH_TYPE_AISINO_FCR255)
#else
# define machine_is_aisino_fcr255()	(0)
#endif

#ifdef CONFIG_MACH_BTWEB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BTWEB
# endif
# define machine_is_btweb()	(machine_arch_type == MACH_TYPE_BTWEB)
#else
# define machine_is_btweb()	(0)
#endif

#ifdef CONFIG_MACH_DBG_LH79520
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DBG_LH79520
# endif
# define machine_is_dbg_lh79520()	(machine_arch_type == MACH_TYPE_DBG_LH79520)
#else
# define machine_is_dbg_lh79520()	(0)
#endif

#ifdef CONFIG_MACH_CM41XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CM41XX
# endif
# define machine_is_cm41xx()	(machine_arch_type == MACH_TYPE_CM41XX)
#else
# define machine_is_cm41xx()	(0)
#endif

#ifdef CONFIG_MACH_TS72XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TS72XX
# endif
# define machine_is_ts72xx()	(machine_arch_type == MACH_TYPE_TS72XX)
#else
# define machine_is_ts72xx()	(0)
#endif

#ifdef CONFIG_MACH_NGGPXA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NGGPXA
# endif
# define machine_is_nggpxa()	(machine_arch_type == MACH_TYPE_NGGPXA)
#else
# define machine_is_nggpxa()	(0)
#endif

#ifdef CONFIG_MACH_CSB535
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CSB535
# endif
# define machine_is_csb535()	(machine_arch_type == MACH_TYPE_CSB535)
#else
# define machine_is_csb535()	(0)
#endif

#ifdef CONFIG_MACH_CSB536
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CSB536
# endif
# define machine_is_csb536()	(machine_arch_type == MACH_TYPE_CSB536)
#else
# define machine_is_csb536()	(0)
#endif

#ifdef CONFIG_MACH_PXA_TRAKPOD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_TRAKPOD
# endif
# define machine_is_pxa_trakpod()	(machine_arch_type == MACH_TYPE_PXA_TRAKPOD)
#else
# define machine_is_pxa_trakpod()	(0)
#endif

#ifdef CONFIG_MACH_PRAXIS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PRAXIS
# endif
# define machine_is_praxis()	(machine_arch_type == MACH_TYPE_PRAXIS)
#else
# define machine_is_praxis()	(0)
#endif

#ifdef CONFIG_MACH_LH75411
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LH75411
# endif
# define machine_is_lh75411()	(machine_arch_type == MACH_TYPE_LH75411)
#else
# define machine_is_lh75411()	(0)
#endif

#ifdef CONFIG_MACH_OTOM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OTOM
# endif
# define machine_is_otom()	(machine_arch_type == MACH_TYPE_OTOM)
#else
# define machine_is_otom()	(0)
#endif

#ifdef CONFIG_MACH_NEXCODER_2440
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NEXCODER_2440
# endif
# define machine_is_nexcoder_2440()	(machine_arch_type == MACH_TYPE_NEXCODER_2440)
#else
# define machine_is_nexcoder_2440()	(0)
#endif

#ifdef CONFIG_MACH_LOOX410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LOOX410
# endif
# define machine_is_loox410()	(machine_arch_type == MACH_TYPE_LOOX410)
#else
# define machine_is_loox410()	(0)
#endif

#ifdef CONFIG_MACH_WESTLAKE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WESTLAKE
# endif
# define machine_is_westlake()	(machine_arch_type == MACH_TYPE_WESTLAKE)
#else
# define machine_is_westlake()	(0)
#endif

#ifdef CONFIG_MACH_NSB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NSB
# endif
# define machine_is_nsb()	(machine_arch_type == MACH_TYPE_NSB)
#else
# define machine_is_nsb()	(0)
#endif

#ifdef CONFIG_MACH_ESL_SARVA_STN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESL_SARVA_STN
# endif
# define machine_is_esl_sarva_stn()	(machine_arch_type == MACH_TYPE_ESL_SARVA_STN)
#else
# define machine_is_esl_sarva_stn()	(0)
#endif

#ifdef CONFIG_MACH_ESL_SARVA_TFT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESL_SARVA_TFT
# endif
# define machine_is_esl_sarva_tft()	(machine_arch_type == MACH_TYPE_ESL_SARVA_TFT)
#else
# define machine_is_esl_sarva_tft()	(0)
#endif

#ifdef CONFIG_MACH_ESL_SARVA_IAD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESL_SARVA_IAD
# endif
# define machine_is_esl_sarva_iad()	(machine_arch_type == MACH_TYPE_ESL_SARVA_IAD)
#else
# define machine_is_esl_sarva_iad()	(0)
#endif

#ifdef CONFIG_MACH_ESL_SARVA_ACC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESL_SARVA_ACC
# endif
# define machine_is_esl_sarva_acc()	(machine_arch_type == MACH_TYPE_ESL_SARVA_ACC)
#else
# define machine_is_esl_sarva_acc()	(0)
#endif

#ifdef CONFIG_MACH_TYPHOON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TYPHOON
# endif
# define machine_is_typhoon()	(machine_arch_type == MACH_TYPE_TYPHOON)
#else
# define machine_is_typhoon()	(0)
#endif

#ifdef CONFIG_MACH_CNAV
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CNAV
# endif
# define machine_is_cnav()	(machine_arch_type == MACH_TYPE_CNAV)
#else
# define machine_is_cnav()	(0)
#endif

#ifdef CONFIG_MACH_A730
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_A730
# endif
# define machine_is_a730()	(machine_arch_type == MACH_TYPE_A730)
#else
# define machine_is_a730()	(0)
#endif

#ifdef CONFIG_MACH_NETSTAR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NETSTAR
# endif
# define machine_is_netstar()	(machine_arch_type == MACH_TYPE_NETSTAR)
#else
# define machine_is_netstar()	(0)
#endif

#ifdef CONFIG_MACH_PHASEFALE_SUPERCON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PHASEFALE_SUPERCON
# endif
# define machine_is_supercon()	(machine_arch_type == MACH_TYPE_PHASEFALE_SUPERCON)
#else
# define machine_is_supercon()	(0)
#endif

#ifdef CONFIG_MACH_SHIVA1100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SHIVA1100
# endif
# define machine_is_shiva1100()	(machine_arch_type == MACH_TYPE_SHIVA1100)
#else
# define machine_is_shiva1100()	(0)
#endif

#ifdef CONFIG_MACH_ETEXSC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ETEXSC
# endif
# define machine_is_etexsc()	(machine_arch_type == MACH_TYPE_ETEXSC)
#else
# define machine_is_etexsc()	(0)
#endif

#ifdef CONFIG_MACH_IXDPG465
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDPG465
# endif
# define machine_is_ixdpg465()	(machine_arch_type == MACH_TYPE_IXDPG465)
#else
# define machine_is_ixdpg465()	(0)
#endif

#ifdef CONFIG_MACH_A9M2410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_A9M2410
# endif
# define machine_is_a9m2410()	(machine_arch_type == MACH_TYPE_A9M2410)
#else
# define machine_is_a9m2410()	(0)
#endif

#ifdef CONFIG_MACH_A9M2440
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_A9M2440
# endif
# define machine_is_a9m2440()	(machine_arch_type == MACH_TYPE_A9M2440)
#else
# define machine_is_a9m2440()	(0)
#endif

#ifdef CONFIG_MACH_A9M9750
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_A9M9750
# endif
# define machine_is_a9m9750()	(machine_arch_type == MACH_TYPE_A9M9750)
#else
# define machine_is_a9m9750()	(0)
#endif

#ifdef CONFIG_MACH_A9M9360
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_A9M9360
# endif
# define machine_is_a9m9360()	(machine_arch_type == MACH_TYPE_A9M9360)
#else
# define machine_is_a9m9360()	(0)
#endif

#ifdef CONFIG_MACH_UNC90
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_UNC90
# endif
# define machine_is_unc90()	(machine_arch_type == MACH_TYPE_UNC90)
#else
# define machine_is_unc90()	(0)
#endif

#ifdef CONFIG_MACH_ECO920
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ECO920
# endif
# define machine_is_eco920()	(machine_arch_type == MACH_TYPE_ECO920)
#else
# define machine_is_eco920()	(0)
#endif

#ifdef CONFIG_MACH_SATVIEW
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SATVIEW
# endif
# define machine_is_satview()	(machine_arch_type == MACH_TYPE_SATVIEW)
#else
# define machine_is_satview()	(0)
#endif

#ifdef CONFIG_MACH_ROADRUNNER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ROADRUNNER
# endif
# define machine_is_roadrunner()	(machine_arch_type == MACH_TYPE_ROADRUNNER)
#else
# define machine_is_roadrunner()	(0)
#endif

#ifdef CONFIG_MACH_AT91RM9200EK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91RM9200EK
# endif
# define machine_is_at91rm9200ek()	(machine_arch_type == MACH_TYPE_AT91RM9200EK)
#else
# define machine_is_at91rm9200ek()	(0)
#endif

#ifdef CONFIG_MACH_GP32
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GP32
# endif
# define machine_is_gp32()	(machine_arch_type == MACH_TYPE_GP32)
#else
# define machine_is_gp32()	(0)
#endif

#ifdef CONFIG_MACH_GEM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GEM
# endif
# define machine_is_gem()	(machine_arch_type == MACH_TYPE_GEM)
#else
# define machine_is_gem()	(0)
#endif

#ifdef CONFIG_MACH_I858
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_I858
# endif
# define machine_is_i858()	(machine_arch_type == MACH_TYPE_I858)
#else
# define machine_is_i858()	(0)
#endif

#ifdef CONFIG_MACH_HX2750
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HX2750
# endif
# define machine_is_hx2750()	(machine_arch_type == MACH_TYPE_HX2750)
#else
# define machine_is_hx2750()	(0)
#endif

#ifdef CONFIG_MACH_MXC91131EVB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MXC91131EVB
# endif
# define machine_is_mxc91131evb()	(machine_arch_type == MACH_TYPE_MXC91131EVB)
#else
# define machine_is_mxc91131evb()	(0)
#endif

#ifdef CONFIG_MACH_P700
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_P700
# endif
# define machine_is_p700()	(machine_arch_type == MACH_TYPE_P700)
#else
# define machine_is_p700()	(0)
#endif

#ifdef CONFIG_MACH_CPE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CPE
# endif
# define machine_is_cpe()	(machine_arch_type == MACH_TYPE_CPE)
#else
# define machine_is_cpe()	(0)
#endif

#ifdef CONFIG_MACH_SPITZ
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPITZ
# endif
# define machine_is_spitz()	(machine_arch_type == MACH_TYPE_SPITZ)
#else
# define machine_is_spitz()	(0)
#endif

#ifdef CONFIG_MACH_NIMBRA340
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NIMBRA340
# endif
# define machine_is_nimbra340()	(machine_arch_type == MACH_TYPE_NIMBRA340)
#else
# define machine_is_nimbra340()	(0)
#endif

#ifdef CONFIG_MACH_LPC22XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LPC22XX
# endif
# define machine_is_lpc22xx()	(machine_arch_type == MACH_TYPE_LPC22XX)
#else
# define machine_is_lpc22xx()	(0)
#endif

#ifdef CONFIG_MACH_COMET3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COMET3
# endif
# define machine_is_omap_comet3()	(machine_arch_type == MACH_TYPE_COMET3)
#else
# define machine_is_omap_comet3()	(0)
#endif

#ifdef CONFIG_MACH_COMET4
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COMET4
# endif
# define machine_is_omap_comet4()	(machine_arch_type == MACH_TYPE_COMET4)
#else
# define machine_is_omap_comet4()	(0)
#endif

#ifdef CONFIG_MACH_CSB625
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CSB625
# endif
# define machine_is_csb625()	(machine_arch_type == MACH_TYPE_CSB625)
#else
# define machine_is_csb625()	(0)
#endif

#ifdef CONFIG_MACH_FORTUNET2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FORTUNET2
# endif
# define machine_is_fortunet2()	(machine_arch_type == MACH_TYPE_FORTUNET2)
#else
# define machine_is_fortunet2()	(0)
#endif

#ifdef CONFIG_MACH_S5H2200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S5H2200
# endif
# define machine_is_s5h2200()	(machine_arch_type == MACH_TYPE_S5H2200)
#else
# define machine_is_s5h2200()	(0)
#endif

#ifdef CONFIG_MACH_OPTORM920
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OPTORM920
# endif
# define machine_is_optorm920()	(machine_arch_type == MACH_TYPE_OPTORM920)
#else
# define machine_is_optorm920()	(0)
#endif

#ifdef CONFIG_MACH_ADSBITSYXB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADSBITSYXB
# endif
# define machine_is_adsbitsyxb()	(machine_arch_type == MACH_TYPE_ADSBITSYXB)
#else
# define machine_is_adsbitsyxb()	(0)
#endif

#ifdef CONFIG_MACH_ADSSPHERE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADSSPHERE
# endif
# define machine_is_adssphere()	(machine_arch_type == MACH_TYPE_ADSSPHERE)
#else
# define machine_is_adssphere()	(0)
#endif

#ifdef CONFIG_MACH_ADSPORTAL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADSPORTAL
# endif
# define machine_is_adsportal()	(machine_arch_type == MACH_TYPE_ADSPORTAL)
#else
# define machine_is_adsportal()	(0)
#endif

#ifdef CONFIG_MACH_LN2410SBC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LN2410SBC
# endif
# define machine_is_ln2410sbc()	(machine_arch_type == MACH_TYPE_LN2410SBC)
#else
# define machine_is_ln2410sbc()	(0)
#endif

#ifdef CONFIG_MACH_CB3RUFC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CB3RUFC
# endif
# define machine_is_cb3rufc()	(machine_arch_type == MACH_TYPE_CB3RUFC)
#else
# define machine_is_cb3rufc()	(0)
#endif

#ifdef CONFIG_MACH_MP2USB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MP2USB
# endif
# define machine_is_mp2usb()	(machine_arch_type == MACH_TYPE_MP2USB)
#else
# define machine_is_mp2usb()	(0)
#endif

#ifdef CONFIG_MACH_WMT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WMT
# endif
# define machine_is_WMT()	(machine_arch_type == MACH_TYPE_WMT)
#else
# define machine_is_WMT()	(0)
#endif

#ifdef CONFIG_MACH_COLIBRI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COLIBRI
# endif
# define machine_is_colibri()	(machine_arch_type == MACH_TYPE_COLIBRI)
#else
# define machine_is_colibri()	(0)
#endif

#ifdef CONFIG_MACH_PCM7220
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PCM7220
# endif
# define machine_is_pcm7220()	(machine_arch_type == MACH_TYPE_PCM7220)
#else
# define machine_is_pcm7220()	(0)
#endif

#ifdef CONFIG_MACH_GATEWAY7001
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GATEWAY7001
# endif
# define machine_is_gateway7001()	(machine_arch_type == MACH_TYPE_GATEWAY7001)
#else
# define machine_is_gateway7001()	(0)
#endif

#ifdef CONFIG_MACH_PCM027
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PCM027
# endif
# define machine_is_pcm027()	(machine_arch_type == MACH_TYPE_PCM027)
#else
# define machine_is_pcm027()	(0)
#endif

#ifdef CONFIG_MACH_CMPXA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CMPXA
# endif
# define machine_is_cmpxa()	(machine_arch_type == MACH_TYPE_CMPXA)
#else
# define machine_is_cmpxa()	(0)
#endif

#ifdef CONFIG_MACH_ANUBIS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ANUBIS
# endif
# define machine_is_anubis()	(machine_arch_type == MACH_TYPE_ANUBIS)
#else
# define machine_is_anubis()	(0)
#endif

#ifdef CONFIG_MACH_ITE8152
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ITE8152
# endif
# define machine_is_ite8152()	(machine_arch_type == MACH_TYPE_ITE8152)
#else
# define machine_is_ite8152()	(0)
#endif

#ifdef CONFIG_MACH_LPC3XXX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LPC3XXX
# endif
# define machine_is_lpc3xxx()	(machine_arch_type == MACH_TYPE_LPC3XXX)
#else
# define machine_is_lpc3xxx()	(0)
#endif

#ifdef CONFIG_MACH_PUPPETEER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PUPPETEER
# endif
# define machine_is_puppeteer()	(machine_arch_type == MACH_TYPE_PUPPETEER)
#else
# define machine_is_puppeteer()	(0)
#endif

#ifdef CONFIG_MACH_MACH_VADATECH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MACH_VADATECH
# endif
# define machine_is_vt001()	(machine_arch_type == MACH_TYPE_MACH_VADATECH)
#else
# define machine_is_vt001()	(0)
#endif

#ifdef CONFIG_MACH_E570
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_E570
# endif
# define machine_is_e570()	(machine_arch_type == MACH_TYPE_E570)
#else
# define machine_is_e570()	(0)
#endif

#ifdef CONFIG_MACH_X50
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_X50
# endif
# define machine_is_x50()	(machine_arch_type == MACH_TYPE_X50)
#else
# define machine_is_x50()	(0)
#endif

#ifdef CONFIG_MACH_RECON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RECON
# endif
# define machine_is_recon()	(machine_arch_type == MACH_TYPE_RECON)
#else
# define machine_is_recon()	(0)
#endif

#ifdef CONFIG_MACH_XBOARDGP8
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XBOARDGP8
# endif
# define machine_is_xboardgp8()	(machine_arch_type == MACH_TYPE_XBOARDGP8)
#else
# define machine_is_xboardgp8()	(0)
#endif

#ifdef CONFIG_MACH_FPIC2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FPIC2
# endif
# define machine_is_fpic2()	(machine_arch_type == MACH_TYPE_FPIC2)
#else
# define machine_is_fpic2()	(0)
#endif

#ifdef CONFIG_MACH_AKITA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AKITA
# endif
# define machine_is_akita()	(machine_arch_type == MACH_TYPE_AKITA)
#else
# define machine_is_akita()	(0)
#endif

#ifdef CONFIG_MACH_A81
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_A81
# endif
# define machine_is_a81()	(machine_arch_type == MACH_TYPE_A81)
#else
# define machine_is_a81()	(0)
#endif

#ifdef CONFIG_MACH_SVM_SC25X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SVM_SC25X
# endif
# define machine_is_svm_sc25x()	(machine_arch_type == MACH_TYPE_SVM_SC25X)
#else
# define machine_is_svm_sc25x()	(0)
#endif

#ifdef CONFIG_MACH_VADATECH020
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VADATECH020
# endif
# define machine_is_vt020()	(machine_arch_type == MACH_TYPE_VADATECH020)
#else
# define machine_is_vt020()	(0)
#endif

#ifdef CONFIG_MACH_TLI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TLI
# endif
# define machine_is_tli()	(machine_arch_type == MACH_TYPE_TLI)
#else
# define machine_is_tli()	(0)
#endif

#ifdef CONFIG_MACH_EDB9315LC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB9315LC
# endif
# define machine_is_edb9315lc()	(machine_arch_type == MACH_TYPE_EDB9315LC)
#else
# define machine_is_edb9315lc()	(0)
#endif

#ifdef CONFIG_MACH_PASSEC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PASSEC
# endif
# define machine_is_passec()	(machine_arch_type == MACH_TYPE_PASSEC)
#else
# define machine_is_passec()	(0)
#endif

#ifdef CONFIG_MACH_DS_TIGER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DS_TIGER
# endif
# define machine_is_ds_tiger()	(machine_arch_type == MACH_TYPE_DS_TIGER)
#else
# define machine_is_ds_tiger()	(0)
#endif

#ifdef CONFIG_MACH_E310
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_E310
# endif
# define machine_is_e310()	(machine_arch_type == MACH_TYPE_E310)
#else
# define machine_is_e310()	(0)
#endif

#ifdef CONFIG_MACH_E330
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_E330
# endif
# define machine_is_e330()	(machine_arch_type == MACH_TYPE_E330)
#else
# define machine_is_e330()	(0)
#endif

#ifdef CONFIG_MACH_RT3000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RT3000
# endif
# define machine_is_rt3000()	(machine_arch_type == MACH_TYPE_RT3000)
#else
# define machine_is_rt3000()	(0)
#endif

#ifdef CONFIG_MACH_NOKIA770
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NOKIA770
# endif
# define machine_is_nokia770()	(machine_arch_type == MACH_TYPE_NOKIA770)
#else
# define machine_is_nokia770()	(0)
#endif

#ifdef CONFIG_MACH_PNX0106
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PNX0106
# endif
# define machine_is_pnx0106()	(machine_arch_type == MACH_TYPE_PNX0106)
#else
# define machine_is_pnx0106()	(0)
#endif

#ifdef CONFIG_MACH_HX21XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HX21XX
# endif
# define machine_is_hx21xx()	(machine_arch_type == MACH_TYPE_HX21XX)
#else
# define machine_is_hx21xx()	(0)
#endif

#ifdef CONFIG_MACH_FARADAY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FARADAY
# endif
# define machine_is_faraday()	(machine_arch_type == MACH_TYPE_FARADAY)
#else
# define machine_is_faraday()	(0)
#endif

#ifdef CONFIG_MACH_SBC9312
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SBC9312
# endif
# define machine_is_sbc9312()	(machine_arch_type == MACH_TYPE_SBC9312)
#else
# define machine_is_sbc9312()	(0)
#endif

#ifdef CONFIG_MACH_BATMAN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BATMAN
# endif
# define machine_is_batman()	(machine_arch_type == MACH_TYPE_BATMAN)
#else
# define machine_is_batman()	(0)
#endif

#ifdef CONFIG_MACH_JPD201
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_JPD201
# endif
# define machine_is_jpd201()	(machine_arch_type == MACH_TYPE_JPD201)
#else
# define machine_is_jpd201()	(0)
#endif

#ifdef CONFIG_MACH_MIPSA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MIPSA
# endif
# define machine_is_mipsa()	(machine_arch_type == MACH_TYPE_MIPSA)
#else
# define machine_is_mipsa()	(0)
#endif

#ifdef CONFIG_MACH_KACOM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KACOM
# endif
# define machine_is_kacom()	(machine_arch_type == MACH_TYPE_KACOM)
#else
# define machine_is_kacom()	(0)
#endif

#ifdef CONFIG_MACH_SWARCOCPU
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SWARCOCPU
# endif
# define machine_is_swarcocpu()	(machine_arch_type == MACH_TYPE_SWARCOCPU)
#else
# define machine_is_swarcocpu()	(0)
#endif

#ifdef CONFIG_MACH_SWARCODSL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SWARCODSL
# endif
# define machine_is_swarcodsl()	(machine_arch_type == MACH_TYPE_SWARCODSL)
#else
# define machine_is_swarcodsl()	(0)
#endif

#ifdef CONFIG_MACH_BLUEANGEL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BLUEANGEL
# endif
# define machine_is_blueangel()	(machine_arch_type == MACH_TYPE_BLUEANGEL)
#else
# define machine_is_blueangel()	(0)
#endif

#ifdef CONFIG_MACH_HAIRYGRAMA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HAIRYGRAMA
# endif
# define machine_is_hairygrama()	(machine_arch_type == MACH_TYPE_HAIRYGRAMA)
#else
# define machine_is_hairygrama()	(0)
#endif

#ifdef CONFIG_MACH_BANFF
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BANFF
# endif
# define machine_is_banff()	(machine_arch_type == MACH_TYPE_BANFF)
#else
# define machine_is_banff()	(0)
#endif

#ifdef CONFIG_MACH_CARMEVA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CARMEVA
# endif
# define machine_is_carmeva()	(machine_arch_type == MACH_TYPE_CARMEVA)
#else
# define machine_is_carmeva()	(0)
#endif

#ifdef CONFIG_MACH_SAM255
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SAM255
# endif
# define machine_is_sam255()	(machine_arch_type == MACH_TYPE_SAM255)
#else
# define machine_is_sam255()	(0)
#endif

#ifdef CONFIG_MACH_PPM10
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PPM10
# endif
# define machine_is_ppm10()	(machine_arch_type == MACH_TYPE_PPM10)
#else
# define machine_is_ppm10()	(0)
#endif

#ifdef CONFIG_MACH_EDB9315A
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB9315A
# endif
# define machine_is_edb9315a()	(machine_arch_type == MACH_TYPE_EDB9315A)
#else
# define machine_is_edb9315a()	(0)
#endif

#ifdef CONFIG_MACH_SUNSET
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SUNSET
# endif
# define machine_is_sunset()	(machine_arch_type == MACH_TYPE_SUNSET)
#else
# define machine_is_sunset()	(0)
#endif

#ifdef CONFIG_MACH_STARGATE2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STARGATE2
# endif
# define machine_is_stargate2()	(machine_arch_type == MACH_TYPE_STARGATE2)
#else
# define machine_is_stargate2()	(0)
#endif

#ifdef CONFIG_MACH_INTELMOTE2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_INTELMOTE2
# endif
# define machine_is_intelmote2()	(machine_arch_type == MACH_TYPE_INTELMOTE2)
#else
# define machine_is_intelmote2()	(0)
#endif

#ifdef CONFIG_MACH_TRIZEPS4
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TRIZEPS4
# endif
# define machine_is_trizeps4()	(machine_arch_type == MACH_TYPE_TRIZEPS4)
#else
# define machine_is_trizeps4()	(0)
#endif

#ifdef CONFIG_MACH_MAINSTONE2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MAINSTONE2
# endif
# define machine_is_mainstone2()	(machine_arch_type == MACH_TYPE_MAINSTONE2)
#else
# define machine_is_mainstone2()	(0)
#endif

#ifdef CONFIG_MACH_EZ_IXP42X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EZ_IXP42X
# endif
# define machine_is_ez_ixp42x()	(machine_arch_type == MACH_TYPE_EZ_IXP42X)
#else
# define machine_is_ez_ixp42x()	(0)
#endif

#ifdef CONFIG_MACH_TAPWAVE_ZODIAC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TAPWAVE_ZODIAC
# endif
# define machine_is_tapwave_zodiac()	(machine_arch_type == MACH_TYPE_TAPWAVE_ZODIAC)
#else
# define machine_is_tapwave_zodiac()	(0)
#endif

#ifdef CONFIG_MACH_UNIVERSALMETER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_UNIVERSALMETER
# endif
# define machine_is_universalmeter()	(machine_arch_type == MACH_TYPE_UNIVERSALMETER)
#else
# define machine_is_universalmeter()	(0)
#endif

#ifdef CONFIG_MACH_HICOARM9
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HICOARM9
# endif
# define machine_is_hicoarm9()	(machine_arch_type == MACH_TYPE_HICOARM9)
#else
# define machine_is_hicoarm9()	(0)
#endif

#ifdef CONFIG_MACH_PNX4008
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PNX4008
# endif
# define machine_is_pnx4008()	(machine_arch_type == MACH_TYPE_PNX4008)
#else
# define machine_is_pnx4008()	(0)
#endif

#ifdef CONFIG_MACH_KWS6000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KWS6000
# endif
# define machine_is_kws6000()	(machine_arch_type == MACH_TYPE_KWS6000)
#else
# define machine_is_kws6000()	(0)
#endif

#ifdef CONFIG_MACH_PORTUX920T
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PORTUX920T
# endif
# define machine_is_portux920t()	(machine_arch_type == MACH_TYPE_PORTUX920T)
#else
# define machine_is_portux920t()	(0)
#endif

#ifdef CONFIG_MACH_EZ_X5
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EZ_X5
# endif
# define machine_is_ez_x5()	(machine_arch_type == MACH_TYPE_EZ_X5)
#else
# define machine_is_ez_x5()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_RUDOLPH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_RUDOLPH
# endif
# define machine_is_omap_rudolph()	(machine_arch_type == MACH_TYPE_OMAP_RUDOLPH)
#else
# define machine_is_omap_rudolph()	(0)
#endif

#ifdef CONFIG_MACH_CPUAT91
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CPUAT91
# endif
# define machine_is_cpuat91()	(machine_arch_type == MACH_TYPE_CPUAT91)
#else
# define machine_is_cpuat91()	(0)
#endif

#ifdef CONFIG_MACH_REA9200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_REA9200
# endif
# define machine_is_rea9200()	(machine_arch_type == MACH_TYPE_REA9200)
#else
# define machine_is_rea9200()	(0)
#endif

#ifdef CONFIG_MACH_ACTS_PUNE_SA1110
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACTS_PUNE_SA1110
# endif
# define machine_is_acts_pune_sa1110()	(machine_arch_type == MACH_TYPE_ACTS_PUNE_SA1110)
#else
# define machine_is_acts_pune_sa1110()	(0)
#endif

#ifdef CONFIG_MACH_IXP425
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXP425
# endif
# define machine_is_ixp425()	(machine_arch_type == MACH_TYPE_IXP425)
#else
# define machine_is_ixp425()	(0)
#endif

#ifdef CONFIG_MACH_I30030ADS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_I30030ADS
# endif
# define machine_is_i30030ads()	(machine_arch_type == MACH_TYPE_I30030ADS)
#else
# define machine_is_i30030ads()	(0)
#endif

#ifdef CONFIG_MACH_PERCH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PERCH
# endif
# define machine_is_perch()	(machine_arch_type == MACH_TYPE_PERCH)
#else
# define machine_is_perch()	(0)
#endif

#ifdef CONFIG_MACH_EIS05R1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EIS05R1
# endif
# define machine_is_eis05r1()	(machine_arch_type == MACH_TYPE_EIS05R1)
#else
# define machine_is_eis05r1()	(0)
#endif

#ifdef CONFIG_MACH_PEPPERPAD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PEPPERPAD
# endif
# define machine_is_pepperpad()	(machine_arch_type == MACH_TYPE_PEPPERPAD)
#else
# define machine_is_pepperpad()	(0)
#endif

#ifdef CONFIG_MACH_SB3010
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SB3010
# endif
# define machine_is_sb3010()	(machine_arch_type == MACH_TYPE_SB3010)
#else
# define machine_is_sb3010()	(0)
#endif

#ifdef CONFIG_MACH_RM9200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RM9200
# endif
# define machine_is_rm9200()	(machine_arch_type == MACH_TYPE_RM9200)
#else
# define machine_is_rm9200()	(0)
#endif

#ifdef CONFIG_MACH_DMA03
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DMA03
# endif
# define machine_is_dma03()	(machine_arch_type == MACH_TYPE_DMA03)
#else
# define machine_is_dma03()	(0)
#endif

#ifdef CONFIG_MACH_ROAD_S101
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ROAD_S101
# endif
# define machine_is_road_s101()	(machine_arch_type == MACH_TYPE_ROAD_S101)
#else
# define machine_is_road_s101()	(0)
#endif

#ifdef CONFIG_MACH_IQ_NEXTGEN_A
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IQ_NEXTGEN_A
# endif
# define machine_is_iq_nextgen_a()	(machine_arch_type == MACH_TYPE_IQ_NEXTGEN_A)
#else
# define machine_is_iq_nextgen_a()	(0)
#endif

#ifdef CONFIG_MACH_IQ_NEXTGEN_B
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IQ_NEXTGEN_B
# endif
# define machine_is_iq_nextgen_b()	(machine_arch_type == MACH_TYPE_IQ_NEXTGEN_B)
#else
# define machine_is_iq_nextgen_b()	(0)
#endif

#ifdef CONFIG_MACH_IQ_NEXTGEN_C
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IQ_NEXTGEN_C
# endif
# define machine_is_iq_nextgen_c()	(machine_arch_type == MACH_TYPE_IQ_NEXTGEN_C)
#else
# define machine_is_iq_nextgen_c()	(0)
#endif

#ifdef CONFIG_MACH_IQ_NEXTGEN_D
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IQ_NEXTGEN_D
# endif
# define machine_is_iq_nextgen_d()	(machine_arch_type == MACH_TYPE_IQ_NEXTGEN_D)
#else
# define machine_is_iq_nextgen_d()	(0)
#endif

#ifdef CONFIG_MACH_IQ_NEXTGEN_E
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IQ_NEXTGEN_E
# endif
# define machine_is_iq_nextgen_e()	(machine_arch_type == MACH_TYPE_IQ_NEXTGEN_E)
#else
# define machine_is_iq_nextgen_e()	(0)
#endif

#ifdef CONFIG_MACH_MALLOW_AT91
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MALLOW_AT91
# endif
# define machine_is_mallow_at91()	(machine_arch_type == MACH_TYPE_MALLOW_AT91)
#else
# define machine_is_mallow_at91()	(0)
#endif

#ifdef CONFIG_MACH_CYBERTRACKER_I
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CYBERTRACKER_I
# endif
# define machine_is_cybertracker_i()	(machine_arch_type == MACH_TYPE_CYBERTRACKER_I)
#else
# define machine_is_cybertracker_i()	(0)
#endif

#ifdef CONFIG_MACH_GESBC931X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GESBC931X
# endif
# define machine_is_gesbc931x()	(machine_arch_type == MACH_TYPE_GESBC931X)
#else
# define machine_is_gesbc931x()	(0)
#endif

#ifdef CONFIG_MACH_CENTIPAD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CENTIPAD
# endif
# define machine_is_centipad()	(machine_arch_type == MACH_TYPE_CENTIPAD)
#else
# define machine_is_centipad()	(0)
#endif

#ifdef CONFIG_MACH_ARMSOC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARMSOC
# endif
# define machine_is_armsoc()	(machine_arch_type == MACH_TYPE_ARMSOC)
#else
# define machine_is_armsoc()	(0)
#endif

#ifdef CONFIG_MACH_SE4200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SE4200
# endif
# define machine_is_se4200()	(machine_arch_type == MACH_TYPE_SE4200)
#else
# define machine_is_se4200()	(0)
#endif

#ifdef CONFIG_MACH_EMS197A
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EMS197A
# endif
# define machine_is_ems197a()	(machine_arch_type == MACH_TYPE_EMS197A)
#else
# define machine_is_ems197a()	(0)
#endif

#ifdef CONFIG_MACH_MICRO9
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MICRO9
# endif
# define machine_is_micro9()	(machine_arch_type == MACH_TYPE_MICRO9)
#else
# define machine_is_micro9()	(0)
#endif

#ifdef CONFIG_MACH_MICRO9L
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MICRO9L
# endif
# define machine_is_micro9l()	(machine_arch_type == MACH_TYPE_MICRO9L)
#else
# define machine_is_micro9l()	(0)
#endif

#ifdef CONFIG_MACH_UC5471DSP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_UC5471DSP
# endif
# define machine_is_uc5471dsp()	(machine_arch_type == MACH_TYPE_UC5471DSP)
#else
# define machine_is_uc5471dsp()	(0)
#endif

#ifdef CONFIG_MACH_SJ5471ENG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SJ5471ENG
# endif
# define machine_is_sj5471eng()	(machine_arch_type == MACH_TYPE_SJ5471ENG)
#else
# define machine_is_sj5471eng()	(0)
#endif

#ifdef CONFIG_MACH_CMPXA26X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CMPXA26X
# endif
# define machine_is_none()	(machine_arch_type == MACH_TYPE_CMPXA26X)
#else
# define machine_is_none()	(0)
#endif

#ifdef CONFIG_MACH_NC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NC
# endif
# define machine_is_nc1()	(machine_arch_type == MACH_TYPE_NC)
#else
# define machine_is_nc1()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_PALMTE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_PALMTE
# endif
# define machine_is_omap_palmte()	(machine_arch_type == MACH_TYPE_OMAP_PALMTE)
#else
# define machine_is_omap_palmte()	(0)
#endif

#ifdef CONFIG_MACH_AJAX52X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AJAX52X
# endif
# define machine_is_ajax52x()	(machine_arch_type == MACH_TYPE_AJAX52X)
#else
# define machine_is_ajax52x()	(0)
#endif

#ifdef CONFIG_MACH_SIRIUSTAR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SIRIUSTAR
# endif
# define machine_is_siriustar()	(machine_arch_type == MACH_TYPE_SIRIUSTAR)
#else
# define machine_is_siriustar()	(0)
#endif

#ifdef CONFIG_MACH_IODATA_HDLG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IODATA_HDLG
# endif
# define machine_is_iodata_hdlg()	(machine_arch_type == MACH_TYPE_IODATA_HDLG)
#else
# define machine_is_iodata_hdlg()	(0)
#endif

#ifdef CONFIG_MACH_AT91RM9200UTL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91RM9200UTL
# endif
# define machine_is_at91rm9200utl()	(machine_arch_type == MACH_TYPE_AT91RM9200UTL)
#else
# define machine_is_at91rm9200utl()	(0)
#endif

#ifdef CONFIG_MACH_BIOSAFE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BIOSAFE
# endif
# define machine_is_biosafe()	(machine_arch_type == MACH_TYPE_BIOSAFE)
#else
# define machine_is_biosafe()	(0)
#endif

#ifdef CONFIG_MACH_MP1000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MP1000
# endif
# define machine_is_mp1000()	(machine_arch_type == MACH_TYPE_MP1000)
#else
# define machine_is_mp1000()	(0)
#endif

#ifdef CONFIG_MACH_PARSY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PARSY
# endif
# define machine_is_parsy()	(machine_arch_type == MACH_TYPE_PARSY)
#else
# define machine_is_parsy()	(0)
#endif

#ifdef CONFIG_MACH_CCXP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CCXP
# endif
# define machine_is_ccxp270()	(machine_arch_type == MACH_TYPE_CCXP)
#else
# define machine_is_ccxp270()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_GSAMPLE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_GSAMPLE
# endif
# define machine_is_omap_gsample()	(machine_arch_type == MACH_TYPE_OMAP_GSAMPLE)
#else
# define machine_is_omap_gsample()	(0)
#endif

#ifdef CONFIG_MACH_REALVIEW_EB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_REALVIEW_EB
# endif
# define machine_is_realview_eb()	(machine_arch_type == MACH_TYPE_REALVIEW_EB)
#else
# define machine_is_realview_eb()	(0)
#endif

#ifdef CONFIG_MACH_SAMOA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SAMOA
# endif
# define machine_is_samoa()	(machine_arch_type == MACH_TYPE_SAMOA)
#else
# define machine_is_samoa()	(0)
#endif

#ifdef CONFIG_MACH_T3XSCALE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_T3XSCALE
# endif
# define machine_is_t3xscale()	(machine_arch_type == MACH_TYPE_T3XSCALE)
#else
# define machine_is_t3xscale()	(0)
#endif

#ifdef CONFIG_MACH_I878
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_I878
# endif
# define machine_is_i878()	(machine_arch_type == MACH_TYPE_I878)
#else
# define machine_is_i878()	(0)
#endif

#ifdef CONFIG_MACH_BORZOI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BORZOI
# endif
# define machine_is_borzoi()	(machine_arch_type == MACH_TYPE_BORZOI)
#else
# define machine_is_borzoi()	(0)
#endif

#ifdef CONFIG_MACH_GECKO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GECKO
# endif
# define machine_is_gecko()	(machine_arch_type == MACH_TYPE_GECKO)
#else
# define machine_is_gecko()	(0)
#endif

#ifdef CONFIG_MACH_DS101
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DS101
# endif
# define machine_is_ds101()	(machine_arch_type == MACH_TYPE_DS101)
#else
# define machine_is_ds101()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_PALMTT2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_PALMTT2
# endif
# define machine_is_omap_palmtt2()	(machine_arch_type == MACH_TYPE_OMAP_PALMTT2)
#else
# define machine_is_omap_palmtt2()	(0)
#endif

#ifdef CONFIG_MACH_XSCALE_PALMLD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XSCALE_PALMLD
# endif
# define machine_is_xscale_palmld()	(machine_arch_type == MACH_TYPE_XSCALE_PALMLD)
#else
# define machine_is_xscale_palmld()	(0)
#endif

#ifdef CONFIG_MACH_CC9C
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CC9C
# endif
# define machine_is_cc9c()	(machine_arch_type == MACH_TYPE_CC9C)
#else
# define machine_is_cc9c()	(0)
#endif

#ifdef CONFIG_MACH_SBC1670
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SBC1670
# endif
# define machine_is_sbc1670()	(machine_arch_type == MACH_TYPE_SBC1670)
#else
# define machine_is_sbc1670()	(0)
#endif

#ifdef CONFIG_MACH_IXDP28X5
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDP28X5
# endif
# define machine_is_ixdp28x5()	(machine_arch_type == MACH_TYPE_IXDP28X5)
#else
# define machine_is_ixdp28x5()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_PALMTT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_PALMTT
# endif
# define machine_is_omap_palmtt()	(machine_arch_type == MACH_TYPE_OMAP_PALMTT)
#else
# define machine_is_omap_palmtt()	(0)
#endif

#ifdef CONFIG_MACH_ML696K
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ML696K
# endif
# define machine_is_ml696k()	(machine_arch_type == MACH_TYPE_ML696K)
#else
# define machine_is_ml696k()	(0)
#endif

#ifdef CONFIG_MACH_ARCOM_ZEUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARCOM_ZEUS
# endif
# define machine_is_arcom_zeus()	(machine_arch_type == MACH_TYPE_ARCOM_ZEUS)
#else
# define machine_is_arcom_zeus()	(0)
#endif

#ifdef CONFIG_MACH_OSIRIS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OSIRIS
# endif
# define machine_is_osiris()	(machine_arch_type == MACH_TYPE_OSIRIS)
#else
# define machine_is_osiris()	(0)
#endif

#ifdef CONFIG_MACH_MAESTRO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MAESTRO
# endif
# define machine_is_maestro()	(machine_arch_type == MACH_TYPE_MAESTRO)
#else
# define machine_is_maestro()	(0)
#endif

#ifdef CONFIG_MACH_TUNGE2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TUNGE2
# endif
# define machine_is_tunge2()	(machine_arch_type == MACH_TYPE_TUNGE2)
#else
# define machine_is_tunge2()	(0)
#endif

#ifdef CONFIG_MACH_IXBBM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXBBM
# endif
# define machine_is_ixbbm()	(machine_arch_type == MACH_TYPE_IXBBM)
#else
# define machine_is_ixbbm()	(0)
#endif

#ifdef CONFIG_MACH_MX27ADS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX27ADS
# endif
# define machine_is_mx27ads()	(machine_arch_type == MACH_TYPE_MX27ADS)
#else
# define machine_is_mx27ads()	(0)
#endif

#ifdef CONFIG_MACH_AX8004
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AX8004
# endif
# define machine_is_ax8004()	(machine_arch_type == MACH_TYPE_AX8004)
#else
# define machine_is_ax8004()	(0)
#endif

#ifdef CONFIG_MACH_AT91SAM9261EK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91SAM9261EK
# endif
# define machine_is_at91sam9261ek()	(machine_arch_type == MACH_TYPE_AT91SAM9261EK)
#else
# define machine_is_at91sam9261ek()	(0)
#endif

#ifdef CONFIG_MACH_LOFT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LOFT
# endif
# define machine_is_loft()	(machine_arch_type == MACH_TYPE_LOFT)
#else
# define machine_is_loft()	(0)
#endif

#ifdef CONFIG_MACH_MAGPIE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MAGPIE
# endif
# define machine_is_magpie()	(machine_arch_type == MACH_TYPE_MAGPIE)
#else
# define machine_is_magpie()	(0)
#endif

#ifdef CONFIG_MACH_MX21ADS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX21ADS
# endif
# define machine_is_mx21ads()	(machine_arch_type == MACH_TYPE_MX21ADS)
#else
# define machine_is_mx21ads()	(0)
#endif

#ifdef CONFIG_MACH_MB87M3400
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MB87M3400
# endif
# define machine_is_mb87m3400()	(machine_arch_type == MACH_TYPE_MB87M3400)
#else
# define machine_is_mb87m3400()	(0)
#endif

#ifdef CONFIG_MACH_MGUARD_DELTA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MGUARD_DELTA
# endif
# define machine_is_mguard_delta()	(machine_arch_type == MACH_TYPE_MGUARD_DELTA)
#else
# define machine_is_mguard_delta()	(0)
#endif

#ifdef CONFIG_MACH_DAVINCI_DVDP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DAVINCI_DVDP
# endif
# define machine_is_davinci_dvdp()	(machine_arch_type == MACH_TYPE_DAVINCI_DVDP)
#else
# define machine_is_davinci_dvdp()	(0)
#endif

#ifdef CONFIG_MACH_HTCUNIVERSAL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HTCUNIVERSAL
# endif
# define machine_is_htcuniversal()	(machine_arch_type == MACH_TYPE_HTCUNIVERSAL)
#else
# define machine_is_htcuniversal()	(0)
#endif

#ifdef CONFIG_MACH_TPAD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TPAD
# endif
# define machine_is_tpad()	(machine_arch_type == MACH_TYPE_TPAD)
#else
# define machine_is_tpad()	(0)
#endif

#ifdef CONFIG_MACH_ROVERP3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ROVERP3
# endif
# define machine_is_roverp3()	(machine_arch_type == MACH_TYPE_ROVERP3)
#else
# define machine_is_roverp3()	(0)
#endif

#ifdef CONFIG_MACH_JORNADA928
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_JORNADA928
# endif
# define machine_is_jornada928()	(machine_arch_type == MACH_TYPE_JORNADA928)
#else
# define machine_is_jornada928()	(0)
#endif

#ifdef CONFIG_MACH_MV88FXX81
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MV88FXX81
# endif
# define machine_is_mv88fxx81()	(machine_arch_type == MACH_TYPE_MV88FXX81)
#else
# define machine_is_mv88fxx81()	(0)
#endif

#ifdef CONFIG_MACH_STMP36XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STMP36XX
# endif
# define machine_is_stmp36xx()	(machine_arch_type == MACH_TYPE_STMP36XX)
#else
# define machine_is_stmp36xx()	(0)
#endif

#ifdef CONFIG_MACH_SXNI79524
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SXNI79524
# endif
# define machine_is_sxni79524()	(machine_arch_type == MACH_TYPE_SXNI79524)
#else
# define machine_is_sxni79524()	(0)
#endif

#ifdef CONFIG_MACH_AMS_DELTA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AMS_DELTA
# endif
# define machine_is_ams_delta()	(machine_arch_type == MACH_TYPE_AMS_DELTA)
#else
# define machine_is_ams_delta()	(0)
#endif

#ifdef CONFIG_MACH_URANIUM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_URANIUM
# endif
# define machine_is_uranium()	(machine_arch_type == MACH_TYPE_URANIUM)
#else
# define machine_is_uranium()	(0)
#endif

#ifdef CONFIG_MACH_UCON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_UCON
# endif
# define machine_is_ucon()	(machine_arch_type == MACH_TYPE_UCON)
#else
# define machine_is_ucon()	(0)
#endif

#ifdef CONFIG_MACH_NAS100D
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NAS100D
# endif
# define machine_is_nas100d()	(machine_arch_type == MACH_TYPE_NAS100D)
#else
# define machine_is_nas100d()	(0)
#endif

#ifdef CONFIG_MACH_L083_1000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_L083_1000
# endif
# define machine_is_l083()	(machine_arch_type == MACH_TYPE_L083_1000)
#else
# define machine_is_l083()	(0)
#endif

#ifdef CONFIG_MACH_EZX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EZX
# endif
# define machine_is_ezx()	(machine_arch_type == MACH_TYPE_EZX)
#else
# define machine_is_ezx()	(0)
#endif

#ifdef CONFIG_MACH_PNX5220
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PNX5220
# endif
# define machine_is_pnx5220()	(machine_arch_type == MACH_TYPE_PNX5220)
#else
# define machine_is_pnx5220()	(0)
#endif

#ifdef CONFIG_MACH_BUTTE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BUTTE
# endif
# define machine_is_butte()	(machine_arch_type == MACH_TYPE_BUTTE)
#else
# define machine_is_butte()	(0)
#endif

#ifdef CONFIG_MACH_SRM2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SRM2
# endif
# define machine_is_srm2()	(machine_arch_type == MACH_TYPE_SRM2)
#else
# define machine_is_srm2()	(0)
#endif

#ifdef CONFIG_MACH_DSBR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DSBR
# endif
# define machine_is_dsbr()	(machine_arch_type == MACH_TYPE_DSBR)
#else
# define machine_is_dsbr()	(0)
#endif

#ifdef CONFIG_MACH_CRYSTALBALL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CRYSTALBALL
# endif
# define machine_is_crystalball()	(machine_arch_type == MACH_TYPE_CRYSTALBALL)
#else
# define machine_is_crystalball()	(0)
#endif

#ifdef CONFIG_MACH_TINYPXA27X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TINYPXA27X
# endif
# define machine_is_tinypxa27x()	(machine_arch_type == MACH_TYPE_TINYPXA27X)
#else
# define machine_is_tinypxa27x()	(0)
#endif

#ifdef CONFIG_MACH_HERBIE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HERBIE
# endif
# define machine_is_herbie()	(machine_arch_type == MACH_TYPE_HERBIE)
#else
# define machine_is_herbie()	(0)
#endif

#ifdef CONFIG_MACH_MAGICIAN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MAGICIAN
# endif
# define machine_is_magician()	(machine_arch_type == MACH_TYPE_MAGICIAN)
#else
# define machine_is_magician()	(0)
#endif

#ifdef CONFIG_MACH_CM4002
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CM4002
# endif
# define machine_is_cm4002()	(machine_arch_type == MACH_TYPE_CM4002)
#else
# define machine_is_cm4002()	(0)
#endif

#ifdef CONFIG_MACH_B4
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_B4
# endif
# define machine_is_b4()	(machine_arch_type == MACH_TYPE_B4)
#else
# define machine_is_b4()	(0)
#endif

#ifdef CONFIG_MACH_MAUI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MAUI
# endif
# define machine_is_maui()	(machine_arch_type == MACH_TYPE_MAUI)
#else
# define machine_is_maui()	(0)
#endif

#ifdef CONFIG_MACH_CYBERTRACKER_G
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CYBERTRACKER_G
# endif
# define machine_is_cybertracker_g()	(machine_arch_type == MACH_TYPE_CYBERTRACKER_G)
#else
# define machine_is_cybertracker_g()	(0)
#endif

#ifdef CONFIG_MACH_NXDKN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NXDKN
# endif
# define machine_is_nxdkn()	(machine_arch_type == MACH_TYPE_NXDKN)
#else
# define machine_is_nxdkn()	(0)
#endif

#ifdef CONFIG_MACH_MIO8390
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MIO8390
# endif
# define machine_is_mio8390()	(machine_arch_type == MACH_TYPE_MIO8390)
#else
# define machine_is_mio8390()	(0)
#endif

#ifdef CONFIG_MACH_OMI_BOARD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMI_BOARD
# endif
# define machine_is_omi_board()	(machine_arch_type == MACH_TYPE_OMI_BOARD)
#else
# define machine_is_omi_board()	(0)
#endif

#ifdef CONFIG_MACH_MX21CIV
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX21CIV
# endif
# define machine_is_mx21civ()	(machine_arch_type == MACH_TYPE_MX21CIV)
#else
# define machine_is_mx21civ()	(0)
#endif

#ifdef CONFIG_MACH_MAHI_CDAC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MAHI_CDAC
# endif
# define machine_is_mahi_cdac()	(machine_arch_type == MACH_TYPE_MAHI_CDAC)
#else
# define machine_is_mahi_cdac()	(0)
#endif

#ifdef CONFIG_MACH_XSCALE_PALMTX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XSCALE_PALMTX
# endif
# define machine_is_xscale_palmtx()	(machine_arch_type == MACH_TYPE_XSCALE_PALMTX)
#else
# define machine_is_xscale_palmtx()	(0)
#endif

#ifdef CONFIG_MACH_S3C2413
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S3C2413
# endif
# define machine_is_s3c2413()	(machine_arch_type == MACH_TYPE_S3C2413)
#else
# define machine_is_s3c2413()	(0)
#endif

#ifdef CONFIG_MACH_SAMSYS_EP0
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SAMSYS_EP0
# endif
# define machine_is_samsys_ep0()	(machine_arch_type == MACH_TYPE_SAMSYS_EP0)
#else
# define machine_is_samsys_ep0()	(0)
#endif

#ifdef CONFIG_MACH_WG302V1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WG302V1
# endif
# define machine_is_wg302v1()	(machine_arch_type == MACH_TYPE_WG302V1)
#else
# define machine_is_wg302v1()	(0)
#endif

#ifdef CONFIG_MACH_WG302V2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WG302V2
# endif
# define machine_is_wg302v2()	(machine_arch_type == MACH_TYPE_WG302V2)
#else
# define machine_is_wg302v2()	(0)
#endif

#ifdef CONFIG_MACH_EB42X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EB42X
# endif
# define machine_is_eb42x()	(machine_arch_type == MACH_TYPE_EB42X)
#else
# define machine_is_eb42x()	(0)
#endif

#ifdef CONFIG_MACH_IQ331ES
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IQ331ES
# endif
# define machine_is_iq331es()	(machine_arch_type == MACH_TYPE_IQ331ES)
#else
# define machine_is_iq331es()	(0)
#endif

#ifdef CONFIG_MACH_COSYDSP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COSYDSP
# endif
# define machine_is_cosydsp()	(machine_arch_type == MACH_TYPE_COSYDSP)
#else
# define machine_is_cosydsp()	(0)
#endif

#ifdef CONFIG_MACH_UPLAT7D
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_UPLAT7D
# endif
# define machine_is_uplat7d_proto()	(machine_arch_type == MACH_TYPE_UPLAT7D)
#else
# define machine_is_uplat7d_proto()	(0)
#endif

#ifdef CONFIG_MACH_PTDAVINCI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PTDAVINCI
# endif
# define machine_is_ptdavinci()	(machine_arch_type == MACH_TYPE_PTDAVINCI)
#else
# define machine_is_ptdavinci()	(0)
#endif

#ifdef CONFIG_MACH_MBUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MBUS
# endif
# define machine_is_mbus()	(machine_arch_type == MACH_TYPE_MBUS)
#else
# define machine_is_mbus()	(0)
#endif

#ifdef CONFIG_MACH_NADIA2VB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NADIA2VB
# endif
# define machine_is_nadia2vb()	(machine_arch_type == MACH_TYPE_NADIA2VB)
#else
# define machine_is_nadia2vb()	(0)
#endif

#ifdef CONFIG_MACH_R1000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_R1000
# endif
# define machine_is_r1000()	(machine_arch_type == MACH_TYPE_R1000)
#else
# define machine_is_r1000()	(0)
#endif

#ifdef CONFIG_MACH_HW90250
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HW90250
# endif
# define machine_is_hw90250()	(machine_arch_type == MACH_TYPE_HW90250)
#else
# define machine_is_hw90250()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_2430SDP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_2430SDP
# endif
# define machine_is_omap_2430sdp()	(machine_arch_type == MACH_TYPE_OMAP_2430SDP)
#else
# define machine_is_omap_2430sdp()	(0)
#endif

#ifdef CONFIG_MACH_DAVINCI_EVM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DAVINCI_EVM
# endif
# define machine_is_davinci_evm()	(machine_arch_type == MACH_TYPE_DAVINCI_EVM)
#else
# define machine_is_davinci_evm()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_TORNADO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_TORNADO
# endif
# define machine_is_omap_tornado()	(machine_arch_type == MACH_TYPE_OMAP_TORNADO)
#else
# define machine_is_omap_tornado()	(0)
#endif

#ifdef CONFIG_MACH_OLOCREEK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OLOCREEK
# endif
# define machine_is_olocreek()	(machine_arch_type == MACH_TYPE_OLOCREEK)
#else
# define machine_is_olocreek()	(0)
#endif

#ifdef CONFIG_MACH_PALMZ72
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PALMZ72
# endif
# define machine_is_palmz72()	(machine_arch_type == MACH_TYPE_PALMZ72)
#else
# define machine_is_palmz72()	(0)
#endif

#ifdef CONFIG_MACH_NXDB500
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NXDB500
# endif
# define machine_is_nxdb500()	(machine_arch_type == MACH_TYPE_NXDB500)
#else
# define machine_is_nxdb500()	(0)
#endif

#ifdef CONFIG_MACH_APF9328
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_APF9328
# endif
# define machine_is_apf9328()	(machine_arch_type == MACH_TYPE_APF9328)
#else
# define machine_is_apf9328()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_WIPOQ
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_WIPOQ
# endif
# define machine_is_omap_wipoq()	(machine_arch_type == MACH_TYPE_OMAP_WIPOQ)
#else
# define machine_is_omap_wipoq()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_TWIP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_TWIP
# endif
# define machine_is_omap_twip()	(machine_arch_type == MACH_TYPE_OMAP_TWIP)
#else
# define machine_is_omap_twip()	(0)
#endif

#ifdef CONFIG_MACH_XSCALE_PALMTREO650
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XSCALE_PALMTREO650
# endif
# define machine_is_xscale_treo650()	(machine_arch_type == MACH_TYPE_XSCALE_PALMTREO650)
#else
# define machine_is_xscale_treo650()	(0)
#endif

#ifdef CONFIG_MACH_ACUMEN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACUMEN
# endif
# define machine_is_acumen()	(machine_arch_type == MACH_TYPE_ACUMEN)
#else
# define machine_is_acumen()	(0)
#endif

#ifdef CONFIG_MACH_XP100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XP100
# endif
# define machine_is_xp100()	(machine_arch_type == MACH_TYPE_XP100)
#else
# define machine_is_xp100()	(0)
#endif

#ifdef CONFIG_MACH_FS2410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FS2410
# endif
# define machine_is_fs2410()	(machine_arch_type == MACH_TYPE_FS2410)
#else
# define machine_is_fs2410()	(0)
#endif

#ifdef CONFIG_MACH_PXA270_CERF
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA270_CERF
# endif
# define machine_is_pxa270_cerf()	(machine_arch_type == MACH_TYPE_PXA270_CERF)
#else
# define machine_is_pxa270_cerf()	(0)
#endif

#ifdef CONFIG_MACH_SQ2FTLPALM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SQ2FTLPALM
# endif
# define machine_is_sq2ftlpalm()	(machine_arch_type == MACH_TYPE_SQ2FTLPALM)
#else
# define machine_is_sq2ftlpalm()	(0)
#endif

#ifdef CONFIG_MACH_BSEMSERVER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BSEMSERVER
# endif
# define machine_is_bsemserver()	(machine_arch_type == MACH_TYPE_BSEMSERVER)
#else
# define machine_is_bsemserver()	(0)
#endif

#ifdef CONFIG_MACH_NETCLIENT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NETCLIENT
# endif
# define machine_is_netclient()	(machine_arch_type == MACH_TYPE_NETCLIENT)
#else
# define machine_is_netclient()	(0)
#endif

#ifdef CONFIG_MACH_XSCALE_PALMTT5
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XSCALE_PALMTT5
# endif
# define machine_is_xscale_palmtt5()	(machine_arch_type == MACH_TYPE_XSCALE_PALMTT5)
#else
# define machine_is_xscale_palmtt5()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_PALMTC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_PALMTC
# endif
# define machine_is_xscale_palmtc()	(machine_arch_type == MACH_TYPE_OMAP_PALMTC)
#else
# define machine_is_xscale_palmtc()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_APOLLON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_APOLLON
# endif
# define machine_is_omap_apollon()	(machine_arch_type == MACH_TYPE_OMAP_APOLLON)
#else
# define machine_is_omap_apollon()	(0)
#endif

#ifdef CONFIG_MACH_MXC30030EVB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MXC30030EVB
# endif
# define machine_is_mxc30030evb()	(machine_arch_type == MACH_TYPE_MXC30030EVB)
#else
# define machine_is_mxc30030evb()	(0)
#endif

#ifdef CONFIG_MACH_REA_2D
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_REA_2D
# endif
# define machine_is_rea_2d()	(machine_arch_type == MACH_TYPE_REA_2D)
#else
# define machine_is_rea_2d()	(0)
#endif

#ifdef CONFIG_MACH_TI3E524
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TI3E524
# endif
# define machine_is_eti3e524()	(machine_arch_type == MACH_TYPE_TI3E524)
#else
# define machine_is_eti3e524()	(0)
#endif

#ifdef CONFIG_MACH_ATEB9200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ATEB9200
# endif
# define machine_is_ateb9200()	(machine_arch_type == MACH_TYPE_ATEB9200)
#else
# define machine_is_ateb9200()	(0)
#endif

#ifdef CONFIG_MACH_AUCKLAND
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AUCKLAND
# endif
# define machine_is_auckland()	(machine_arch_type == MACH_TYPE_AUCKLAND)
#else
# define machine_is_auckland()	(0)
#endif

#ifdef CONFIG_MACH_AK3320M
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AK3320M
# endif
# define machine_is_ak3220m()	(machine_arch_type == MACH_TYPE_AK3320M)
#else
# define machine_is_ak3220m()	(0)
#endif

#ifdef CONFIG_MACH_DURAMAX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DURAMAX
# endif
# define machine_is_duramax()	(machine_arch_type == MACH_TYPE_DURAMAX)
#else
# define machine_is_duramax()	(0)
#endif

#ifdef CONFIG_MACH_N35
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_N35
# endif
# define machine_is_n35()	(machine_arch_type == MACH_TYPE_N35)
#else
# define machine_is_n35()	(0)
#endif

#ifdef CONFIG_MACH_PRONGHORN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PRONGHORN
# endif
# define machine_is_pronghorn()	(machine_arch_type == MACH_TYPE_PRONGHORN)
#else
# define machine_is_pronghorn()	(0)
#endif

#ifdef CONFIG_MACH_FUNDY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FUNDY
# endif
# define machine_is_fundy()	(machine_arch_type == MACH_TYPE_FUNDY)
#else
# define machine_is_fundy()	(0)
#endif

#ifdef CONFIG_MACH_LOGICPD_PXA270
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LOGICPD_PXA270
# endif
# define machine_is_logicpd_pxa270()	(machine_arch_type == MACH_TYPE_LOGICPD_PXA270)
#else
# define machine_is_logicpd_pxa270()	(0)
#endif

#ifdef CONFIG_MACH_CPU777
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CPU777
# endif
# define machine_is_cpu777()	(machine_arch_type == MACH_TYPE_CPU777)
#else
# define machine_is_cpu777()	(0)
#endif

#ifdef CONFIG_MACH_SIMICON9201
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SIMICON9201
# endif
# define machine_is_simicon9201()	(machine_arch_type == MACH_TYPE_SIMICON9201)
#else
# define machine_is_simicon9201()	(0)
#endif

#ifdef CONFIG_MACH_LEAP2_HPM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LEAP2_HPM
# endif
# define machine_is_leap2_hpm()	(machine_arch_type == MACH_TYPE_LEAP2_HPM)
#else
# define machine_is_leap2_hpm()	(0)
#endif

#ifdef CONFIG_MACH_CM922TXA10
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CM922TXA10
# endif
# define machine_is_cm922txa10()	(machine_arch_type == MACH_TYPE_CM922TXA10)
#else
# define machine_is_cm922txa10()	(0)
#endif

#ifdef CONFIG_MACH_PXA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA
# endif
# define machine_is_sandgate()	(machine_arch_type == MACH_TYPE_PXA)
#else
# define machine_is_sandgate()	(0)
#endif

#ifdef CONFIG_MACH_SANDGATE2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SANDGATE2
# endif
# define machine_is_sandgate2()	(machine_arch_type == MACH_TYPE_SANDGATE2)
#else
# define machine_is_sandgate2()	(0)
#endif

#ifdef CONFIG_MACH_SANDGATE2G
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SANDGATE2G
# endif
# define machine_is_sandgate2g()	(machine_arch_type == MACH_TYPE_SANDGATE2G)
#else
# define machine_is_sandgate2g()	(0)
#endif

#ifdef CONFIG_MACH_SANDGATE2P
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SANDGATE2P
# endif
# define machine_is_sandgate2p()	(machine_arch_type == MACH_TYPE_SANDGATE2P)
#else
# define machine_is_sandgate2p()	(0)
#endif

#ifdef CONFIG_MACH_FRED_JACK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FRED_JACK
# endif
# define machine_is_fred_jack()	(machine_arch_type == MACH_TYPE_FRED_JACK)
#else
# define machine_is_fred_jack()	(0)
#endif

#ifdef CONFIG_MACH_TTG_COLOR1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TTG_COLOR1
# endif
# define machine_is_ttg_color1()	(machine_arch_type == MACH_TYPE_TTG_COLOR1)
#else
# define machine_is_ttg_color1()	(0)
#endif

#ifdef CONFIG_MACH_NXEB500HMI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NXEB500HMI
# endif
# define machine_is_nxeb500hmi()	(machine_arch_type == MACH_TYPE_NXEB500HMI)
#else
# define machine_is_nxeb500hmi()	(0)
#endif

#ifdef CONFIG_MACH_NETDCU8
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NETDCU8
# endif
# define machine_is_netdcu8()	(machine_arch_type == MACH_TYPE_NETDCU8)
#else
# define machine_is_netdcu8()	(0)
#endif

#ifdef CONFIG_MACH_ML675050_CPU_BOA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ML675050_CPU_BOA
# endif
# define machine_is_ml675050_cpu_boa()	(machine_arch_type == MACH_TYPE_ML675050_CPU_BOA)
#else
# define machine_is_ml675050_cpu_boa()	(0)
#endif

#ifdef CONFIG_MACH_NG_FVX538
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NG_FVX538
# endif
# define machine_is_ng_fvx538()	(machine_arch_type == MACH_TYPE_NG_FVX538)
#else
# define machine_is_ng_fvx538()	(0)
#endif

#ifdef CONFIG_MACH_NG_FVS338
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NG_FVS338
# endif
# define machine_is_ng_fvs338()	(machine_arch_type == MACH_TYPE_NG_FVS338)
#else
# define machine_is_ng_fvs338()	(0)
#endif

#ifdef CONFIG_MACH_PNX4103
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PNX4103
# endif
# define machine_is_pnx4103()	(machine_arch_type == MACH_TYPE_PNX4103)
#else
# define machine_is_pnx4103()	(0)
#endif

#ifdef CONFIG_MACH_HESDB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HESDB
# endif
# define machine_is_hesdb()	(machine_arch_type == MACH_TYPE_HESDB)
#else
# define machine_is_hesdb()	(0)
#endif

#ifdef CONFIG_MACH_XSILO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XSILO
# endif
# define machine_is_xsilo()	(machine_arch_type == MACH_TYPE_XSILO)
#else
# define machine_is_xsilo()	(0)
#endif

#ifdef CONFIG_MACH_ESPRESSO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESPRESSO
# endif
# define machine_is_espresso()	(machine_arch_type == MACH_TYPE_ESPRESSO)
#else
# define machine_is_espresso()	(0)
#endif

#ifdef CONFIG_MACH_EMLC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EMLC
# endif
# define machine_is_emlc()	(machine_arch_type == MACH_TYPE_EMLC)
#else
# define machine_is_emlc()	(0)
#endif

#ifdef CONFIG_MACH_SISTERON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SISTERON
# endif
# define machine_is_sisteron()	(machine_arch_type == MACH_TYPE_SISTERON)
#else
# define machine_is_sisteron()	(0)
#endif

#ifdef CONFIG_MACH_RX1950
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RX1950
# endif
# define machine_is_rx1950()	(machine_arch_type == MACH_TYPE_RX1950)
#else
# define machine_is_rx1950()	(0)
#endif

#ifdef CONFIG_MACH_TSC_VENUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TSC_VENUS
# endif
# define machine_is_tsc_venus()	(machine_arch_type == MACH_TYPE_TSC_VENUS)
#else
# define machine_is_tsc_venus()	(0)
#endif

#ifdef CONFIG_MACH_DS101J
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DS101J
# endif
# define machine_is_ds101j()	(machine_arch_type == MACH_TYPE_DS101J)
#else
# define machine_is_ds101j()	(0)
#endif

#ifdef CONFIG_MACH_MXC30030ADS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MXC30030ADS
# endif
# define machine_is_mxc30030ads()	(machine_arch_type == MACH_TYPE_MXC30030ADS)
#else
# define machine_is_mxc30030ads()	(0)
#endif

#ifdef CONFIG_MACH_FUJITSU_WIMAXSOC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FUJITSU_WIMAXSOC
# endif
# define machine_is_fujitsu_wimaxsoc()	(machine_arch_type == MACH_TYPE_FUJITSU_WIMAXSOC)
#else
# define machine_is_fujitsu_wimaxsoc()	(0)
#endif

#ifdef CONFIG_MACH_DUALPCMODEM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DUALPCMODEM
# endif
# define machine_is_dualpcmodem()	(machine_arch_type == MACH_TYPE_DUALPCMODEM)
#else
# define machine_is_dualpcmodem()	(0)
#endif

#ifdef CONFIG_MACH_GESBC9312
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GESBC9312
# endif
# define machine_is_gesbc9312()	(machine_arch_type == MACH_TYPE_GESBC9312)
#else
# define machine_is_gesbc9312()	(0)
#endif

#ifdef CONFIG_MACH_HTCAPACHE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HTCAPACHE
# endif
# define machine_is_htcapache()	(machine_arch_type == MACH_TYPE_HTCAPACHE)
#else
# define machine_is_htcapache()	(0)
#endif

#ifdef CONFIG_MACH_IXDP435
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDP435
# endif
# define machine_is_ixdp435()	(machine_arch_type == MACH_TYPE_IXDP435)
#else
# define machine_is_ixdp435()	(0)
#endif

#ifdef CONFIG_MACH_CATPROVT100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CATPROVT100
# endif
# define machine_is_catprovt100()	(machine_arch_type == MACH_TYPE_CATPROVT100)
#else
# define machine_is_catprovt100()	(0)
#endif

#ifdef CONFIG_MACH_PICOTUX1XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PICOTUX1XX
# endif
# define machine_is_picotux1xx()	(machine_arch_type == MACH_TYPE_PICOTUX1XX)
#else
# define machine_is_picotux1xx()	(0)
#endif

#ifdef CONFIG_MACH_PICOTUX2XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PICOTUX2XX
# endif
# define machine_is_picotux2xx()	(machine_arch_type == MACH_TYPE_PICOTUX2XX)
#else
# define machine_is_picotux2xx()	(0)
#endif

#ifdef CONFIG_MACH_DSMG600
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DSMG600
# endif
# define machine_is_dsmg600()	(machine_arch_type == MACH_TYPE_DSMG600)
#else
# define machine_is_dsmg600()	(0)
#endif

#ifdef CONFIG_MACH_EMPC2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EMPC2
# endif
# define machine_is_empc2()	(machine_arch_type == MACH_TYPE_EMPC2)
#else
# define machine_is_empc2()	(0)
#endif

#ifdef CONFIG_MACH_VENTURA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VENTURA
# endif
# define machine_is_ventura()	(machine_arch_type == MACH_TYPE_VENTURA)
#else
# define machine_is_ventura()	(0)
#endif

#ifdef CONFIG_MACH_PHIDGET_SBC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PHIDGET_SBC
# endif
# define machine_is_phidget_sbc()	(machine_arch_type == MACH_TYPE_PHIDGET_SBC)
#else
# define machine_is_phidget_sbc()	(0)
#endif

#ifdef CONFIG_MACH_IJ3K
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IJ3K
# endif
# define machine_is_ij3k()	(machine_arch_type == MACH_TYPE_IJ3K)
#else
# define machine_is_ij3k()	(0)
#endif

#ifdef CONFIG_MACH_PISGAH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PISGAH
# endif
# define machine_is_pisgah()	(machine_arch_type == MACH_TYPE_PISGAH)
#else
# define machine_is_pisgah()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_FSAMPLE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_FSAMPLE
# endif
# define machine_is_omap_fsample()	(machine_arch_type == MACH_TYPE_OMAP_FSAMPLE)
#else
# define machine_is_omap_fsample()	(0)
#endif

#ifdef CONFIG_MACH_SG720
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SG720
# endif
# define machine_is_sg720()	(machine_arch_type == MACH_TYPE_SG720)
#else
# define machine_is_sg720()	(0)
#endif

#ifdef CONFIG_MACH_REDFOX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_REDFOX
# endif
# define machine_is_redfox()	(machine_arch_type == MACH_TYPE_REDFOX)
#else
# define machine_is_redfox()	(0)
#endif

#ifdef CONFIG_MACH_MYSH_EP9315_1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MYSH_EP9315_1
# endif
# define machine_is_mysh_ep9315_1()	(machine_arch_type == MACH_TYPE_MYSH_EP9315_1)
#else
# define machine_is_mysh_ep9315_1()	(0)
#endif

#ifdef CONFIG_MACH_TPF106
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TPF106
# endif
# define machine_is_tpf106()	(machine_arch_type == MACH_TYPE_TPF106)
#else
# define machine_is_tpf106()	(0)
#endif

#ifdef CONFIG_MACH_AT91RM9200KG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91RM9200KG
# endif
# define machine_is_at91rm9200kg()	(machine_arch_type == MACH_TYPE_AT91RM9200KG)
#else
# define machine_is_at91rm9200kg()	(0)
#endif

#ifdef CONFIG_MACH_SLEDB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SLEDB
# endif
# define machine_is_rcmt2()	(machine_arch_type == MACH_TYPE_SLEDB)
#else
# define machine_is_rcmt2()	(0)
#endif

#ifdef CONFIG_MACH_ONTRACK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ONTRACK
# endif
# define machine_is_ontrack()	(machine_arch_type == MACH_TYPE_ONTRACK)
#else
# define machine_is_ontrack()	(0)
#endif

#ifdef CONFIG_MACH_PM1200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PM1200
# endif
# define machine_is_pm1200()	(machine_arch_type == MACH_TYPE_PM1200)
#else
# define machine_is_pm1200()	(0)
#endif

#ifdef CONFIG_MACH_ESS24XXX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESS24XXX
# endif
# define machine_is_ess24562()	(machine_arch_type == MACH_TYPE_ESS24XXX)
#else
# define machine_is_ess24562()	(0)
#endif

#ifdef CONFIG_MACH_COREMP7
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COREMP7
# endif
# define machine_is_coremp7()	(machine_arch_type == MACH_TYPE_COREMP7)
#else
# define machine_is_coremp7()	(0)
#endif

#ifdef CONFIG_MACH_NEXCODER_6446
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NEXCODER_6446
# endif
# define machine_is_nexcoder_6446()	(machine_arch_type == MACH_TYPE_NEXCODER_6446)
#else
# define machine_is_nexcoder_6446()	(0)
#endif

#ifdef CONFIG_MACH_STVC8380
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STVC8380
# endif
# define machine_is_stvc8380()	(machine_arch_type == MACH_TYPE_STVC8380)
#else
# define machine_is_stvc8380()	(0)
#endif

#ifdef CONFIG_MACH_TEKLYNX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TEKLYNX
# endif
# define machine_is_teklynx()	(machine_arch_type == MACH_TYPE_TEKLYNX)
#else
# define machine_is_teklynx()	(0)
#endif

#ifdef CONFIG_MACH_CARBONADO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CARBONADO
# endif
# define machine_is_carbonado()	(machine_arch_type == MACH_TYPE_CARBONADO)
#else
# define machine_is_carbonado()	(0)
#endif

#ifdef CONFIG_MACH_SYSMOS_MP730
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SYSMOS_MP730
# endif
# define machine_is_sysmos_mp730()	(machine_arch_type == MACH_TYPE_SYSMOS_MP730)
#else
# define machine_is_sysmos_mp730()	(0)
#endif

#ifdef CONFIG_MACH_SNAPPER_CL15
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SNAPPER_CL15
# endif
# define machine_is_snapper_cl15()	(machine_arch_type == MACH_TYPE_SNAPPER_CL15)
#else
# define machine_is_snapper_cl15()	(0)
#endif

#ifdef CONFIG_MACH_PGIGIM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PGIGIM
# endif
# define machine_is_pgigim()	(machine_arch_type == MACH_TYPE_PGIGIM)
#else
# define machine_is_pgigim()	(0)
#endif

#ifdef CONFIG_MACH_PTX9160P2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PTX9160P2
# endif
# define machine_is_ptx9160p2()	(machine_arch_type == MACH_TYPE_PTX9160P2)
#else
# define machine_is_ptx9160p2()	(0)
#endif

#ifdef CONFIG_MACH_DCORE1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DCORE1
# endif
# define machine_is_dcore1()	(machine_arch_type == MACH_TYPE_DCORE1)
#else
# define machine_is_dcore1()	(0)
#endif

#ifdef CONFIG_MACH_VICTORPXA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VICTORPXA
# endif
# define machine_is_victorpxa()	(machine_arch_type == MACH_TYPE_VICTORPXA)
#else
# define machine_is_victorpxa()	(0)
#endif

#ifdef CONFIG_MACH_MX2DTB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX2DTB
# endif
# define machine_is_mx2dtb()	(machine_arch_type == MACH_TYPE_MX2DTB)
#else
# define machine_is_mx2dtb()	(0)
#endif

#ifdef CONFIG_MACH_PXA_IREX_ER0100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_IREX_ER0100
# endif
# define machine_is_pxa_irex_er0100()	(machine_arch_type == MACH_TYPE_PXA_IREX_ER0100)
#else
# define machine_is_pxa_irex_er0100()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_PALMZ71
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_PALMZ71
# endif
# define machine_is_omap_palmz71()	(machine_arch_type == MACH_TYPE_OMAP_PALMZ71)
#else
# define machine_is_omap_palmz71()	(0)
#endif

#ifdef CONFIG_MACH_BARTEC_DEG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BARTEC_DEG
# endif
# define machine_is_bartec_deg()	(machine_arch_type == MACH_TYPE_BARTEC_DEG)
#else
# define machine_is_bartec_deg()	(0)
#endif

#ifdef CONFIG_MACH_HW50251
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HW50251
# endif
# define machine_is_hw50251()	(machine_arch_type == MACH_TYPE_HW50251)
#else
# define machine_is_hw50251()	(0)
#endif

#ifdef CONFIG_MACH_IBOX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IBOX
# endif
# define machine_is_ibox()	(machine_arch_type == MACH_TYPE_IBOX)
#else
# define machine_is_ibox()	(0)
#endif

#ifdef CONFIG_MACH_ATLASLH7A404
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ATLASLH7A404
# endif
# define machine_is_atlaslh7a404()	(machine_arch_type == MACH_TYPE_ATLASLH7A404)
#else
# define machine_is_atlaslh7a404()	(0)
#endif

#ifdef CONFIG_MACH_PT2026
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PT2026
# endif
# define machine_is_pt2026()	(machine_arch_type == MACH_TYPE_PT2026)
#else
# define machine_is_pt2026()	(0)
#endif

#ifdef CONFIG_MACH_HTCALPINE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HTCALPINE
# endif
# define machine_is_htcalpine()	(machine_arch_type == MACH_TYPE_HTCALPINE)
#else
# define machine_is_htcalpine()	(0)
#endif

#ifdef CONFIG_MACH_BARTEC_VTU
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BARTEC_VTU
# endif
# define machine_is_bartec_vtu()	(machine_arch_type == MACH_TYPE_BARTEC_VTU)
#else
# define machine_is_bartec_vtu()	(0)
#endif

#ifdef CONFIG_MACH_VCOREII
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VCOREII
# endif
# define machine_is_vcoreii()	(machine_arch_type == MACH_TYPE_VCOREII)
#else
# define machine_is_vcoreii()	(0)
#endif

#ifdef CONFIG_MACH_PDNB3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PDNB3
# endif
# define machine_is_pdnb3()	(machine_arch_type == MACH_TYPE_PDNB3)
#else
# define machine_is_pdnb3()	(0)
#endif

#ifdef CONFIG_MACH_HTCBEETLES
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HTCBEETLES
# endif
# define machine_is_htcbeetles()	(machine_arch_type == MACH_TYPE_HTCBEETLES)
#else
# define machine_is_htcbeetles()	(0)
#endif

#ifdef CONFIG_MACH_S3C6400
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S3C6400
# endif
# define machine_is_s3c6400()	(machine_arch_type == MACH_TYPE_S3C6400)
#else
# define machine_is_s3c6400()	(0)
#endif

#ifdef CONFIG_MACH_S3C2443
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S3C2443
# endif
# define machine_is_s3c2443()	(machine_arch_type == MACH_TYPE_S3C2443)
#else
# define machine_is_s3c2443()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_LDK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_LDK
# endif
# define machine_is_omap_ldk()	(machine_arch_type == MACH_TYPE_OMAP_LDK)
#else
# define machine_is_omap_ldk()	(0)
#endif

#ifdef CONFIG_MACH_SMDK2460
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK2460
# endif
# define machine_is_smdk2460()	(machine_arch_type == MACH_TYPE_SMDK2460)
#else
# define machine_is_smdk2460()	(0)
#endif

#ifdef CONFIG_MACH_SMDK2440
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK2440
# endif
# define machine_is_smdk2440()	(machine_arch_type == MACH_TYPE_SMDK2440)
#else
# define machine_is_smdk2440()	(0)
#endif

#ifdef CONFIG_MACH_SMDK2412
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK2412
# endif
# define machine_is_smdk2412()	(machine_arch_type == MACH_TYPE_SMDK2412)
#else
# define machine_is_smdk2412()	(0)
#endif

#ifdef CONFIG_MACH_WEBBOX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WEBBOX
# endif
# define machine_is_webbox()	(machine_arch_type == MACH_TYPE_WEBBOX)
#else
# define machine_is_webbox()	(0)
#endif

#ifdef CONFIG_MACH_CWWNDP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CWWNDP
# endif
# define machine_is_cwwndp()	(machine_arch_type == MACH_TYPE_CWWNDP)
#else
# define machine_is_cwwndp()	(0)
#endif

#ifdef CONFIG_MACH_DRAGON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DRAGON
# endif
# define machine_is_i839()	(machine_arch_type == MACH_TYPE_DRAGON)
#else
# define machine_is_i839()	(0)
#endif

#ifdef CONFIG_MACH_OPENDO_CPU_BOARD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OPENDO_CPU_BOARD
# endif
# define machine_is_opendo_cpu_board()	(machine_arch_type == MACH_TYPE_OPENDO_CPU_BOARD)
#else
# define machine_is_opendo_cpu_board()	(0)
#endif

#ifdef CONFIG_MACH_CCM2200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CCM2200
# endif
# define machine_is_ccm2200()	(machine_arch_type == MACH_TYPE_CCM2200)
#else
# define machine_is_ccm2200()	(0)
#endif

#ifdef CONFIG_MACH_ETWARM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ETWARM
# endif
# define machine_is_etwarm()	(machine_arch_type == MACH_TYPE_ETWARM)
#else
# define machine_is_etwarm()	(0)
#endif

#ifdef CONFIG_MACH_M93030
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_M93030
# endif
# define machine_is_m93030()	(machine_arch_type == MACH_TYPE_M93030)
#else
# define machine_is_m93030()	(0)
#endif

#ifdef CONFIG_MACH_CC7U
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CC7U
# endif
# define machine_is_cc7u()	(machine_arch_type == MACH_TYPE_CC7U)
#else
# define machine_is_cc7u()	(0)
#endif

#ifdef CONFIG_MACH_MTT_RANGER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MTT_RANGER
# endif
# define machine_is_mtt_ranger()	(machine_arch_type == MACH_TYPE_MTT_RANGER)
#else
# define machine_is_mtt_ranger()	(0)
#endif

#ifdef CONFIG_MACH_NEXUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NEXUS
# endif
# define machine_is_nexus()	(machine_arch_type == MACH_TYPE_NEXUS)
#else
# define machine_is_nexus()	(0)
#endif

#ifdef CONFIG_MACH_DESMAN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DESMAN
# endif
# define machine_is_desman()	(machine_arch_type == MACH_TYPE_DESMAN)
#else
# define machine_is_desman()	(0)
#endif

#ifdef CONFIG_MACH_BKDE303
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BKDE303
# endif
# define machine_is_bkde303()	(machine_arch_type == MACH_TYPE_BKDE303)
#else
# define machine_is_bkde303()	(0)
#endif

#ifdef CONFIG_MACH_SMDK2413
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK2413
# endif
# define machine_is_smdk2413()	(machine_arch_type == MACH_TYPE_SMDK2413)
#else
# define machine_is_smdk2413()	(0)
#endif

#ifdef CONFIG_MACH_AML_M7200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AML_M7200
# endif
# define machine_is_aml_m7200()	(machine_arch_type == MACH_TYPE_AML_M7200)
#else
# define machine_is_aml_m7200()	(0)
#endif

#ifdef CONFIG_MACH_AML_M5900
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AML_M5900
# endif
# define machine_is_aml_m5900()	(machine_arch_type == MACH_TYPE_AML_M5900)
#else
# define machine_is_aml_m5900()	(0)
#endif

#ifdef CONFIG_MACH_SG640
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SG640
# endif
# define machine_is_sg640()	(machine_arch_type == MACH_TYPE_SG640)
#else
# define machine_is_sg640()	(0)
#endif

#ifdef CONFIG_MACH_EDG79524
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDG79524
# endif
# define machine_is_edg79524()	(machine_arch_type == MACH_TYPE_EDG79524)
#else
# define machine_is_edg79524()	(0)
#endif

#ifdef CONFIG_MACH_AI2410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AI2410
# endif
# define machine_is_ai2410()	(machine_arch_type == MACH_TYPE_AI2410)
#else
# define machine_is_ai2410()	(0)
#endif

#ifdef CONFIG_MACH_IXP465
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXP465
# endif
# define machine_is_ixp465()	(machine_arch_type == MACH_TYPE_IXP465)
#else
# define machine_is_ixp465()	(0)
#endif

#ifdef CONFIG_MACH_BALLOON3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BALLOON3
# endif
# define machine_is_balloon3()	(machine_arch_type == MACH_TYPE_BALLOON3)
#else
# define machine_is_balloon3()	(0)
#endif

#ifdef CONFIG_MACH_HEINS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HEINS
# endif
# define machine_is_heins()	(machine_arch_type == MACH_TYPE_HEINS)
#else
# define machine_is_heins()	(0)
#endif

#ifdef CONFIG_MACH_MPLUSEVA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MPLUSEVA
# endif
# define machine_is_mpluseva()	(machine_arch_type == MACH_TYPE_MPLUSEVA)
#else
# define machine_is_mpluseva()	(0)
#endif

#ifdef CONFIG_MACH_RT042
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RT042
# endif
# define machine_is_rt042()	(machine_arch_type == MACH_TYPE_RT042)
#else
# define machine_is_rt042()	(0)
#endif

#ifdef CONFIG_MACH_CWIEM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CWIEM
# endif
# define machine_is_cwiem()	(machine_arch_type == MACH_TYPE_CWIEM)
#else
# define machine_is_cwiem()	(0)
#endif

#ifdef CONFIG_MACH_CM_X270
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CM_X270
# endif
# define machine_is_cm_x270()	(machine_arch_type == MACH_TYPE_CM_X270)
#else
# define machine_is_cm_x270()	(0)
#endif

#ifdef CONFIG_MACH_CM_X255
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CM_X255
# endif
# define machine_is_cm_x255()	(machine_arch_type == MACH_TYPE_CM_X255)
#else
# define machine_is_cm_x255()	(0)
#endif

#ifdef CONFIG_MACH_ESH_AT91
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESH_AT91
# endif
# define machine_is_esh_at91()	(machine_arch_type == MACH_TYPE_ESH_AT91)
#else
# define machine_is_esh_at91()	(0)
#endif

#ifdef CONFIG_MACH_SANDGATE3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SANDGATE3
# endif
# define machine_is_sandgate3()	(machine_arch_type == MACH_TYPE_SANDGATE3)
#else
# define machine_is_sandgate3()	(0)
#endif

#ifdef CONFIG_MACH_PRIMO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PRIMO
# endif
# define machine_is_primo()	(machine_arch_type == MACH_TYPE_PRIMO)
#else
# define machine_is_primo()	(0)
#endif

#ifdef CONFIG_MACH_GEMSTONE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GEMSTONE
# endif
# define machine_is_gemstone()	(machine_arch_type == MACH_TYPE_GEMSTONE)
#else
# define machine_is_gemstone()	(0)
#endif

#ifdef CONFIG_MACH_PRONGHORNMETRO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PRONGHORNMETRO
# endif
# define machine_is_pronghorn_metro()	(machine_arch_type == MACH_TYPE_PRONGHORNMETRO)
#else
# define machine_is_pronghorn_metro()	(0)
#endif

#ifdef CONFIG_MACH_SIDEWINDER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SIDEWINDER
# endif
# define machine_is_sidewinder()	(machine_arch_type == MACH_TYPE_SIDEWINDER)
#else
# define machine_is_sidewinder()	(0)
#endif

#ifdef CONFIG_MACH_PICOMOD1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PICOMOD1
# endif
# define machine_is_picomod1()	(machine_arch_type == MACH_TYPE_PICOMOD1)
#else
# define machine_is_picomod1()	(0)
#endif

#ifdef CONFIG_MACH_SG590
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SG590
# endif
# define machine_is_sg590()	(machine_arch_type == MACH_TYPE_SG590)
#else
# define machine_is_sg590()	(0)
#endif

#ifdef CONFIG_MACH_AKAI9307
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AKAI9307
# endif
# define machine_is_akai9307()	(machine_arch_type == MACH_TYPE_AKAI9307)
#else
# define machine_is_akai9307()	(0)
#endif

#ifdef CONFIG_MACH_FONTAINE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FONTAINE
# endif
# define machine_is_fontaine()	(machine_arch_type == MACH_TYPE_FONTAINE)
#else
# define machine_is_fontaine()	(0)
#endif

#ifdef CONFIG_MACH_WOMBAT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WOMBAT
# endif
# define machine_is_wombat()	(machine_arch_type == MACH_TYPE_WOMBAT)
#else
# define machine_is_wombat()	(0)
#endif

#ifdef CONFIG_MACH_ACQ300
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACQ300
# endif
# define machine_is_acq300()	(machine_arch_type == MACH_TYPE_ACQ300)
#else
# define machine_is_acq300()	(0)
#endif

#ifdef CONFIG_MACH_MOD_270
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MOD_270
# endif
# define machine_is_mod272()	(machine_arch_type == MACH_TYPE_MOD_270)
#else
# define machine_is_mod272()	(0)
#endif

#ifdef CONFIG_MACH_VC0820
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VC0820
# endif
# define machine_is_vmc_vc0820()	(machine_arch_type == MACH_TYPE_VC0820)
#else
# define machine_is_vmc_vc0820()	(0)
#endif

#ifdef CONFIG_MACH_ANI_AIM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ANI_AIM
# endif
# define machine_is_ani_aim()	(machine_arch_type == MACH_TYPE_ANI_AIM)
#else
# define machine_is_ani_aim()	(0)
#endif

#ifdef CONFIG_MACH_JELLYFISH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_JELLYFISH
# endif
# define machine_is_jellyfish()	(machine_arch_type == MACH_TYPE_JELLYFISH)
#else
# define machine_is_jellyfish()	(0)
#endif

#ifdef CONFIG_MACH_AMANITA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AMANITA
# endif
# define machine_is_amanita()	(machine_arch_type == MACH_TYPE_AMANITA)
#else
# define machine_is_amanita()	(0)
#endif

#ifdef CONFIG_MACH_VLINK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VLINK
# endif
# define machine_is_vlink()	(machine_arch_type == MACH_TYPE_VLINK)
#else
# define machine_is_vlink()	(0)
#endif

#ifdef CONFIG_MACH_DEXFLEX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DEXFLEX
# endif
# define machine_is_dexflex()	(machine_arch_type == MACH_TYPE_DEXFLEX)
#else
# define machine_is_dexflex()	(0)
#endif

#ifdef CONFIG_MACH_EIGEN_TTQ
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EIGEN_TTQ
# endif
# define machine_is_eigen_ttq()	(machine_arch_type == MACH_TYPE_EIGEN_TTQ)
#else
# define machine_is_eigen_ttq()	(0)
#endif

#ifdef CONFIG_MACH_ARCOM_TITAN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARCOM_TITAN
# endif
# define machine_is_arcom_titan()	(machine_arch_type == MACH_TYPE_ARCOM_TITAN)
#else
# define machine_is_arcom_titan()	(0)
#endif

#ifdef CONFIG_MACH_TABLA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TABLA
# endif
# define machine_is_tabla()	(machine_arch_type == MACH_TYPE_TABLA)
#else
# define machine_is_tabla()	(0)
#endif

#ifdef CONFIG_MACH_MDIRAC3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MDIRAC3
# endif
# define machine_is_mdirac3()	(machine_arch_type == MACH_TYPE_MDIRAC3)
#else
# define machine_is_mdirac3()	(0)
#endif

#ifdef CONFIG_MACH_MRHFBP2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MRHFBP2
# endif
# define machine_is_mrhfbp2()	(machine_arch_type == MACH_TYPE_MRHFBP2)
#else
# define machine_is_mrhfbp2()	(0)
#endif

#ifdef CONFIG_MACH_AT91RM9200RB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91RM9200RB
# endif
# define machine_is_at91rm9200rb()	(machine_arch_type == MACH_TYPE_AT91RM9200RB)
#else
# define machine_is_at91rm9200rb()	(0)
#endif

#ifdef CONFIG_MACH_ANI_APM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ANI_APM
# endif
# define machine_is_ani_apm()	(machine_arch_type == MACH_TYPE_ANI_APM)
#else
# define machine_is_ani_apm()	(0)
#endif

#ifdef CONFIG_MACH_ELLA1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ELLA1
# endif
# define machine_is_ella1()	(machine_arch_type == MACH_TYPE_ELLA1)
#else
# define machine_is_ella1()	(0)
#endif

#ifdef CONFIG_MACH_INHAND_PXA27X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_INHAND_PXA27X
# endif
# define machine_is_inhand_pxa27x()	(machine_arch_type == MACH_TYPE_INHAND_PXA27X)
#else
# define machine_is_inhand_pxa27x()	(0)
#endif

#ifdef CONFIG_MACH_INHAND_PXA25X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_INHAND_PXA25X
# endif
# define machine_is_inhand_pxa25x()	(machine_arch_type == MACH_TYPE_INHAND_PXA25X)
#else
# define machine_is_inhand_pxa25x()	(0)
#endif

#ifdef CONFIG_MACH_EMPOS_XM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EMPOS_XM
# endif
# define machine_is_empos_xm()	(machine_arch_type == MACH_TYPE_EMPOS_XM)
#else
# define machine_is_empos_xm()	(0)
#endif

#ifdef CONFIG_MACH_EMPOS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EMPOS
# endif
# define machine_is_empos()	(machine_arch_type == MACH_TYPE_EMPOS)
#else
# define machine_is_empos()	(0)
#endif

#ifdef CONFIG_MACH_EMPOS_TINY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EMPOS_TINY
# endif
# define machine_is_empos_tiny()	(machine_arch_type == MACH_TYPE_EMPOS_TINY)
#else
# define machine_is_empos_tiny()	(0)
#endif

#ifdef CONFIG_MACH_EMPOS_SM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EMPOS_SM
# endif
# define machine_is_empos_sm()	(machine_arch_type == MACH_TYPE_EMPOS_SM)
#else
# define machine_is_empos_sm()	(0)
#endif

#ifdef CONFIG_MACH_EGRET
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EGRET
# endif
# define machine_is_egret()	(machine_arch_type == MACH_TYPE_EGRET)
#else
# define machine_is_egret()	(0)
#endif

#ifdef CONFIG_MACH_OSTRICH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OSTRICH
# endif
# define machine_is_ostrich()	(machine_arch_type == MACH_TYPE_OSTRICH)
#else
# define machine_is_ostrich()	(0)
#endif

#ifdef CONFIG_MACH_N50
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_N50
# endif
# define machine_is_n50()	(machine_arch_type == MACH_TYPE_N50)
#else
# define machine_is_n50()	(0)
#endif

#ifdef CONFIG_MACH_ECBAT91
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ECBAT91
# endif
# define machine_is_ecbat91()	(machine_arch_type == MACH_TYPE_ECBAT91)
#else
# define machine_is_ecbat91()	(0)
#endif

#ifdef CONFIG_MACH_STAREAST
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STAREAST
# endif
# define machine_is_stareast()	(machine_arch_type == MACH_TYPE_STAREAST)
#else
# define machine_is_stareast()	(0)
#endif

#ifdef CONFIG_MACH_DSPG_DW
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DSPG_DW
# endif
# define machine_is_dspg_dw()	(machine_arch_type == MACH_TYPE_DSPG_DW)
#else
# define machine_is_dspg_dw()	(0)
#endif

#ifdef CONFIG_MACH_ONEARM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ONEARM
# endif
# define machine_is_onearm()	(machine_arch_type == MACH_TYPE_ONEARM)
#else
# define machine_is_onearm()	(0)
#endif

#ifdef CONFIG_MACH_MRG110_6
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MRG110_6
# endif
# define machine_is_mrg110_6()	(machine_arch_type == MACH_TYPE_MRG110_6)
#else
# define machine_is_mrg110_6()	(0)
#endif

#ifdef CONFIG_MACH_WRT300NV2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WRT300NV2
# endif
# define machine_is_wrt300nv2()	(machine_arch_type == MACH_TYPE_WRT300NV2)
#else
# define machine_is_wrt300nv2()	(0)
#endif

#ifdef CONFIG_MACH_XM_BULVERDE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XM_BULVERDE
# endif
# define machine_is_xm_bulverde()	(machine_arch_type == MACH_TYPE_XM_BULVERDE)
#else
# define machine_is_xm_bulverde()	(0)
#endif

#ifdef CONFIG_MACH_MSM6100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM6100
# endif
# define machine_is_msm6100()	(machine_arch_type == MACH_TYPE_MSM6100)
#else
# define machine_is_msm6100()	(0)
#endif

#ifdef CONFIG_MACH_ETI_B1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ETI_B1
# endif
# define machine_is_eti_b1()	(machine_arch_type == MACH_TYPE_ETI_B1)
#else
# define machine_is_eti_b1()	(0)
#endif

#ifdef CONFIG_MACH_ZILOG_ZA9L
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ZILOG_ZA9L
# endif
# define machine_is_za9l_series()	(machine_arch_type == MACH_TYPE_ZILOG_ZA9L)
#else
# define machine_is_za9l_series()	(0)
#endif

#ifdef CONFIG_MACH_BIT2440
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BIT2440
# endif
# define machine_is_bit2440()	(machine_arch_type == MACH_TYPE_BIT2440)
#else
# define machine_is_bit2440()	(0)
#endif

#ifdef CONFIG_MACH_NBI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NBI
# endif
# define machine_is_nbi()	(machine_arch_type == MACH_TYPE_NBI)
#else
# define machine_is_nbi()	(0)
#endif

#ifdef CONFIG_MACH_SMDK2443
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK2443
# endif
# define machine_is_smdk2443()	(machine_arch_type == MACH_TYPE_SMDK2443)
#else
# define machine_is_smdk2443()	(0)
#endif

#ifdef CONFIG_MACH_VDAVINCI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VDAVINCI
# endif
# define machine_is_vdavinci()	(machine_arch_type == MACH_TYPE_VDAVINCI)
#else
# define machine_is_vdavinci()	(0)
#endif

#ifdef CONFIG_MACH_ATC6
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ATC6
# endif
# define machine_is_atc6()	(machine_arch_type == MACH_TYPE_ATC6)
#else
# define machine_is_atc6()	(0)
#endif

#ifdef CONFIG_MACH_MULTMDW
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MULTMDW
# endif
# define machine_is_multmdw()	(machine_arch_type == MACH_TYPE_MULTMDW)
#else
# define machine_is_multmdw()	(0)
#endif

#ifdef CONFIG_MACH_MBA2440
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MBA2440
# endif
# define machine_is_mba2440()	(machine_arch_type == MACH_TYPE_MBA2440)
#else
# define machine_is_mba2440()	(0)
#endif

#ifdef CONFIG_MACH_ECSD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ECSD
# endif
# define machine_is_ecsd()	(machine_arch_type == MACH_TYPE_ECSD)
#else
# define machine_is_ecsd()	(0)
#endif

#ifdef CONFIG_MACH_ZIRE31
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ZIRE31
# endif
# define machine_is_zire31()	(machine_arch_type == MACH_TYPE_ZIRE31)
#else
# define machine_is_zire31()	(0)
#endif

#ifdef CONFIG_MACH_FSG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FSG
# endif
# define machine_is_fsg()	(machine_arch_type == MACH_TYPE_FSG)
#else
# define machine_is_fsg()	(0)
#endif

#ifdef CONFIG_MACH_RAZOR101
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RAZOR101
# endif
# define machine_is_razor101()	(machine_arch_type == MACH_TYPE_RAZOR101)
#else
# define machine_is_razor101()	(0)
#endif

#ifdef CONFIG_MACH_OPERA_TDM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OPERA_TDM
# endif
# define machine_is_opera_tdm()	(machine_arch_type == MACH_TYPE_OPERA_TDM)
#else
# define machine_is_opera_tdm()	(0)
#endif

#ifdef CONFIG_MACH_COMCERTO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COMCERTO
# endif
# define machine_is_comcerto()	(machine_arch_type == MACH_TYPE_COMCERTO)
#else
# define machine_is_comcerto()	(0)
#endif

#ifdef CONFIG_MACH_TB0319
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TB0319
# endif
# define machine_is_tb0319()	(machine_arch_type == MACH_TYPE_TB0319)
#else
# define machine_is_tb0319()	(0)
#endif

#ifdef CONFIG_MACH_KWS8000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KWS8000
# endif
# define machine_is_kws8000()	(machine_arch_type == MACH_TYPE_KWS8000)
#else
# define machine_is_kws8000()	(0)
#endif

#ifdef CONFIG_MACH_B2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_B2
# endif
# define machine_is_b2()	(machine_arch_type == MACH_TYPE_B2)
#else
# define machine_is_b2()	(0)
#endif

#ifdef CONFIG_MACH_LCL54
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LCL54
# endif
# define machine_is_lcl54()	(machine_arch_type == MACH_TYPE_LCL54)
#else
# define machine_is_lcl54()	(0)
#endif

#ifdef CONFIG_MACH_AT91SAM9260EK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91SAM9260EK
# endif
# define machine_is_at91sam9260ek()	(machine_arch_type == MACH_TYPE_AT91SAM9260EK)
#else
# define machine_is_at91sam9260ek()	(0)
#endif

#ifdef CONFIG_MACH_GLANTANK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GLANTANK
# endif
# define machine_is_glantank()	(machine_arch_type == MACH_TYPE_GLANTANK)
#else
# define machine_is_glantank()	(0)
#endif

#ifdef CONFIG_MACH_N2100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_N2100
# endif
# define machine_is_n2100()	(machine_arch_type == MACH_TYPE_N2100)
#else
# define machine_is_n2100()	(0)
#endif

#ifdef CONFIG_MACH_N4100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_N4100
# endif
# define machine_is_n4100()	(machine_arch_type == MACH_TYPE_N4100)
#else
# define machine_is_n4100()	(0)
#endif

#ifdef CONFIG_MACH_VERTICAL_RSC4
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VERTICAL_RSC4
# endif
# define machine_is_rsc4()	(machine_arch_type == MACH_TYPE_VERTICAL_RSC4)
#else
# define machine_is_rsc4()	(0)
#endif

#ifdef CONFIG_MACH_SG8100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SG8100
# endif
# define machine_is_sg8100()	(machine_arch_type == MACH_TYPE_SG8100)
#else
# define machine_is_sg8100()	(0)
#endif

#ifdef CONFIG_MACH_IM42XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IM42XX
# endif
# define machine_is_im42xx()	(machine_arch_type == MACH_TYPE_IM42XX)
#else
# define machine_is_im42xx()	(0)
#endif

#ifdef CONFIG_MACH_FTXX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FTXX
# endif
# define machine_is_ftxx()	(machine_arch_type == MACH_TYPE_FTXX)
#else
# define machine_is_ftxx()	(0)
#endif

#ifdef CONFIG_MACH_LWFUSION
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LWFUSION
# endif
# define machine_is_lwfusion()	(machine_arch_type == MACH_TYPE_LWFUSION)
#else
# define machine_is_lwfusion()	(0)
#endif

#ifdef CONFIG_MACH_QT2410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_QT2410
# endif
# define machine_is_qt2410()	(machine_arch_type == MACH_TYPE_QT2410)
#else
# define machine_is_qt2410()	(0)
#endif

#ifdef CONFIG_MACH_KIXRP435
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KIXRP435
# endif
# define machine_is_kixrp435()	(machine_arch_type == MACH_TYPE_KIXRP435)
#else
# define machine_is_kixrp435()	(0)
#endif

#ifdef CONFIG_MACH_CCW9C
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CCW9C
# endif
# define machine_is_ccw9c()	(machine_arch_type == MACH_TYPE_CCW9C)
#else
# define machine_is_ccw9c()	(0)
#endif

#ifdef CONFIG_MACH_DABHS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DABHS
# endif
# define machine_is_dabhs()	(machine_arch_type == MACH_TYPE_DABHS)
#else
# define machine_is_dabhs()	(0)
#endif

#ifdef CONFIG_MACH_GZMX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GZMX
# endif
# define machine_is_gzmx()	(machine_arch_type == MACH_TYPE_GZMX)
#else
# define machine_is_gzmx()	(0)
#endif

#ifdef CONFIG_MACH_IPNW100AP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IPNW100AP
# endif
# define machine_is_ipnw100ap()	(machine_arch_type == MACH_TYPE_IPNW100AP)
#else
# define machine_is_ipnw100ap()	(0)
#endif

#ifdef CONFIG_MACH_CC9P9360DEV
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CC9P9360DEV
# endif
# define machine_is_cc9p9360dev()	(machine_arch_type == MACH_TYPE_CC9P9360DEV)
#else
# define machine_is_cc9p9360dev()	(0)
#endif

#ifdef CONFIG_MACH_CC9P9750DEV
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CC9P9750DEV
# endif
# define machine_is_cc9p9750dev()	(machine_arch_type == MACH_TYPE_CC9P9750DEV)
#else
# define machine_is_cc9p9750dev()	(0)
#endif

#ifdef CONFIG_MACH_CC9P9360VAL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CC9P9360VAL
# endif
# define machine_is_cc9p9360val()	(machine_arch_type == MACH_TYPE_CC9P9360VAL)
#else
# define machine_is_cc9p9360val()	(0)
#endif

#ifdef CONFIG_MACH_CC9P9750VAL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CC9P9750VAL
# endif
# define machine_is_cc9p9750val()	(machine_arch_type == MACH_TYPE_CC9P9750VAL)
#else
# define machine_is_cc9p9750val()	(0)
#endif

#ifdef CONFIG_MACH_NX70V
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NX70V
# endif
# define machine_is_nx70v()	(machine_arch_type == MACH_TYPE_NX70V)
#else
# define machine_is_nx70v()	(0)
#endif

#ifdef CONFIG_MACH_AT91RM9200DF
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91RM9200DF
# endif
# define machine_is_at91rm9200df()	(machine_arch_type == MACH_TYPE_AT91RM9200DF)
#else
# define machine_is_at91rm9200df()	(0)
#endif

#ifdef CONFIG_MACH_SE_PILOT2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SE_PILOT2
# endif
# define machine_is_se_pilot2()	(machine_arch_type == MACH_TYPE_SE_PILOT2)
#else
# define machine_is_se_pilot2()	(0)
#endif

#ifdef CONFIG_MACH_MTCN_T800
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MTCN_T800
# endif
# define machine_is_mtcn_t800()	(machine_arch_type == MACH_TYPE_MTCN_T800)
#else
# define machine_is_mtcn_t800()	(0)
#endif

#ifdef CONFIG_MACH_VCMX212
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VCMX212
# endif
# define machine_is_vcmx212()	(machine_arch_type == MACH_TYPE_VCMX212)
#else
# define machine_is_vcmx212()	(0)
#endif

#ifdef CONFIG_MACH_LYNX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LYNX
# endif
# define machine_is_lynx()	(machine_arch_type == MACH_TYPE_LYNX)
#else
# define machine_is_lynx()	(0)
#endif

#ifdef CONFIG_MACH_AT91SAM9260ID
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91SAM9260ID
# endif
# define machine_is_at91sam9260id()	(machine_arch_type == MACH_TYPE_AT91SAM9260ID)
#else
# define machine_is_at91sam9260id()	(0)
#endif

#ifdef CONFIG_MACH_HW86052
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HW86052
# endif
# define machine_is_hw86052()	(machine_arch_type == MACH_TYPE_HW86052)
#else
# define machine_is_hw86052()	(0)
#endif

#ifdef CONFIG_MACH_PILZ_PMI3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PILZ_PMI3
# endif
# define machine_is_pilz_pmi3()	(machine_arch_type == MACH_TYPE_PILZ_PMI3)
#else
# define machine_is_pilz_pmi3()	(0)
#endif

#ifdef CONFIG_MACH_EDB9302A
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB9302A
# endif
# define machine_is_edb9302a()	(machine_arch_type == MACH_TYPE_EDB9302A)
#else
# define machine_is_edb9302a()	(0)
#endif

#ifdef CONFIG_MACH_EDB9307A
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB9307A
# endif
# define machine_is_edb9307a()	(machine_arch_type == MACH_TYPE_EDB9307A)
#else
# define machine_is_edb9307a()	(0)
#endif

#ifdef CONFIG_MACH_CT_DFS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CT_DFS
# endif
# define machine_is_ct_dfs()	(machine_arch_type == MACH_TYPE_CT_DFS)
#else
# define machine_is_ct_dfs()	(0)
#endif

#ifdef CONFIG_MACH_PILZ_PMI4
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PILZ_PMI4
# endif
# define machine_is_pilz_pmi4()	(machine_arch_type == MACH_TYPE_PILZ_PMI4)
#else
# define machine_is_pilz_pmi4()	(0)
#endif

#ifdef CONFIG_MACH_XCEEDNP_IXP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XCEEDNP_IXP
# endif
# define machine_is_xceednp_ixp()	(machine_arch_type == MACH_TYPE_XCEEDNP_IXP)
#else
# define machine_is_xceednp_ixp()	(0)
#endif

#ifdef CONFIG_MACH_SMDK2442B
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK2442B
# endif
# define machine_is_smdk2442b()	(machine_arch_type == MACH_TYPE_SMDK2442B)
#else
# define machine_is_smdk2442b()	(0)
#endif

#ifdef CONFIG_MACH_XNODE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XNODE
# endif
# define machine_is_xnode()	(machine_arch_type == MACH_TYPE_XNODE)
#else
# define machine_is_xnode()	(0)
#endif

#ifdef CONFIG_MACH_AIDX270
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AIDX270
# endif
# define machine_is_aidx270()	(machine_arch_type == MACH_TYPE_AIDX270)
#else
# define machine_is_aidx270()	(0)
#endif

#ifdef CONFIG_MACH_REMA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_REMA
# endif
# define machine_is_rema()	(machine_arch_type == MACH_TYPE_REMA)
#else
# define machine_is_rema()	(0)
#endif

#ifdef CONFIG_MACH_BPS1000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BPS1000
# endif
# define machine_is_bps1000()	(machine_arch_type == MACH_TYPE_BPS1000)
#else
# define machine_is_bps1000()	(0)
#endif

#ifdef CONFIG_MACH_HW90350
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HW90350
# endif
# define machine_is_hw90350()	(machine_arch_type == MACH_TYPE_HW90350)
#else
# define machine_is_hw90350()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_SDP3430
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_SDP3430
# endif
# define machine_is_omap_sdp3430()	(machine_arch_type == MACH_TYPE_OMAP_SDP3430)
#else
# define machine_is_omap_sdp3430()	(0)
#endif

#ifdef CONFIG_MACH_BLUETOUCH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BLUETOUCH
# endif
# define machine_is_bluetouch()	(machine_arch_type == MACH_TYPE_BLUETOUCH)
#else
# define machine_is_bluetouch()	(0)
#endif

#ifdef CONFIG_MACH_VSTMS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VSTMS
# endif
# define machine_is_vstms()	(machine_arch_type == MACH_TYPE_VSTMS)
#else
# define machine_is_vstms()	(0)
#endif

#ifdef CONFIG_MACH_XSBASE270
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XSBASE270
# endif
# define machine_is_xsbase270()	(machine_arch_type == MACH_TYPE_XSBASE270)
#else
# define machine_is_xsbase270()	(0)
#endif

#ifdef CONFIG_MACH_AT91SAM9260EK_CN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91SAM9260EK_CN
# endif
# define machine_is_at91sam9260ek_cn()	(machine_arch_type == MACH_TYPE_AT91SAM9260EK_CN)
#else
# define machine_is_at91sam9260ek_cn()	(0)
#endif

#ifdef CONFIG_MACH_ADSTURBOXB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADSTURBOXB
# endif
# define machine_is_adsturboxb()	(machine_arch_type == MACH_TYPE_ADSTURBOXB)
#else
# define machine_is_adsturboxb()	(0)
#endif

#ifdef CONFIG_MACH_OTI4110
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OTI4110
# endif
# define machine_is_oti4110()	(machine_arch_type == MACH_TYPE_OTI4110)
#else
# define machine_is_oti4110()	(0)
#endif

#ifdef CONFIG_MACH_HME_PXA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HME_PXA
# endif
# define machine_is_hme_pxa()	(machine_arch_type == MACH_TYPE_HME_PXA)
#else
# define machine_is_hme_pxa()	(0)
#endif

#ifdef CONFIG_MACH_DEISTERDCA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DEISTERDCA
# endif
# define machine_is_deisterdca()	(machine_arch_type == MACH_TYPE_DEISTERDCA)
#else
# define machine_is_deisterdca()	(0)
#endif

/*
 * These have not yet been registered
 */
/* #define MACH_TYPE_367                  <<not registered>> */
#define machine_is_esl_wireless_tab()	(0)

#ifndef machine_arch_type
#define machine_arch_type	__machine_arch_type
#endif

#if defined(CONFIG_NAKED_BOOT) && (machine_arch_type == __machine_arch_type)
#error "CONFIG_NAKED_BOOT may be used with a single-machine config only"
#endif

#endif
